#ifndef _COMPOSER_H_
#define _COMPOSER_H_

#include "event/event.h"
#include "event/listeners.h"
#include "unit/unit.h"
#include "msg/msg.h"
#include "event/eventqueue_table.h"
#include "socket/socket.h"


struct listener;
struct socket;

typedef struct composer* (*FUNC_COMPOSER_FACTORY)(struct unit*);


typedef struct composer {
	struct unit* unit;

	void* (*run_composer)(void*);
	
	// composer is an evt subscriber 
	void (*func_event_received)(struct event*, struct listener* node);
	
	// composer is a msg publisher
	void (*func_publish_msg)(struct msg*, struct composer* /*socket* */);
	struct listener_list_entry* msg_listeners;
	
	// composer is an event publisher
	struct listener_list_entry* evt_listeners;
	void (*func_publish_event)(struct event*, struct parser*);
	
	FUNC_COMPOSER_FACTORY factory;
	
} COMPOSER;
typedef COMPOSER* COMPOSER_PTR;

typedef struct compose_thread_data {
	EVENTQUEUE_TABLE_ENTRY_PTR entry_event_queue;
	struct composer* composer;
} COMPOSETHREADDATA;
typedef COMPOSETHREADDATA* COMPOSETHREADDATA_PTR;


// composer is a msg publisher
int composer_add_msglistener(COMPOSER_PTR composer, struct listener* evt_list);
int composer_add_msglistener_unit(COMPOSER_PTR composer, struct unit* u);
int composer_add_msglistener_socket(COMPOSER_PTR composer, struct socket* s);
void composer_publish_msg(MSG_PTR msg, COMPOSER_PTR composer);

// composer is an evt subscriber 
void composer_event_received(EVENT_PTR evt, struct listener* node);


COMPOSER_PTR composer_create(struct unit* u);
COMPOSER_PTR slpcomposer_create(struct unit* u);
COMPOSER_PTR upnpcomposer_create(struct unit* u);
COMPOSER_PTR wsdcomposer_create(struct unit* u);
COMPOSER_PTR wsdlcomposer_create(struct unit* u);
COMPOSER_PTR ssdp_composer_create(struct unit* u);
COMPOSER_PTR upnp_devicedescr_composer_create(struct unit* u);
COMPOSER_PTR upnp_servicedescr_composer_create(struct unit* u);
COMPOSER_PTR jrmp_composer_create(struct unit* u);
COMPOSER_PTR upnpproxy_composer_create(struct unit* u);
COMPOSER_PTR rmiproxy_composer_create(struct unit* u);
COMPOSER_PTR soapcomposer_create(struct unit* u);
COMPOSER_PTR wsdproxy_composer_create(struct unit* u);

int composer_add_evtlistener(COMPOSER_PTR composer, struct listener* evt_list);
int composer_add_evtlistener_unit(COMPOSER_PTR composer, struct unit* u);
void composer_publish_event(EVENT_PTR evt, COMPOSER_PTR composer);


void *ssdp_run_composer(void* ptr);




#endif //_COMPOSER_H_
