#ifndef _EVENT_H_

#define _EVENT_H_

#include "event/servicedef.h"

typedef struct event {
	int type;
	union {
	   int value_int;
	   long long value_longlong;
	   char* value_string;
	   void* value_void_pointer;
	} value;
	int stream_id;
	void* src;
} EVENT;
typedef EVENT* EVENT_PTR;


typedef struct event_src {
	void* src_thread_parser;
	struct unit* src_unit;
} EVENTSRC;
typedef EVENTSRC* EVENTSRC_PTR;


typedef struct event_method_def {
	METHODDEFPTR method;
	char* interface_name;
	char* service_name;
} EVENT_METHOD_DEF;
typedef EVENT_METHOD_DEF* EVENT_METHOD_DEF_PTR;

enum list_event_type {
		
	SDP_REQ_LANGAGE			=	1,
	SDP_REQ_SCOPE			=	2,
	SDP_REQ_PREDICAT			=	3,
	SDP_REQ_REQUESTER		=	4,
	SDP_REQ_ID				=	5,
	SDP_REQ_DELAY			=	7,	
	SDP_REQ_MULTICAST		=	11,	
	SDP_REQ_UNICAST			=	12,	
	SDP_REQ_SERVICE_TYPE		=	97,	
	
	
	SDP_UPNP					=	91, // COMPRESS IN SDP_TYPE WITH UPNP VALUE
	SDP_JINI					=	92, // COMPRESS	IN SDP_TYPE WITH JINI VALUE
	SDP_SLP					=	93,	// COMPRESS IN SDP_TYPE WITH SLP  VALUE
	SDP_WSD					=	89,
	SCP_SOAP				=   88,
	SCP_RMI					=   87,
	
	SDP_START 				=	90, //OK	
	SDP_STOP 				=	94,	//OK
	
	SDP_VERSION				=	95,	
	SDP_ERROR				=	96,	
	SDP_SOURCE_ADDR			=	105, //@IP + port
	SDP_SOURCE_PORT	= 106,
	
/*	SDP_DEVICE_DESCRIPTION 	=	106,	//Device description in XML
 * 
 */
	/* SDP_DEVICE_URL_DESC 		= 	107,	//Url to get device description */
	
	
/*	SDP_SERVICE_URL_CTRL		=	98,		//URL to control the service */
/*	
	SDP_SERVICE_URL_DESC		=  	107,	//URL to get the description of the URL
	SDP_SERVICE_URL_EVT_CTRL	=  	108,	//URL to manage evt base state mechanism
*/	
//	SDP_SERVICE_DESCRIPTION	=	99, 	//Service description in XML
	SDP_SERVICE_ID			=	100,	
	
	SDP_SERVICE_REQUEST		=	101,	//OK
	SDP_SERVICE_RESPONSE 	= 	47,		//OK
	SDP_SERVICE_ALIVE		=	102,	//OK
	SDP_SERVICE_BYEBYE		=	103,	//OK
	
	SDP_SERVICE_LIFETIME		=	104,	
	
	SDP_RES_OK				=	41,		
	SDP_RES_ERR				=	42,		
	SDP_RES_ATTR				=	43,		
	SDP_RES_SOURCE_ADR		=	44,		
	SDP_RES_SERVICE_TYPE		=	46,		
	
	
	SDP_PARSER_SWITCH 		= 	109,	
	SDP_PARSER_SUSPEND		=	110,	
	SDP_PARSER_CONTINUE		=	111,
	
	/*
	 * 
//	SDP_REQFWD_SRC_ADDR		=	51,
 * 	 */
 
//	SDP_CONTROL_URL			= 	200,
/*	SDP_CONTROL_URL_PATH		=	201, */
/*	SDP_SERVICE_DESCRIPTION_URL_PATH	= 202, */
	
	SDP_SERVICE_DESCRIPTION = 301,
	SDP_SERVICE_METHOD_DESCRIPTION			=	300,
	SDP_SERVICE_DESCRIPTION_SERVICETYPE = 302,
	SDP_SERVICE_DESCRIPTION_INTERFACETYPE = 303,
	SDP_SERVICE_DESCRIPTION_INTERFACE_NAMESPACE = 304,
	SDP_SERVICE_VAR_DESCRIPTION				=	320,
	GENERATE_PROXY							 =	400, 
	
	SDP_SERVICEPROXY_URL_CTRL		=	401,
	
	
	SDP_SERVICE_DESCR_URL_NOPATH		=	500,
	SDP_SERVICE_DESCR_URL_PATH		=	501,
	SDP_INTERFACE_DESCR_URL_PATH		=	502,
	
	SDP_SERVICE_CTRL_URL_NOPATH		=	510,
//	SDP_SERVICE_CTRL_URL_PATH		=	511,
	SDP_INTERFACE_CTRL_URL_PATH		=	512,

	/* 
	 * Added to create RMI method call 
	 * from UPnP Proxy
	 */
	SCP_RMIOBJ_NUM		=	600,
	SCP_RMIOBJ_UNIQUE	=	601,
	SCP_RMIOBJ_TIME		=	602,
	SCP_RMIOBJ_COUNT		=	603,
	SCP_RMIOBJ_IP		=	604,
	SCP_RMIOBJ_PORT		=	605,
	SCP_RMI_STUB_OBJ	=	606,
	
	 SDP_SWITCH_COMPOSER	 = 1000,
	 SDP_SEND_SERVICE_REQUEST		=	1001,
	 SDP_SEND_SERVICE_REPLY		=	1002,
	 
	 SCP_REQUEST			= 704,
	 SCP_RESPONSE			= 705,
	 SCP_SERVICE_NAME		= 700,
	 SCP_METHOD_NAME		= 701,
	 SCP_METHOD_NAMESPACE		= 706,
	 SCP_ARGUMENT_IN		= 702,
	 SCP_ARGUMENT_OUT		= 703,
	 
	 SCP_SEND_SERVICE_REQUEST		=	711,
	 SCP_SEND_SERVICE_REPLY		=	712,
	 SCP_SOCKET_ID		=	713
	};

EVENT_PTR event_create(int type, int stream_id, void* src);
EVENT_PTR event_create_int(int type, int value, int stream_id, void* src);
EVENT_PTR event_create_string(int type, char* value, int stream_id, void* src);
EVENT_PTR event_create_void_pointer(int type, void* value, int stream_id, void* src);
EVENT_PTR event_create_longlong(int type, long long value, int stream_id, void* src);

EVENTSRC_PTR event_create_src(void* src_thread_parser, struct unit* src_unit);



void event_print(EVENT_PTR evt);

#endif //_EVENT_H_
	
