#ifndef _EVENTQUEUE_TABLE_H_
#define _EVENTQUEUE_TABLE_H_

#include "event.h"
#include <pthread.h>

typedef struct eventqueue_entry 
{
	EVENT_PTR evt;
	struct eventqueue_entry* next;
} EVENTQUEUE_ENTRY;
typedef EVENTQUEUE_ENTRY* EVENTQUEUE_ENTRY_PTR;

typedef struct eventqueue 
{
	struct eventqueue_entry* queue;
	int size;
} EVENTQUEUE;
typedef EVENTQUEUE* EVENTQUEUE_PTR;


typedef struct composer_thread_list_entry 
{
	struct composer* composer;
	
	int socket_reply_port;
	
	struct socket* socket_reply;
	
	char* composer_listener_string;
	
	pthread_cond_t cond;
	
	
	//pthread_t composer_thread_evtlistener;
	
	struct composer_thread_list_entry* next;
	
} COMPOSERTHREAD_LIST_ENTRY;
typedef COMPOSERTHREAD_LIST_ENTRY* COMPOSERTHREAD_LIST_ENTRY_PTR;

typedef struct eventqueue_table_entry 
{
	int stream_id; 
	int sub_stream_id;
	
	EVENTQUEUE_PTR event_queue;
	
	COMPOSERTHREAD_LIST_ENTRY_PTR composers_threads_list;

	int current_socket_reply_port;
	
	
	struct eventqueue_table_entry*  next;
	
	pthread_mutex_t mutex;
	
} EVENTQUEUE_TABLE_ENTRY;
typedef EVENTQUEUE_TABLE_ENTRY* EVENTQUEUE_TABLE_ENTRY_PTR;



/*
typedef struct composer_thread_list_entry 
{
	//struct composer* composer_thread_evtlistener;
	pthread_t composer_thread_evtlistener;
	struct composer_thread_list_entry* next;
} COMPOSERTHREAD_LIST_ENTRY;
typedef COMPOSERTHREAD_LIST_ENTRY* COMPOSERTHREAD_LIST_ENTRY_PTR;

typedef struct eventqueue_table_entry 
{
	int stream_id; //event_src_request; 
	int sub_stream_id;
	//void* composer_evtlistener_thread;
	
	EVENTQUEUE_PTR event_queue;
	
	
	int socket_reply_port;
	struct socket* socket_reply;
	//struct composer* composer_evtlistener;
	COMPOSERTHREAD_LIST_ENTRY_PTR composers_threads_list;
	char* composer_listener_string;
	
	struct eventqueue_table_entry*  next;
	
	pthread_mutex_t mutex;
	
} EVENTQUEUE_TABLE_ENTRY;
typedef EVENTQUEUE_TABLE_ENTRY* EVENTQUEUE_TABLE_ENTRY_PTR;
*/

typedef struct eventqueue_table
{
	struct eventqueue_table_entry* table;
	int size;
	
} EVENTQUEUE_TABLE;
typedef EVENTQUEUE_TABLE* EVENTQUEUE_TABLE_PTR;


EVENTQUEUE_TABLE_PTR eventqueue_table_create();
EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_add_entry(EVENTQUEUE_TABLE_PTR table, int stream_id, int sub_stream_id /*, void* thread_composer*/);
int eventqueue_table_contain_entry_stream_id(EVENTQUEUE_TABLE_PTR table, int stream_id);
//int eventqueue_table_contain_entry(EVENTQUEUE_TABLE_PTR table, int stream_id, void* thread_composer);
//EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_get_entry(EVENTQUEUE_TABLE_PTR table, int stream_id, void* thread_composer);
EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_get_entry_by_stream_id(EVENTQUEUE_TABLE_PTR table, int stream_id);
EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_get_entry_by_sub_stream_id(EVENTQUEUE_TABLE_PTR table, int sub_stream_id);
EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_get_entry_by_current_socket_reply_port(EVENTQUEUE_TABLE_PTR table, int socket_reply_port);

void eventqueue_table_enqueue_event(EVENTQUEUE_TABLE_ENTRY_PTR entry, EVENT_PTR evt);
EVENT_PTR eventqueue_table_get_event(EVENTQUEUE_TABLE_ENTRY_PTR entry);

void eventqueue_table_print(EVENTQUEUE_TABLE_PTR table);

int eventqueue_table_get_stream_id_by_sub_stream_id(EVENTQUEUE_TABLE_PTR table, int sub_stream_id, int* stream_id);
int eventqueue_table_get_sub_stream_ids_by_stream_id(EVENTQUEUE_TABLE_PTR table, int stream_id, int* n_sub_stream_ids, int* sub_stream_ids);


/*
void eventqueue_table_add_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry, pthread_t composer_thread_evtlistener);
pthread_t eventqueue_table_get_first_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
pthread_t eventqueue_table_get_last_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
pthread_t eventqueue_table_remove_first_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
int eventqueue_table_get_composer_thread_list_length(EVENTQUEUE_TABLE_ENTRY_PTR entry);

int eventqueue_table_lock_entry_mutex(EVENTQUEUE_TABLE_ENTRY_PTR entry);
int eventqueue_table_unlock_entry_mutex(EVENTQUEUE_TABLE_ENTRY_PTR entry);
*/

void eventqueue_table_add_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry, char* composer_listener_string, struct composer* composer,
			struct socket* socket_reply, int socket_reply_port /*, pthread_cond_t* cond*/);
COMPOSERTHREAD_LIST_ENTRY_PTR eventqueue_table_get_last_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
COMPOSERTHREAD_LIST_ENTRY_PTR eventqueue_table_get_secondlast_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
COMPOSERTHREAD_LIST_ENTRY_PTR eventqueue_table_get_first_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
COMPOSERTHREAD_LIST_ENTRY_PTR eventqueue_table_remove_first_composer_thread(EVENTQUEUE_TABLE_ENTRY_PTR entry);
int eventqueue_table_get_composer_thread_list_length(EVENTQUEUE_TABLE_ENTRY_PTR entry);
//int eventqueue_table_composer_thread_lock_entry_mutex(COMPOSERTHREAD_LIST_ENTRY_PTR composer);
//int eventqueue_table_composer_thread_unlock_entry_mutex(COMPOSERTHREAD_LIST_ENTRY_PTR composer);

EVENTQUEUE_TABLE_ENTRY_PTR eventqueue_table_copy_entry(EVENTQUEUE_TABLE_PTR table, EVENTQUEUE_TABLE_ENTRY_PTR entry_tmp);

#endif //_EVENTQUEUE_TABLE_H_

