#include <io/jtype.h>

#ifndef _SERVICEDEF_H_
#define _SERVICEDEF_H_

#define ARG_IN	0
#define ARG_OUT	1
#define METHODS_HASH_INDEX 1000

// method: argDir:
//public static final int IN = 0;
//public static final int OUT = 1;

typedef struct Argument {
	char* name;
	int dir;
	char* datatype;
	char* value;
	int index;
	
	struct Argument* next;
} ARGUMENTDEF, *ARGUMENTDEFPTR;

typedef struct Method {
	char* name;
	ARGUMENTDEFPTR args;
	struct Method* next;
//	String serviceType = null;
//	String interfaceType = null;	
//	boolean notImplemented;
	struct InterfaceDefinition* interface;
	int nArgs;
	char* namespace;
	int hash_index;
	
} METHODDEF, *METHODDEFPTR;

typedef struct InterfaceDefinition {
	char* name;
	char* namespace;
	char* XMLfile;
	METHODDEFPTR methods;
	char* uriPath;
	struct InterfaceDefinition* next;
	struct ServiceDefinition* service;
} INTERFACEDEF, *INTERFACEDEFPTR;

typedef struct ServiceDefinition {
	char* XMLfile;
	INTERFACEDEFPTR interfacesDefinition;
	char* host;
	int port;
	int indexService;
	
	char* name;
	char* type;
	long long RMI_ObjNum;
	long RMI_ObjUnique;
	long long RMI_ObjTime;
	short RMI_ObjCount;
	long RMI_ObjPort;

	
	j_long methods_hash_index;
    	/*long long  objNum;
    	int	  objId;
    	long long  objTime;         
    	short	  objCount;*/

} SERVICEDEF, *SERVICEDEFPTR;

typedef struct PathServiceDef_Elem {
	char* path;
	SERVICEDEFPTR service;
	struct PathServiceDef_Elem* next;
} PATH_SERVICEDEF, *PATH_SERVICEDEF_PTR;


/*
 *	function to handle table path/serviceDef
 */
int Table_add_PathServiceDef_Elem(PATH_SERVICEDEF_PTR* table, char* path, SERVICEDEFPTR service);
PATH_SERVICEDEF_PTR create_PathServiceDef_Table();
SERVICEDEFPTR Table_get_ServiceDef(PATH_SERVICEDEF_PTR table, char* path);
SERVICEDEFPTR Table_get_ServiceDef_and_path_by_RMIObjID(PATH_SERVICEDEF_PTR table, j_long objNum, j_int unique, j_long time, j_short count, char** service_path);


/*
 *	functions to handle servicedef
 */
SERVICEDEFPTR servicedef_new();
int servicedef_set_name(SERVICEDEFPTR servicedef, char* name);
int servicedef_set_host(SERVICEDEFPTR servicedef, char* host);
int servicedef_set_type(SERVICEDEFPTR servicedef, char* type);
int servicedef_set_port(SERVICEDEFPTR servicedef, int port);
int servicedef_set_indexService(SERVICEDEFPTR servicedef, int indexService);
int servicedef_add_interface(SERVICEDEFPTR servicedef, INTERFACEDEFPTR iface);
int servicedef_print(SERVICEDEFPTR servicedef);

METHODDEFPTR servicedef_getmethod_byname(SERVICEDEFPTR servicedef, char* name);
METHODDEFPTR servicedef_getmethod_byHash(SERVICEDEFPTR servicedef, j_long hash);
int servicedef_contains_method(SERVICEDEFPTR servicedef, METHODDEFPTR method);

int servicedef_set_RMI_ObjNum(SERVICEDEFPTR servicedef, long long num);
int servicedef_set_RMI_ObjUnique(SERVICEDEFPTR servicedef, long unique);
int servicedef_set_RMI_ObjTime(SERVICEDEFPTR servicedef, long long time);
int servicedef_set_RMI_ObjCount(SERVICEDEFPTR servicedef, short count);
int servicedef_set_RMI_ObjPort(SERVICEDEFPTR servicedef, long port);



/*
 *	functions to handle interfacedef
 */
INTERFACEDEFPTR interfacedef_new2();
INTERFACEDEFPTR interfacedef_new(SERVICEDEFPTR srv);
int interfacedef_set_name(INTERFACEDEFPTR interfacedef, char* name);
int interfacedef_set_namespace(INTERFACEDEFPTR interfacedef, char* namespace);
int interfacedef_set_uripath(INTERFACEDEFPTR interfacedef, char* uripath);
int interfacedef_add_method(INTERFACEDEFPTR interfacedef, METHODDEFPTR meth);
int interfacedef_print(INTERFACEDEFPTR interfacedef);
METHODDEFPTR interfacedef_getmethod_byname(INTERFACEDEFPTR interfacedef, char* name);
METHODDEFPTR interfacedef_getmethod_byHash(INTERFACEDEFPTR interfacedef, j_long hash);


/*
 *	functions to handle methoddef
 */
METHODDEFPTR methoddef_new2();
METHODDEFPTR methoddef_new(INTERFACEDEFPTR iface);
int methoddef_set_name(METHODDEFPTR methoddef, char* name);
int methoddef_set_namespace(METHODDEFPTR methoddef, char* namespace);
int methoddef_add_argument(METHODDEFPTR methoddef, ARGUMENTDEFPTR arg);
int methoddef_print(METHODDEFPTR methoddef);
int methoddef_equal(METHODDEFPTR methoddef1, METHODDEFPTR methoddef2);
char* get_method_signature(ARGUMENTDEFPTR args, char* returnType);
/*char* methoddef_get_Java_signature(METHODDEFPTR methoddef);*/

/*
 *	functions to handle argumentdef
 */
ARGUMENTDEFPTR argumentdef_new();
int argumentdef_set_name(ARGUMENTDEFPTR argumentdef, char* name);
int argumentdef_set_dir(ARGUMENTDEFPTR argumentdef, int dir);
int argumentdef_set_datatype(ARGUMENTDEFPTR argumentdef, char* datatype);
int argumentdef_set_value(ARGUMENTDEFPTR argumentdef, char* value);
int argumentdef_set_index(ARGUMENTDEFPTR argumentdef, int index);
int argumentdef_print(ARGUMENTDEFPTR argumentdef) ;


static int GlobalServiceIndex = 3;

#endif //_SERVICEDEF_H_

