#ifndef _CONSTANTPOOL_H_
#define _CONSTANTPOOL_H_

#ifndef _JCONST_H_
#include <io/jconst.h>
#endif 
#ifndef _JTYPE_H_
#include <io/jtype.h>
#endif 
#ifndef LIST_H
#include <kazlib/list.h>
#endif 

enum ConstantTag {
	CONSTANT_Empty 		=0,
	CONSTANT_Utf8 		=1, 
	CONSTANT_Integer	=3,
	CONSTANT_Float		=4,
	CONSTANT_Long		=5,
	CONSTANT_Double 	=6,
	CONSTANT_Class		=7,
	CONSTANT_String 	=8,
	CONSTANT_Fieldref	=9,
	CONSTANT_Methodref 	=10,
	CONSTANT_InterfaceMethodref	=11,
	CONSTANT_NameAndType 		=12
};


static char* tagtable[] ={
      "CONSTANT_Empty",
	  "CONSTANT_Utf8",
	  "CONSTANT_????",
	  "CONSTANT_Integer",
	  "CONSTANT_Float",
	  "CONSTANT_Long",
	  "CONSTANT_Class",
	  "CONSTANT_String",
	  "CONSTANT_Fieldref",
	  "CONSTANT_Methodref",
	  "CONSTANT_InterfaceMethodref",
	  "CONSTANT_Empty",
	  "CONSTANT_NameAndType"};
	  
typedef enum ConstantTag TAG;

	  
struct ConstantClassInfo {
    	uint16* name_index;
}; 

typedef struct ConstantClassInfo CLASS_INFO;
typedef CLASS_INFO* CLASS_INFO_PTR;

struct ConstantReferenceInfo {
    	uint16* class_index;
    	uint16* name_and_type_index;
};

typedef struct ConstantReferenceInfo REFERENCE_INFO;
typedef REFERENCE_INFO* REF_INFO_PTR;


struct ConstantStringInfo {
    	uint16* string_index;
};

typedef struct ConstantStringInfo STRING_INFO;
typedef STRING_INFO* STRING_INFO_PTR;

struct ConstantNameAndTypeInfo{
    	uint16* name_index;
    	uint16* descriptor_index;
};

typedef struct ConstantNameAndTypeInfo NAME_TYPE_INFO;
typedef NAME_TYPE_INFO* NAME_TYPE_INFO_PTR;

struct ConstantIntergerInfo {
    	j_int bytes;
};

typedef struct ConstantIntegerInfo INTEGER_INFO;
typedef INTEGER_INFO* INTEGER_INFO_PTR;


struct ConstantLongInfo {
    	j_long long_bytes;
};

typedef struct ConstantLongInfo LONG_INFO;
typedef LONG_INFO* LONG_INFO_PTR;

struct ConstantUtf8Info{
		UTF8_PTR utf8;
}; 
typedef struct ConstantUtf8Info UTF8_INFO;
typedef UTF8_INFO* UTF8_INFO_PTR;

struct ConstantPoolEntry{
	uint8 	tag;
	uint16 	index; 
	union{
			UTF8_INFO_PTR		utf8;
			NAME_TYPE_INFO_PTR	nameandtype;
			STRING_INFO_PTR		string;
			REF_INFO_PTR 		ref;
			CLASS_INFO_PTR		cl;
			LONG_INFO_PTR		l;
			INTEGER_INFO_PTR 	i;
		}info;
	void (*write)(int,struct ConstantPoolEntry*);
	};
	 
typedef struct ConstantPoolEntry POOL_ENTRY;
typedef POOL_ENTRY* POOL_ENTRY_PTR;

struct Pool{
	POOL_ENTRY_PTR* tab;
	uint16 			size;
	list_t* 		listConstString;
	list_t* 		listConstClass;
	list_t*			listConstRef;
	list_t* 		listConstType;
	list_t* 		listConstNameType;
	list_t* 		listConstUtf8;		
	list_t*			listConstPrimitive;
};

typedef struct Pool POOL;
typedef POOL* POOL_PTR;

extern POOL_PTR new_pool();
extern int add_pool_entry(POOL_PTR pool,TAG t,POOL_ENTRY_PTR entry);
/*extern uint16* add_rec_entry(POOL_PTR pool,TAG t, const char* classname);*/
extern uint16* add_rec_class_entry(POOL_PTR pool,const char* classname);
extern uint16* add_rec_utf8_entry(POOL_PTR pool,const char* classname);
extern uint16* add_rec_ref_entry(POOL_PTR pool,TAG t,const char* classname,const char* varName,const char* type);
extern uint16* add_rec_name_type_entry(POOL_PTR pool,const char* varName,const char* type);
extern uint16* add_rec_long_entry(POOL_PTR pool,j_long l);
extern uint16* add_rec_string_entry(POOL_PTR pool,char* name);
void pool_list_incr(list_t* l,lnode_t* n,void* count);


// Methods to create new entry 
extern POOL_ENTRY_PTR new_constant_utf8_info(const char* str);
extern POOL_ENTRY_PTR new_constant_class_info(uint16* index);
extern POOL_ENTRY_PTR new_constant_name_type_info(uint16* i1,uint16* i2);

/*extern POOL_ENTRY_PTR new_constant_name_type_info(const char* name,const char* type);
extern POOL_ENTRY_PTR new_constant_ref_info(const char* refname);
extern POOL_ENTRY_PTR new_constant_long_info(j_long l);
extern POOL_ENTRY_PTR new_constant_integer_info(j_int i);
extern POOL_ENTRY_PTR new_constant_class_info();
extern POOL_ENTRY_PTR new_constant_string_info(const char* str);*/
extern void print_pool(POOL_PTR pool,TAG t);
extern int 	pool_search_for_UTF8(POOL_PTR pool,const char* pattern);
extern void pool_sort_constant_pool(POOL_PTR pool);
extern void pool_free_pool_entry(POOL_ENTRY_PTR entry);
extern const char*	pool_get_utf8_string(POOL_PTR pool,uint16 index);
extern POOL_ENTRY_PTR pool_search_for_class_name( POOL_PTR pool,const char* nameClass);
//extern POOL_ENTRY_PTR pool_search_for_class_index(POOL_PTR pool,uint16 index);
extern POOL_ENTRY_PTR pool_search_for_name_and_type_index(POOL_PTR pool,\
											uint16 iname, uint16 itype);
											
#endif
