#include "composer/composer.h"
#include "unit/unit.h"
#include "event/eventqueue_table.h"
#include <stdio.h>
#include <pthread.h>

#include "composer/slpcomposer.h"


int composer_add_msglistener(COMPOSER_PTR composer, struct listener* evt_list)
{
	fprintf(stdout, "composer_add_msglistener\n");
        LISTENER_LIST_ENTRY_PTR list_entry = (LISTENER_LIST_ENTRY_PTR)malloc(sizeof(LISTENER_LIST_ENTRY));
	list_entry->node = evt_list;
	list_entry->next = NULL;
	
	if (composer->msg_listeners == NULL) {
		composer->msg_listeners = list_entry;	
	} else {
		list_entry->next = composer->msg_listeners;
		composer->msg_listeners = list_entry;
	}
}

int composer_add_msglistener_socket(COMPOSER_PTR composer, struct socket* s)
{
	fprintf(stdout, "composer_add_msglistener_socket\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER_PTR));
	evt_list->type = LISTENER_SOCKET;
	evt_list->listener.socket = s;
	composer_add_msglistener(composer, evt_list);
}
	
int composer_add_msglistener_unit(COMPOSER_PTR composer, struct unit* u)
{
	fprintf(stdout, "composer_add_msglistener_unit\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER_PTR));
	evt_list->type = LISTENER_UNIT;
	evt_list->listener.unit = u;
	composer_add_msglistener(composer, evt_list);
}

void composer_publish_msg(MSG_PTR msg, COMPOSER_PTR composer)
{
	fprintf(stdout, "composer_publish_msg\n");
	LISTENER_LIST_PTR l;

	for (l = composer->msg_listeners; l != NULL; l = l->next)
	{
	   LISTENER_PTR node = l->node;

	   if (node->type == LISTENER_PARSER) {
		PARSER_PTR p = (PARSER_PTR)node->listener.parser;
		fprintf(stdout, "composer_publish_msg on parser %p\n",p);
		
			p->func_msg_received(msg, node);
	   }
	   else if (node->type == LISTENER_UNIT) {
		UNIT_PTR u = (UNIT_PTR)node->listener.unit;
		fprintf(stdout, "composer_publish_msg on unit %p\n",u);
		
			u->func_msg_received(msg, node);
	   }
	   else if (node->type == LISTENER_SOCKET) {
		SOCKET_PTR s = (SOCKET_PTR)node->listener.socket;
		fprintf(stdout, "composer_publish_msg on socket %p\n",s);
		
			s->func_msg_received(msg, node);
	   }
	   
	}
}



int composer_add_evtlistener(COMPOSER_PTR composer, struct listener* evt_list)
{
	fprintf(stdout, "composer_add_evtlistener\n");
      LISTENER_LIST_ENTRY_PTR list_entry = (LISTENER_LIST_ENTRY_PTR)malloc(sizeof(LISTENER_LIST_ENTRY));
	list_entry->node = evt_list;
	list_entry->next = NULL;
	
	if (composer->evt_listeners == NULL) {
		composer->evt_listeners = list_entry;	
	} else {
		list_entry->next = composer->evt_listeners;
		composer->evt_listeners = list_entry;
	}
}


int composer_add_evtlistener_unit(COMPOSER_PTR composer, struct unit* u)
{
	fprintf(stdout, "composer_add_evtlistener_unit\n");
	LISTENER_PTR evt_list = (LISTENER_PTR)malloc(sizeof(LISTENER/*_PTR*/));
	evt_list->type = LISTENER_UNIT;
	evt_list->listener.unit = u;
	composer_add_evtlistener(composer, evt_list);
}




void composer_publish_event(EVENT_PTR evt, COMPOSER_PTR composer)
{
	fprintf(stdout, "composer_publish_event\n");
	LISTENER_LIST_PTR l;

	for (l = composer->evt_listeners; l != NULL; l = l->next)
	{
		if (l != NULL)
		{
		   LISTENER_PTR node = l->node;
	
		   if (node != NULL && node->type == LISTENER_UNIT) {
			 UNIT_PTR u = (UNIT_PTR)node->listener.unit;
			 fprintf(stdout, "composer_publish_event on unit %p\n",u);
			 //if (u != evt->src)
				u->func_event_received(evt, node);
		   }
		}
	}
}




void composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
/*

	fprintf(stdout, "composer receives event type=%d src=%p. composer listeners=%p\n", evt->type, evt->stream_id, node->listener.composer);
	
	COMPOSER_PTR composer = node->listener.composer; 
	
	// check if evt->stream_id exists in table/column stream_id
	EVENTQUEUE_TABLE_ENTRY_PTR entry = eventqueue_table_get_entry_by_stream_id(composer->unit->event_table, evt->stream_id);
	// if evt->stream_id does not exist in table/column stream_id, check in column sub_stream_id
	if (entry == NULL)
		entry = eventqueue_table_get_entry_by_sub_stream_id(composer->unit->event_table, evt->stream_id);
	
	// if not found in column stream_id and nor in column sub_stream_id, create a new entry
	if (entry == NULL)	
	{
		fprintf(stdout, "composer_event_received: event src not contained in table \n");
*/
//		entry = eventqueue_table_add_entry(composer->unit->event_table, evt->stream_id ,  -1/*sub_stream_id*/ /*, (void*)0*/);
/*		eventqueue_table_enqueue_event(entry, evt);
		
		pthread_t composer_thread;
		COMPOSETHREADDATA_PTR data = malloc(sizeof(COMPOSETHREADDATA));
	
		data->entry_event_queue = (EVENTQUEUE_TABLE_ENTRY_PTR)entry;
		data->composer = node->listener.composer;
	
	
		
		fprintf(stdout, "\nCREATING PTHREAD composer = %p\n", composer_thread);
	
		
		pthread_create(&composer_thread, NULL, composer->run_composer, (void*)data);
		
	}
	else
	{
		fprintf(stdout, "composer_event_received: event src contained in table \n");
		eventqueue_table_enqueue_event(entry, evt);
		
	}
*/
}


COMPOSER_PTR composer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating composer\n");
	
	COMPOSER_PTR c = (COMPOSER_PTR)malloc(sizeof(COMPOSER));
	c->func_event_received = composer_event_received;
	c->func_publish_msg = composer_publish_msg;
	c->msg_listeners = NULL;
	c->evt_listeners = NULL;
	c->unit = u;	
	
	return c;	
}

