#include "composer/composer.h"
#include "unit/unit.h"
#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>

#include "composer/SOAPComposer.h"
#include "composer/JRMPComposer.h"
#include "monitor/tools.h"

#define STR_CONTENT_LENGTH "Content-Length"
#define STR_SOAP_ACTION "SOAPACTION"

#define STR_COLON ":"
#define STR_SPACE " "
#define STR_HASH "#"
#define STR_LINE_DELIMITER "\r\n"


//#define URL "urn:schemas-upnp-org:service:power:1"
//#define URL "http://tempuri.org/HelloWorld"

#define HTTP_OK_RESPONSE "HTTP/1.1 200 OK"
#define HTTP_VERSION "HTTP/1.1"

MSG_PTR soapcomposer_soap(int bSoapType, METHODDEFPTR method_to_call, char* dest_addr, int dest_port, char* dest_url, SERVICEDEFPTR service_ptr, int src_socket_id)
{
       char *data=NULL;
       int iContentSize =0;
			 int iDataSize =0;
			 int iHeaderSize =0;

       // Adjusting POST HTTP HEADER
       if(bSoapType == SOAP_REQ)
       {
               printf("\nsoapcomposer_soap REQ\n");
               SOAP_AdjustHttpPOSTHeaderLength(&iHeaderSize, dest_url);
       }
			 else if (bSoapType == SOAP_RES)
			 {
               printf("\nsoapcomposer_soap RES\n");
							 SOAP_AdjustHttpResultHeaderLength(&iHeaderSize,HTTP_OK_RESPONSE);							 
			 }
       SOAP_AdjustHttpHOSTHeaderLength(&iHeaderSize, dest_addr, dest_port);

       iContentSize += strlen(SOAP_ELEMENT_PREPRECESSOR);
       iContentSize += strlen(SOAP_ELEMENT_ENVELOPE_START);

       // Adjusting Body Length
       iContentSize += strlen(SOAP_ELEMENT_BODY_START);
       if(bSoapType == SOAP_REQ)
       {
               SOAP_AdjustMethodStartTagLength(&iContentSize,method_to_call->name);
       }
       else if(bSoapType == SOAP_RES)
       {
               iContentSize += strlen(STR_RESPONSE);
               SOAP_AdjustMethodStartTagLength(&iContentSize,method_to_call->name);
       }


       ARGUMENTDEFPTR args = method_to_call->args;
       while(args != NULL)
       {
               if(bSoapType == SOAP_REQ)
               {
                       if(args->dir == ARG_IN)
                       {
                               SOAP_AdjustArgumentStartTagLength(&iContentSize,args->name,args->datatype);
                               SOAP_AdjustArgumentValueLength(&iContentSize,args->value);
                               SOAP_AdjustArgumentEndTagLength(&iContentSize,args->name);
                       }
               }
               else if(bSoapType == SOAP_RES)
               {
                       if(args->dir == ARG_OUT)
                       {
                               SOAP_AdjustArgumentStartTagLength(&iContentSize,args->name,args->datatype);
                               SOAP_AdjustArgumentValueLength(&iContentSize,args->value);
                               SOAP_AdjustArgumentEndTagLength(&iContentSize,args->name);
                       }
               }
               args = args->next;
       }

       if(bSoapType == SOAP_REQ)
       {
               SOAP_AdjustMethodEndTagLength(&iContentSize,method_to_call->name);
       }
       else if(bSoapType == SOAP_RES)
       {
               iContentSize += strlen(STR_RESPONSE);
               SOAP_AdjustMethodEndTagLength(&iContentSize,method_to_call->name);
       }
       iContentSize += strlen(SOAP_ELEMENT_BODY_END);
       iContentSize += strlen(SOAP_ELEMENT_ENVELOPE_END);

        SOAP_AdjustHttpEndHeaderLength(bSoapType, &iHeaderSize,iContentSize,/*URL,*/method_to_call, service_ptr);
			 			 
	 // TODO : Change here to incorporate the change in the arguments of length
			 
			 
       data = (char*)malloc(sizeof(char)*(iContentSize+iHeaderSize));

       if(bSoapType == SOAP_REQ)
       {
               printf("\nsoapcomposer_soap REQ url=%s\n",dest_url);
               SOAP_ComposeHttpPOSTHeader(data, dest_url);
       }
			 else if (bSoapType == SOAP_RES)
			 {
              						 printf("\nsoapcomposer_soap RES \n");
						 	 SOAP_ComposeHttpResultHeader(data, HTTP_OK_RESPONSE );							 
			 }
       SOAP_ComposeHttpHOSTHeader(data, dest_addr, dest_port);			 			 
       SOAP_ComposeHttpEndHeader(bSoapType, data,iContentSize,/*URL,*/method_to_call, service_ptr);
 
       strcat(data,SOAP_ELEMENT_PREPRECESSOR);
       strcat(data,SOAP_ELEMENT_ENVELOPE_START);
       SOAP_ComposeBodyElement(bSoapType, data,method_to_call);
       strcat(data,SOAP_ELEMENT_ENVELOPE_END);

       printf("The composer SOAP message is \n %s\n",data);
       //MSG_PTR reply_msg = msg_create(data,strlen(data), "128.93.8.194",0, "239.255.255.250", 3702, NULL);
       
   	   MSG_PTR reply_msg = msg_create_sock_id(data,strlen(data), get_host_ip()/*"192.168.2.2"*/,0, dest_addr, dest_port, src_socket_id, NULL);

       fprintf(stdout, "REPLY_MSG->DATA = %p\n",&reply_msg->data );
       return reply_msg;
}


void *soap_run_composer(void* ptr)
{
       fprintf(stdout,"soap_run_composer: -----------------------------------------------\n");

       COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;

       EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
       COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;

       pthread_mutex_lock(&(entry->mutex));

       // Add the identifier of the thread composer to the eventqueue_table
       pthread_t thread_composer = pthread_self();

   
       /*
        char* reply_dest_addr;
       int reply_dest_port;
       char* call_dest_addr;
       int call_dest_port;

       METHODDEFPTR method_to_call;
       METHODDEFPTR method_to_reply;

       char* method_to_call_service_name;
       */
   		SERVICE_COMM_TABLE services_table;
   		service_comm_table_init(&services_table);
       
       while (1)
       {
               EVENT_PTR evt = eventqueue_table_get_event(entry);
               if (evt != NULL)
               {
                       event_print(evt);

                       switch (evt->type)
                       {
                       case SCP_SOCKET_ID:
                       {
                    	   fprintf(stdout, "soap_run_composer: SCP_SOCKET_ID value=%d\n",evt->value.value_int);
              				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
            				{
            					service_comm_table_add_entry(&services_table, evt->stream_id);
            				}
            				service_comm_table_set_src_socket_id(&services_table, evt->stream_id, evt->value.value_int);
                       }
                       break;
                       
                       case SDP_SOURCE_ADDR :
	           				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
	        				{
	        					service_comm_table_add_entry(&services_table, evt->stream_id);
	        				}
	        				service_comm_table_set_source_addr(&services_table, evt->stream_id, evt->value.value_string);
	                        fprintf(stdout, "soap_run_composer: SDP_SOURCE_ADDR");
                            //reply_dest_addr =strdup(evt->value.value_string);
                            break;

                       case SDP_SOURCE_PORT :
           				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
        					service_comm_table_add_entry(&services_table, evt->stream_id);
        				}
        				service_comm_table_set_source_port(&services_table, evt->stream_id, evt->value.value_int);
                               fprintf(stdout, "soap_run_composer: SDP_SOURCE_PORT");
                               //reply_dest_port =  evt->value.value_int;
                               break;

                       case SDP_REQ_LANGAGE :
                               break;
                       case SDP_REQ_SCOPE :
                               break;
                       case SDP_REQ_PREDICAT :
                               break;
                       case SDP_REQ_REQUESTER :
                               break;
                       case SDP_REQ_SERVICE_TYPE :
                               break;
                       case SDP_REQ_ID :
                               break;
                       case SDP_VERSION :
                               break;
                       case SDP_STOP :
                               return;
                               break;
                               
           				case SCP_SERVICE_NAME:
           					if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
           					{
           						service_comm_table_add_entry(&services_table, evt->stream_id);
           					}
           					service_comm_table_set_service_name(&services_table, evt->stream_id, evt->value.value_string);
           					//method_to_call_service_name = strdup(evt->value.value_string); 
           					fprintf(stdout, "SCP_SERVICE_NAME value = %s\n",evt->value.value_string);
        				break;

           				case SCP_METHOD_NAMESPACE:
           				{
               				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
            				{
            					service_comm_table_add_entry(&services_table, evt->stream_id);
            				}
            				METHODDEFPTR m = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);
            				//method_name = strdup(evt->value.value_string);
            				//method_to_call = methoddef_new2();
            				methoddef_set_namespace(m, evt->value.value_string);	
           					//methoddef_set_namespace(method_to_call, evt->value.value_string); 
           					fprintf(stdout, "SCP_SERVICE_NAME value = %s\n",evt->value.value_string);
           				}
        				break;
        				
        				
                       case SCP_SEND_SERVICE_REPLY:
           				if (service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
        					fprintf(stdout, "soap_run_composer: SCP_SEND_SERVICE_REPLY");
        					
        					char* reply_dest_addr = service_comm_table_get_source_addr_by_stream_id(services_table, evt->stream_id);
        					METHODDEFPTR method_to_reply = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);
        					int reply_dest_port =  service_comm_table_get_source_port_by_stream_id(services_table, evt->stream_id);
        					int fd = service_comm_table_get_src_socket_id_by_stream_id(services_table, evt->stream_id);
        					
                               methoddef_print(method_to_reply);

                               MSG_PTR msg = soapcomposer_soap(SOAP_RES, method_to_reply, reply_dest_addr, reply_dest_port, NULL, NULL, fd);
                               composer_publish_msg(msg, composer);
        				}
                               break;

                       case SCP_SEND_SERVICE_REQUEST:
                       {
           				if (service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
           					fprintf(stdout, "soap_run_composer: SCP_SEND_SERVICE_REQUEST");
        					
        					char* method_to_call_service_name = service_comm_table_get_service_name_by_stream_id(services_table, evt->stream_id);
        					METHODDEFPTR method_to_call = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);

                               SERVICEDEFPTR srvTmp = Table_get_ServiceDef(ServiceDefinitionTable, method_to_call_service_name);
               			    	if (srvTmp != NULL)
               			    	{
                               
		                               //MSG_PTR msg = soapcomposer_soap(SOAP_REQ, method_to_call, /*get_host_ip()/*/"192.168.2.3", /*4004*/1045, /*"/service/power/control"*/ "/MyService/service.asmx");
               			    		MSG_PTR msg = soapcomposer_soap(SOAP_REQ, method_to_call, srvTmp->host, srvTmp->port, srvTmp->interfacesDefinition->uriPath, srvTmp, -1);
		
		
		                               fprintf(stdout, "slp_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
		                               COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
		                               // set current_socket_reply_port for ssdp parser to find the right entry on the table
		                               entry->current_socket_reply_port = first_composer_thread->socket_reply_port;
		
		
		                               composer_publish_msg(msg, composer);
		
		
		                               pthread_cond_signal(&(first_composer_thread->cond));
		
		                               fprintf(stdout, "slp_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
		
		                               pthread_mutex_unlock(&(entry->mutex));
		
		                               fprintf(stdout, "slp_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
               			    	}
        				}
                       }
                               break;

                       case SCP_METHOD_NAME:
                       {
           				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
        					service_comm_table_add_entry(&services_table, evt->stream_id);
        				}
        				METHODDEFPTR m = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);
        				//method_name = strdup(evt->value.value_string);
        				//method_to_call = methoddef_new2();
        				methoddef_set_name(m, evt->value.value_string);	
        				/*
                               char* methodName = strdup(evt->value.value_string);
                               method_to_reply = methoddef_new2();
                               methoddef_set_name(method_to_reply, methodName);

                               method_to_call = methoddef_new2();
                               methoddef_set_name(method_to_call, methodName);
                               	*/
                               fprintf(stdout, "soap_run_composer: SCP_METHOD_NAME");
                       }
                               break;

                       case SCP_ARGUMENT_IN:
                       {
           				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
        					service_comm_table_add_entry(&services_table, evt->stream_id);
        				}
        				METHODDEFPTR m = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);
        				//method_name = strdup(evt->value.value_string);
        				//method_to_call = methoddef_new2();
        				//methoddef_set_name(method_to_call, method_name);	
        				//service_comm_table_set_method(&services_table, evt->stream_id, method_to_call);
        				methoddef_add_argument(m, evt->value.value_void_pointer);
                               fprintf(stdout, "soap_run_composer: SCP_ARGUMENT_IN");
                              // methoddef_add_argument(method_to_call, evt->value.value_void_pointer);
                       }
                               break;

                       case SCP_ARGUMENT_OUT:
                       {
           				if (!service_comm_table_contains_stream_id(services_table, evt->stream_id))	
        				{
        					service_comm_table_add_entry(&services_table, evt->stream_id);
        				}
        				METHODDEFPTR m = service_comm_table_get_methoddef_by_stream_id(services_table, evt->stream_id);
        				//method_name = strdup(evt->value.value_string);
        				//method_to_call = methoddef_new2();
        				//methoddef_set_name(method_to_call, method_name);	
        				//service_comm_table_set_method(&services_table, evt->stream_id, method_to_call);
        				methoddef_add_argument(m, evt->value.value_void_pointer);
                               fprintf(stdout, "soap_run_composer: SCP_ARGUMENT_OUT");
                            //   methoddef_add_argument(method_to_reply, evt->value.value_void_pointer);
                       }
                               break;


                       }
               }
       }
}

void soap_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
       fprintf(stdout,"SOAP");
       composer_event_received(evt, node);
}

COMPOSER_PTR soapcomposer_create(UNIT_PTR u)
{
       fprintf(stdout, "Creating SOAP composer\n");


       COMPOSER_PTR c = composer_create(u);

       c->run_composer = soap_run_composer;
       c->factory = soapcomposer_create;

       c->func_event_received = soap_composer_event_received;
       return c;
}

void SOAP_ComposeBodyElement(int bSoapType, char *data, METHODDEFPTR method_to_call)
{
       strcat(data,SOAP_ELEMENT_BODY_START);
       ComposerMethodStartTag(bSoapType, data,method_to_call->name);

       ARGUMENTDEFPTR args = method_to_call->args;
       while(args != NULL)
       {
               if(bSoapType == SOAP_REQ)
               {
                       if(args->dir == ARG_IN)
                       {
                               ComposeArgumentStartTag(data,args->name,args->datatype);
                               ComposeArgumentValue(data,args->value);
                               ComposeArgumentEndTag(data,args->name);
                       }
               }
               else if(bSoapType == SOAP_RES)
               {
                       if(args->dir == ARG_OUT)
                       {
                               ComposeArgumentStartTag(data,args->name,args->datatype);
                               ComposeArgumentValue(data,args->value);
                               ComposeArgumentEndTag(data,args->name);
                       }
               }
               args = args->next;
       }

       ComposerMethodEndTag(bSoapType, data,method_to_call->name);
       strcat(data,SOAP_ELEMENT_BODY_END);
}

void ComposerMethodStartTag(int bSoapType, char *data,char *methodName)
{
       strcat(data,SOAP_ELEMENT_START);

       if(bSoapType == SOAP_REQ)
       {
               strcat(data,methodName);
       }
       else if(bSoapType == SOAP_RES)
       {
               strcat(data,methodName);
               strcat(data, STR_RESPONSE);
       }
       strcat(data,SOAP_ELEMENT_END);
}

void ComposerMethodEndTag(int bSoapType, char *data,char *methodName)
{
       strcat(data,SOAP_ELEMENT_START);
       strcat(data,SOAP_ELEMENT_END_MARKER);
       if(bSoapType == SOAP_REQ)
       {
               strcat(data,methodName);
       }
       else if(bSoapType == SOAP_RES)
       {
               strcat(data,methodName);
               strcat(data, STR_RESPONSE);
       }
       strcat(data,SOAP_ELEMENT_END);
}

void ComposeArgumentStartTag(char *data,char *argName,char *type)
{
       strcat(data,SOAP_ELEMENT_START);
       strcat(data,argName);
       strcat(data,SPACE);
       strcat(data,STR_TYPE_QUALIFIER);
       strcat(data,EQUALITY);
       strcat(data,QUOTE);
       strcat(data,type);
       strcat(data,QUOTE);
       strcat(data,SOAP_ELEMENT_END);
}

void ComposeArgumentValue(char *data,char *Value)
{
       strcat(data,Value);
}

void ComposeArgumentEndTag(char *data,char *argName)
{
       strcat(data,SOAP_ELEMENT_START);
       strcat(data,SOAP_ELEMENT_END_MARKER);
       strcat(data,argName);
       strcat(data,SOAP_ELEMENT_END);
}

void SOAP_ComposeHttpPOSTHeader(char* data, char* dest_url)
{       
       strcat(data, SOAP_HTTP_POST);
       strcat(data, dest_url);
       strcat(data, SPACE);
       strcat(data, HTTP_VERSION);       
       strcat(data, STR_LINE_DELIMITER);
}
void SOAP_ComposeHttpResultHeader(char* data, char* resultStr)
{
       strcat(data, resultStr);
       strcat(data, STR_LINE_DELIMITER);
}
void SOAP_ComposeHttpHOSTHeader(char* data, char* dest_addr, int dest_port)
{
       strcat(data, SOAP_HTTP_HOST);
       strcat(data, dest_addr);
       strcat(data, STR_COLON);
       char str_port[10];
       sprintf(str_port,"%d",dest_port);
       strcat(data, str_port);
       strcat(data, STR_LINE_DELIMITER);
}

void SOAP_ComposeHttpEndHeader(int bSoapType, char* data,int iContentLength,/*char *url,*/ METHODDEFPTR method_to_call, SERVICEDEFPTR service_ptr)
{				
		// Content length
       char str_ContentLength[10];
       sprintf(str_ContentLength,"%d",iContentLength);	 
       strcat(data, STR_CONTENT_LENGTH); 
       strcat(data, STR_COLON); 			 
       strcat(data, STR_SPACE); 			 			 			 
       strcat(data, str_ContentLength);
       strcat(data, STR_LINE_DELIMITER);
       
       if(bSoapType == SOAP_REQ)
       {
	       // SOAP ACTION
	       strcat(data, STR_SOAP_ACTION);			 
		   strcat(data, STR_COLON); 			 
		   strcat(data, STR_SPACE); 			 			 			 			 
	       strcat(data, QUOTE);
	       
	       fprintf(stdout, "SOAP_ComposeHttpEndHeader 0\n");
	       methoddef_print(method_to_call);
	       
	       if (method_to_call->namespace != NULL)
	       {
	    	   //strcat(data, url);
	    	   strcat(data, method_to_call->namespace );
	       }
	       else
	       {
	       // strcat(data, STR_HASH);       
		   // strcat(data, methodName);
	    	   strcat(data, service_ptr->interfacesDefinition->namespace);
	    	   strcat(data, STR_HASH);       
	    	   strcat(data, method_to_call->name);
	       }
	       
	       strcat(data, QUOTE);			 
	       strcat(data, STR_LINE_DELIMITER);       

		   fprintf(stdout, "SOAP_ComposeHttpEndHeader1: namespace=%s\n",service_ptr->interfacesDefinition->namespace);
       }
	   strcat(data, STR_LINE_DELIMITER);
}


void SOAP_AdjustMethodStartTagLength(int *iContentSize,char *methodName)
{
       *iContentSize += strlen(SOAP_ELEMENT_START);
       *iContentSize += strlen(methodName);
       *iContentSize += strlen(SOAP_ELEMENT_END);
}

void SOAP_AdjustMethodEndTagLength(int *iContentSize,char *methodName)
{
       *iContentSize += strlen(SOAP_ELEMENT_START);
       *iContentSize += strlen(SOAP_ELEMENT_END_MARKER);
       *iContentSize += strlen(methodName);
       *iContentSize += strlen(SOAP_ELEMENT_END);
}

void SOAP_AdjustArgumentStartTagLength(int *iContentSize,char *argName,char *type)
{
       *iContentSize += strlen(SOAP_ELEMENT_START);
       *iContentSize += strlen(argName);
       *iContentSize += strlen(SPACE);
       *iContentSize += strlen(STR_TYPE_QUALIFIER);
       *iContentSize += strlen(EQUALITY);
       *iContentSize += strlen(QUOTE);
       *iContentSize += strlen(type);
       *iContentSize += strlen(QUOTE);
       *iContentSize += strlen(SOAP_ELEMENT_END);
}

void SOAP_AdjustArgumentValueLength(int *iContentSize,char *Value)
{
       *iContentSize += strlen(Value);
}

void SOAP_AdjustArgumentEndTagLength(int *iContentSize,char *argName)
{
       *iContentSize += strlen(SOAP_ELEMENT_START);
       *iContentSize += strlen(SOAP_ELEMENT_END_MARKER);
       *iContentSize += strlen(argName);
       *iContentSize += strlen(SOAP_ELEMENT_END);
}

void SOAP_AdjustHttpPOSTHeaderLength(int *iHeaderSize, char* dest_url)
{

       *iHeaderSize += strlen(SOAP_HTTP_POST);
       *iHeaderSize += strlen(dest_url);
       *iHeaderSize += strlen(SPACE);
       *iHeaderSize += strlen(HTTP_VERSION);
       *iHeaderSize += strlen(STR_LINE_DELIMITER);
}
void SOAP_AdjustHttpResultHeaderLength(int *iHeaderSize, char* resultStr)
{
       *iHeaderSize += strlen(resultStr);
       *iHeaderSize += strlen(STR_LINE_DELIMITER);
}

void SOAP_AdjustHttpHOSTHeaderLength(int *iHeaderSize, char* dest_addr, int dest_port)
{
       *iHeaderSize += strlen(SOAP_HTTP_HOST);
       *iHeaderSize += strlen(dest_addr);
       *iHeaderSize += 1;
       char str_port[10];
       sprintf(str_port,"%d",dest_port);
       *iHeaderSize += strlen(str_port);
       *iHeaderSize += strlen(STR_LINE_DELIMITER);
}

void SOAP_AdjustHttpEndHeaderLength(int bSoapType, int *iHeaderSize,int iContentLength,/*char *url,*/METHODDEFPTR method_to_call, SERVICEDEFPTR service_ptr)
{
       char str_ContentLength[10];
       sprintf(str_ContentLength,"%d",iContentLength);
			 
			 *iHeaderSize += strlen(STR_CONTENT_LENGTH); 
			 *iHeaderSize += strlen(STR_COLON); 			 
			 *iHeaderSize += strlen(STR_SPACE); 			 			 			 
       *iHeaderSize += strlen(str_ContentLength);
       *iHeaderSize += strlen(STR_LINE_DELIMITER);			 
       
       
       if(bSoapType == SOAP_REQ)
       {
	       *iHeaderSize += strlen(STR_SOAP_ACTION);			 
				 *iHeaderSize += strlen(STR_COLON); 			 
				 *iHeaderSize += strlen(STR_SPACE); 			 			 			 			 
	       *iHeaderSize += strlen(QUOTE);
	       
	       if (method_to_call->namespace != NULL)
	       {
	    	   //*iHeaderSize += strlen(url);
	    	   *iHeaderSize += strlen(method_to_call->namespace );
	       }
	       else
	       {
	    	    //*iHeaderSize += strlen(STR_HASH);       
				 //*iHeaderSize += strlen(methodName);
	    	   *iHeaderSize += strlen(service_ptr->interfacesDefinition->namespace);
	    	   *iHeaderSize += strlen(STR_HASH);       
	    	   *iHeaderSize += strlen(method_to_call->name);
	       }
		
	       *iHeaderSize += strlen(QUOTE);			 
	       *iHeaderSize += strlen(STR_LINE_DELIMITER);       
	    
       }
       *iHeaderSize += strlen(STR_LINE_DELIMITER);
}

/*
int main()
{
       wsdcomposer_wsd_reply();
       return 0;
}
*/ 
