#include "composer/composer.h"
#include "unit/unit.h"
#include "protocol/ssdpmsg.h"

#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>
#include "monitor/tools.h"
#include "composer/upnpdevice_composer.h"

MSG_PTR upnp_servicedescr_composer_ssdp_request(SSDPMSG_PTR ssdpmsg)
{

	
	MSG_PTR request_msg = msg_create("", 0, get_host_ip()/*"192.168.2.2"*/, 1000, ssdpmsg_get_dest_addr(ssdpmsg), ssdpmsg_get_dest_port(ssdpmsg), NULL);	

	
	request_msg->length = 0;
	
	// calculate length first line
	char* method = ssdpmsg_get_method(ssdpmsg);
	char* version = ssdpmsg_get_version(ssdpmsg);
	char* uri =	ssdpmsg_get_uri(ssdpmsg);
	
	fprintf(stdout, "upnp_servicedescr_composer_ssdp_request: method=%s(%d) uri = %s\n",method,strlen(method),uri);
	
	
	request_msg->length += strlen(method) + strlen(version) + strlen (uri) + 2 /* 2 chars for 2 spaces to divide tokens */ + 1 /* for \n */;  
	
	
	//	calculate length headers
	SSDP_HEADER_PTR headers = NULL;
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
		request_msg->length += strlen(headers->name) + strlen(headers->value) + 2 /* ": " */ +1 /* \n */;
		
	// for final \0 
	request_msg->length += 1; 
	// for final \n 
	request_msg->length += 1; 
	
	request_msg->data = malloc(request_msg->length);
	bzero (request_msg->data , request_msg->length);
	
	// first line
	strncpy(request_msg->data, method, strlen(method));
	strncat(request_msg->data, " ", 1);
	strncat(request_msg->data, uri, strlen(uri));
	strncat(request_msg->data, " ", 1);
	strncat(request_msg->data, version, strlen(version));
	strncat(request_msg->data, "\n", 1);
	
	// add headers
	for (headers = ssdpmsg_get_headers(ssdpmsg); headers != NULL; headers = headers->next)
	{
		strncat(request_msg->data, headers->name, strlen(headers->name));
		strncat(request_msg->data, ": ", 2);
		strncat(request_msg->data, headers->value, strlen(headers->value));
		strncat(request_msg->data, "\n", 1);
	}
	strncat(request_msg->data, "\0", 1);

	strncat(request_msg->data, "\n", 1);

	return request_msg;
}

void *upnp_servicedescr_run_composer(void* ptr)
{

	COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;
	
	EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
	COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;
	
	
	
	

	
	// Add the identifier of the thread composer to the eventqueue_table 
	pthread_t thread_composer = pthread_self();
	//entry->composer_evtlistener_thread = (void*)thread_composer;
//	eventqueue_table_add_composer_thread(entry, thread_composer);
//	// if there are other composers currently active on the entry, wait for the last one to terminate
//	if (eventqueue_table_get_composer_thread_list_length(entry) > 1)
//	{
//		fprintf(stdout,"\n\n\nupnp_servicedescr_run_composer: WAIT FOR THREAD %p TO END: LIST = %d\n\n\n\n", eventqueue_table_get_first_composer_thread(entry), eventqueue_table_get_composer_thread_list_length(entry));
//		pthread_join(eventqueue_table_get_first_composer_thread(entry), NULL);
//	}
//	else
//		fprintf(stdout,"\n\n\nupnp_servicedescr_run_composer: NOT WAIT FOR THREAD TO END: LIST = %d\n\n\n", eventqueue_table_get_composer_thread_list_length(entry));

	
	
	//pthread_t thread_event_src = pthread_self();
	//fprintf(stdout, "\nSLP PTHREAD composer = %p\n", thread_event_src);
	
	//slp_parse_msg(data->parser, data->msg->data, data->msg->length, thread_event_src);
	
	//eventqueue_table_print(eventqueue_table);
	//fprintf(stdout,"table contains 13,456 = %d\n", eventqueue_table_contain_entry(eventqueue_table, 13,456));
	
	//char* reply_dest_addr;
	//int reply_dest_port;
	
	

	
	
	pthread_mutex_lock(&(entry->mutex));
	
	fprintf(stdout, "upnp_servicedescr_run_composer: (test length) eventqueue_table_get_composer_thread_list_length(entry=%p events size=%d) = %d\n", entry,entry->event_queue->size,eventqueue_table_get_composer_thread_list_length(entry));
	
	if (eventqueue_table_get_composer_thread_list_length(entry) > 1)
	{
		
		// get first composer thread. if different from current compsoer thread, thsi thread must lock fro the mutex of the last 
		// thread of composer_thread_list
		COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_get_first_composer_thread(entry);		
	
		fprintf(stdout, "upnp_servicedescr_run_composer: first  = %s (entry=%p)\n", first_composer_thread->composer_listener_string, entry);
		
		
		if (first_composer_thread->composer != composer)
		{
			// get last composer thread. ITs reference is used to lock current compsoer thread until the end
			// of the previous composer thread
			COMPOSERTHREAD_LIST_ENTRY_PTR previous_composer_thread  = eventqueue_table_get_secondlast_composer_thread(entry);
			
			fprintf(stdout, "upnp_servicedescr_run_composer: BEFORE LOCKING on cond var %p\n", &(previous_composer_thread->cond));
	/*		
			struct timeval now;
            struct timespec timeout;
            
            gettimeofday(&now);
            timeout.tv_sec = now.tv_sec + 5;
            timeout.tv_nsec = now.tv_usec * 1000;
            pthread_cond_timedwait(&(previous_composer_thread->cond), &(entry->mutex), &timeout);
      */      

			
			
			pthread_cond_wait(&(previous_composer_thread->cond), &(entry->mutex));
			// lock this thread
			//int lock = eventqueue_table_composer_thread_lock_entry_mutex(previous_composer_thread);
			fprintf(stdout, "upnp_servicedescr_run_composer: AFTER LOCKING on cond var  %p \n",&(previous_composer_thread->cond));
		} 
			
	}
//	pthread_mutex_unlock(&(entry->mutex));
	
	
	//SSDPMSG_PTR ssdpmsg = ssdpmsg_create();
	
	
	//char* location_url_nopath;
	//char* location_url_path;
	
	UPNP_SERVICE_DISCOVER_TABLE services_table;
	upnp_service_discover_table_init(&services_table);
	
	while (1)
	{
		EVENT_PTR evt = eventqueue_table_get_event(entry);
		if (evt != NULL)
		{
			fprintf(stdout,"upnp_servicedescr_run_composer (pthread=%p): evt is type=%d, src=%p\n", thread_composer, evt->type,evt->stream_id);
			event_print(evt);  
		
			
			
			switch (evt->type) 
			{
			case SDP_SERVICE_DESCR_URL_NOPATH:
				if (upnp_service_discover_table_contains_stream_id(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/))
					upnp_service_discover_table_set_location_url_nopath(&services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/, evt->value.value_string);
				else
				{
					upnp_service_discover_table_add_entry(&services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/, evt->value.value_string, NULL);
				}
				//location_url_nopath = strdup(evt->value.value_string);
				break;
				
			case SDP_INTERFACE_DESCR_URL_PATH:
				if (upnp_service_discover_table_contains_stream_id(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/))
				{
					upnp_service_discover_table_set_location_url_path(&services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/, evt->value.value_string);
					char* tmp_location_url_path = upnp_service_discover_table_get_location_url_path(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/);
					fprintf(stdout,"SDP_INTERFACE_DESCR_URL_PATH  tmp_location_url_path=%s\n",tmp_location_url_path);
				}
				else
				{
					upnp_service_discover_table_add_entry(&services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/, NULL, evt->value.value_string);
				}
				//location_url_path = strdup(evt->value.value_string);
				break;
				
				
			case SDP_STOP:	
				{
				
				if (upnp_service_discover_table_contains_stream_id(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/))
				{
				//if (location_url_path != NULL)
				//{
					fprintf(stdout, "upnp_servicedescr_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
	
	
					COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
					
					// set current_socket_reply_port for ssdp parser to find the right entry on the table
					entry->current_socket_reply_port = first_composer_thread->socket_reply_port;	
					
					char* location_url_nopath = upnp_service_discover_table_get_location_url_nopath(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/);
					char* location_url_path = upnp_service_discover_table_get_location_url_path(services_table, (int)((EVENTSRC_PTR)evt->src)->src_thread_parser/*evt->stream_id*/);
					
					fprintf(stdout,"upnp_devicedescr_run_composer (location_url_nopath=%s,location_url_path=%s)\n", location_url_nopath , location_url_path );
					
					
					if (location_url_nopath != NULL && strcmp(location_url_nopath,"") && 
							location_url_path != NULL && strcmp(location_url_path,""))
					{
					
						
						//char* location_service_descr = malloc((strlen(location_url_nopath)+strlen(location_url_path)+1)); 
						//strncpy(location_service_descr, location_url_nopath, strlen(location_url_nopath));
						//strcat(location_service_descr, location_url_path);
		
						SSDPMSG_PTR ssdpmsg = ssdpmsg_create();
						
						ssdpmsg_set_method(ssdpmsg, "GET");
						ssdpmsg_set_version(ssdpmsg, "HTTP/1.1");
						ssdpmsg_set_uri(ssdpmsg, location_url_path);
						//ssdpmsg_add_header(ssdpmsg, "HOST", "128.93.8.23:4004" /*location_url_nopath*/);
						ssdpmsg_add_header(ssdpmsg, "Accept-Language", "en-us,en;q=0.5");
		
						
						char* host;
						int port;
						upnp_devicedescr_composer_address_get_host_port(location_url_nopath, &host, &port);
						fprintf(stdout, "upnp_servicedescr_composer_address_get_host_port: host=%s port=%d\n", host, port);
		
						
						ssdpmsg_set_dest_addr(ssdpmsg, host /*"128.93.8.23"*/);
						ssdpmsg_set_dest_port(ssdpmsg, port /*4004*/);
		
						char host_header[50];
						sprintf(host_header, "%s:%d",host,port);
						ssdpmsg_add_header(ssdpmsg, "HOST", host_header);
						
						//ssdpmsg_set_dest_addr(ssdpmsg, "192.168.2.2");
						//ssdpmsg_set_dest_port(ssdpmsg, 38007);
						
						MSG_PTR msg = upnp_servicedescr_composer_ssdp_request(ssdpmsg);
						
						
						
						
						int i;
						for (i=0; i<msg->length; i++)
								fprintf(stdout, "%c",msg->data[i]&0xff,msg->data[i]&0xff);
						fprintf(stdout, "\n\n",msg->length);
						
						composer_publish_msg(msg, composer);
					
						
						pthread_cond_signal(&(first_composer_thread->cond));
						
		
						fprintf(stdout, "upnp_servicedescr_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
						
						
						pthread_mutex_unlock(&(entry->mutex));
						
						fprintf(stdout, "upnp_servicedescr_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
					}
					return;
				}
				
				}
				break;

			//case SDP_SERVICE_CTRL_URL_NOPATH, location_url_nopath, evt_stream_id, (void*)evt_src), parser);
			//	break;
			}
		    
			

		}
		usleep(10);
	}
}

void upnp_servicedescr_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout,"UPNPSERVICE");
	composer_event_received(evt, node);
}



COMPOSER_PTR upnp_servicedescr_composer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creting upnp_servicedescr composer\n");
		
	
	COMPOSER_PTR c = composer_create(u);

	c->run_composer = upnp_servicedescr_run_composer;
	c->factory = upnp_servicedescr_composer_create;

	c->func_event_received = upnp_servicedescr_composer_event_received;
	return c;	
}
