#include "composer/composer.h"
#include "unit/unit.h"
#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>

#include "event/servicedef.h"

#include "monitor/tools.h"
#include "composer/proxy_composer.h"

char* upnpproxy_create_devicedescription(SERVICEDEFPTR service_ptr)
{
	FILE *device_file;
	char *buffer;
	int n;
	
	char device_filename[50];
	sprintf(device_filename, "/device%d.xml", service_ptr->indexService);
	char filename[255];
	sprintf(filename,"%s%s%s", get_httpserver_path(), get_httpserver_urlpath(), device_filename);
	device_file = fopen(filename, "w");
	
	if(device_file==NULL) {
	  printf("Error: can't create file %s for writing.\n",filename);
	}
	else {
	              
		fprintf(device_file,"<root xmlns=\"urn:schema-upnp-device-org:device-1-0\">\n");
		fprintf(device_file,"    <specVersion>\n");
		fprintf(device_file,"       <major>1</major>\n");
		fprintf(device_file,"       <minor>0</minor>\n");
		fprintf(device_file,"    </specVersion>\n");
		fprintf(device_file,"    <URLBase>http://%s:%d</URLBase>\n", get_host_ip(), get_httpserver_port());
		fprintf(device_file,"    <device>\n");
		fprintf(device_file,"         <deviceType>urn:schemas-upnp-org:device:%s:1</deviceType>\n", service_ptr->name);
		fprintf(device_file,"         <friendlyName>AmigoUPnPProxy</friendlyName>\n");
		fprintf(device_file,"         <manufacturer>Amigo</manufacturer>\n");
		fprintf(device_file,"         <manufacturerURL>http://www.hitech-projects.com/euprojects/amigo/</manufacturerURL>\n");
		//fprintf(device_file,"         <modelDescription>UPnP</modelDescription>\n");
		//fprintf(device_file,"         <modelName>Plug</modelName>\n");
		//fprintf(device_file,"         <modelNumber>1.0</modelNumber>\n");
		//fprintf(device_file,"         <modelURL>http://ikerlan.es</modelURL>\n");
		//fprintf(device_file,"         <serialNumber>123456789</serialNumber>\n");
		//fprintf(device_file,"         <UDN>uuid:Oscar - UPnP demo</UDN>\n");
		//fprintf(device_file,"         <UPC>123456789</UPC>\n");
		fprintf(device_file,"         <serviceList>\n");
		
		
		INTERFACEDEFPTR interface_tmp = service_ptr->interfacesDefinition;
    	for (; interface_tmp != NULL; interface_tmp = interface_tmp->next)
    	{
			fprintf(device_file,"            <service>\n");
			fprintf(device_file,"                  <serviceType>urn:schemas-upnp-org:service:%sService:1</serviceType>\n",interface_tmp->name);
			fprintf(device_file,"                  <serviceId>urn:upnp-org:serviceId:%sService:1</serviceId>\n",interface_tmp->name);
			fprintf(device_file,"                  <SCPDURL>%s/%s_servicedescr%d.xml</SCPDURL>\n",get_httpserver_urlpath(), interface_tmp->name,service_ptr->indexService);
			

			
			char* uri_iface_path = service_ptr->interfacesDefinition->uriPath;
			fprintf(stdout, "\nuri_iface_path=%s\n",uri_iface_path);
			//fprintf(device_file,"                  <controlURL>http://192.168.2.2:10000/%s/control</controlURL>\n",interface_tmp->name);
				if (uri_iface_path == NULL)
					fprintf(device_file,"                  <controlURL>http://%s:%d/%s/control</controlURL>\n",service_ptr->host, service_ptr->port, interface_tmp->name);
				else
					fprintf(device_file,"                  <controlURL>http://%s:%d%s</controlURL>\n",service_ptr->host, service_ptr->port, uri_iface_path);
					
			//fprintf(device_file,"                  <eventSubURL>/tmp/%s/event</eventSubURL>\n",interface_tmp->name);
			fprintf(device_file,"            </service>\n");
    	}

		fprintf(device_file,"         </serviceList>\n");
		fprintf(device_file,"    </device>\n");
		fprintf(device_file,"</root>\n");
		                                
	
	  fclose(device_file);
	}
	return strdup(device_filename);
}

char* upnpproxy_mapping_JavaType2UPnP(char* java_type)
{
	if (!strcmp("I", java_type))
		return "ui2";
	else if (!strcmp("Ljava/lang/String;", java_type))
		return "string";
		
}

int upnpproxy_create_servicedescription(INTERFACEDEFPTR interface_ptr, SERVICEDEFPTR service_ptr) {

	FILE *service_file;
	
	char service_filename[100];
	sprintf(service_filename, "%s%s/%s_servicedescr%d.xml",get_httpserver_path(), get_httpserver_urlpath(), interface_ptr->name, service_ptr->indexService);
	service_file = fopen(service_filename, "w");
	
	if(service_file == NULL) {
	  printf("Error: can't create file %s for writing.\n", service_filename);
	}
	else {
		
		

	/*HashMap serviceStateTable = new HashMap();
	
	int indexService = serviceDef.getIndexService();
	
    try {
        PrintWriter out = createFile("",interfaceDef.getName()+"_servicedescr"+indexService+".xml");
*/
        fprintf(service_file,"<scpd xmlns=\"urn:schema-upnp-device-org:service-1-0\">\n"); 
        fprintf(service_file,"    <specVersion>\n"); 
        fprintf(service_file,"       <major>1</major>\n"); 
   		fprintf(service_file,"       <minor>0</minor>\n"); 
        fprintf(service_file,"    </specVersion>\n"); 
        
        METHODDEFPTR methods = interface_ptr->methods;
        
        int index_method = 0;
        if (methods != NULL) {

        	
        	// count number of arguments to allocate array to store relatedstatevars
        	
        	int count_related_state_vars = 0;
        	METHODDEFPTR methods_tmp = methods;
        	for (; methods_tmp != NULL; methods_tmp = methods_tmp->next)
        	{
        		ARGUMENTDEFPTR args_tmp = methods_tmp->args;
        		for (; args_tmp != NULL; args_tmp = args_tmp->next) 
        			count_related_state_vars++;
        	}
        	// related_state_vars_and_types: even positions contain the name of the variable, and
        	//	odd postitions contain the type of the variable. 
        	//   ex: related_state_vars_and_types[0] = "x"
        	//   ex: related_state_vars_and_types[1] = "int"
        	char** related_state_vars_and_types = (char**)malloc((count_related_state_vars*2)*sizeof(char*)); 
        	
        	int index_related_state_vars = 0;
        	
            fprintf(service_file,"    <actionList>\n");    
            
            // for each method in the interface, add an action tag
	        for (; methods != NULL; methods=methods->next) {
	            fprintf(service_file,"    <action>\n"); 
				fprintf(service_file,"        <name>%s</name>\n",methods->name); 
				
				ARGUMENTDEFPTR args = methods->args;
		        
		        if (args != NULL) {
					

		        	
					fprintf(service_file,"        <argumentList>\n"); 
					int index_arg = 0;
					
					for (; args != NULL; args = args->next) {
						fprintf(service_file,"        <argument>\n"); 
						fprintf(service_file,"            <name>%s</name>\n",args->name); 
						if (args->dir == ARG_IN)
							fprintf(service_file,"            <direction>in</direction>\n"); 
						else 
							fprintf(service_file,"            <direction>out</direction>\n"); 
					
						// relatedStateVariable is generated as follows:
						// methodName+methodIndex+?_?+argName+argIndex to prevent 2 relatedStateVariables from
						// having the same name
						char relatedStateVariable[100];
						sprintf(relatedStateVariable, "%s%d_%s%d",methods->name, index_method, args->name, index_arg);					
						
						
						fprintf(stdout, "%s / %d",relatedStateVariable, index_related_state_vars*2);
						related_state_vars_and_types[index_related_state_vars*2] = (char*)strdup(relatedStateVariable);
						related_state_vars_and_types[index_related_state_vars*2+1] = (char*)strdup(args->datatype);
						index_related_state_vars++;
/*							
 * 
						String relatedStateVariable_Type = actionList[i].getArgDataType(j);
						
						// relatedStateVariable added in hashmap: they will be added to the xml file
						// in the following step
						serviceStateTable.put(relatedStateVariable, relatedStateVariable_Type);
*/						
						fprintf(service_file,"            <relatedStateVariable>%s</relatedStateVariable>\n",relatedStateVariable); 
						fprintf(service_file,"        </argument>\n"); 
						index_arg++;
					}
					fprintf(service_file,"        </argumentList>\n"); 
				}
				fprintf(service_file,"    </action>\n"); 
				
				index_method++;
	        }
            fprintf(service_file,"    </actionList>\n");     	        
	        
	
	        // relatedStateVariables are added to the xml file
	        fprintf(service_file,"    <serviceStateTable>\n"); 
	        
	        for (index_related_state_vars = 0; index_related_state_vars < count_related_state_vars; index_related_state_vars++)
	        {
				fprintf(service_file,"    <stateVariable>\n"); 
				fprintf(service_file,"        <name>%s</name> <sendEventsAttribute>no</sendEventsAttribute>\n",related_state_vars_and_types[index_related_state_vars*2]); 
				fprintf(service_file,"        <dataType>%s</dataType>\n",upnpproxy_mapping_JavaType2UPnP(related_state_vars_and_types[index_related_state_vars*2+1])); 
				fprintf(service_file,"    </stateVariable>\n"); 
	        }	
	        fprintf(service_file,"    </serviceStateTable>\n"); 
        
        }
        fprintf(service_file,"</scpd>\n"); 
                                             
  	  fclose(service_file);
	}	
}




void *upnpproxy_run_composer(void* ptr)
{
	
	COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;
	
	EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
	COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;
	
	pthread_mutex_lock(&(entry->mutex));	
	
	
	// Add the identifier of the thread composer to the eventqueue_table 
	pthread_t thread_composer = pthread_self();
	
/*	
	SERVICEDEFPTR service_ptr = servicedef_new();
	INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);
	servicedef_add_interface(service_ptr, interface_ptr);
*/	
	
	
	// initialized at default Amigo Mdw values (HTTP/SOAP unit running at this address)
	// if the service is based on SOAP for communication, these values will be replaced
	// with the values of host/port of the real service
	
	
/*	
	servicedef_set_host(service_ptr, get_host_ip());
	servicedef_set_port(service_ptr, get_soapproxy_port());
	
	fprintf(stdout,"upnpproxy_run_composer: entry=%p starts. number of events=%d\n",entry, entry->event_queue->size);
	
	long long num;
	long unique;
	long long time;
	short count;
	long port;
*/	
	
	PROXY_SERVICE_DISCOVER_TABLE services_table;
	proxy_service_discover_table_init(&services_table);
	
	
//int already_written = 0;	
	while (1) 				
	{
		//fprintf(stdout,"upnpproxy_run_composer: before get evt");
		EVENT_PTR evt = eventqueue_table_get_event(entry);
		//fprintf(stdout,"upnpproxy_run_composer: after get evt");
			
		if (evt != NULL)
		{
			fprintf(stdout,"upnpproxy_run_composer: evt is type=%d, src=%p\n", evt->type,evt->stream_id);
			event_print(evt);  
			
			switch (evt->type) 
			{
				case SDP_SERVICE_DESCR_URL_NOPATH:
				case SDP_SERVICE_CTRL_URL_NOPATH:
				{
					char *ip, *url;
					int port;
					parseURL_host_port(evt->value.value_string, &ip, &port, &url);
					fprintf(stdout,"upnpproxy_run_composer: ip=%s, port=%d, url=%s\n", ip, port, url);
					
					SERVICEDEFPTR service_ptr;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						
						servicedef_set_host(service_ptr, ip);
						servicedef_set_port(service_ptr, port);
						
					}
					else
					{
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						
						servicedef_set_host(service_ptr, ip);
						servicedef_set_port(service_ptr, port);
					}
					//servicedef_set_host(service_ptr, ip);
					//servicedef_set_port(service_ptr, port);
				}
					break;
				
					
				
				case SDP_SERVICE_METHOD_DESCRIPTION:
				{
					
					SERVICEDEFPTR service_ptr;
					INTERFACEDEFPTR interface_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
					}
					
					
					EVENT_METHOD_DEF_PTR event_methoddef = evt->value.value_void_pointer;
					
				    METHODDEFPTR methoddef = event_methoddef->method;
				    methoddef_print(methoddef);
				    
				    if (!service_is_new_entry)
				    {
				    	if (!servicedef_contains_method(service_ptr, methoddef))
				    	{
				    		interface_ptr = service_ptr->interfacesDefinition;
				    		interfacedef_add_method(interface_ptr, methoddef);
						    servicedef_set_name(service_ptr, event_methoddef->service_name);
						    interfacedef_set_name(interface_ptr, event_methoddef->interface_name);
				    	}
				    }
				    else
				    {
				    	interface_ptr = service_ptr->interfacesDefinition;
				    	interfacedef_add_method(interface_ptr, methoddef);
					    servicedef_set_name(service_ptr, event_methoddef->service_name);
					    interfacedef_set_name(interface_ptr, event_methoddef->interface_name);
				    }
					

					/*					
					EVENT_METHOD_DEF_PTR event_methoddef = evt->value.value_void_pointer;
					
				    METHODDEFPTR methoddef = event_methoddef->method;
				    methoddef_print(methoddef);
				    
				    interfacedef_add_method(interface_ptr, methoddef);
				    
				    servicedef_set_name(service_ptr, event_methoddef->service_name);
				    interfacedef_set_name(interface_ptr, event_methoddef->interface_name);
				    */
		    
				}
				break;

				
				case SCP_RMIOBJ_NUM:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjNum(service_ptr, evt->value.value_longlong);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjNum(service_ptr, evt->value.value_longlong);
					}
					//num = evt->value.value_longlong;
					fprintf(stdout, "SCP_RMIOBJ_NUM value = %llu\n",evt->value.value_longlong);
				}
					break;
				case SCP_RMIOBJ_UNIQUE:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjUnique(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjUnique(service_ptr, evt->value.value_int);
					}
					//unique = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_UNIQUE value = %d\n",evt->value.value_int);
				}
					break;
				case SCP_RMIOBJ_TIME:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjTime(service_ptr, evt->value.value_longlong);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjTime(service_ptr, evt->value.value_longlong);
					}
					//time = evt->value.value_longlong;
					fprintf(stdout, "SCP_RMIOBJ_TIME value = %llu\n",evt->value.value_longlong);
				}
					break;
				case SCP_RMIOBJ_COUNT:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjCount(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjCount(service_ptr, evt->value.value_int);
					}
					//count = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_COUNT value = %d\n",evt->value.value_int);
				}
					break;
					
				/*case SCP_RMIOBJ_IP:
				{
					fprintf(stdout, "SCP_RMIOBJ_IP value = %s\n",evt->value.value_string);
				}
					break;
					*/
				case SCP_RMIOBJ_PORT:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						servicedef_set_RMI_ObjPort(service_ptr, evt->value.value_int);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = interfacedef_new(service_ptr);;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						servicedef_set_RMI_ObjPort(service_ptr, evt->value.value_int);
					}
					//port = evt->value.value_int;
					fprintf(stdout, "SCP_RMIOBJ_PORT value = %d\n",evt->value.value_int);
				}
					break;
				
				
				case SDP_INTERFACE_CTRL_URL_PATH:
				{
					SERVICEDEFPTR service_ptr;
					int service_is_new_entry = 1;
					
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						service_is_new_entry = 0;
						service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);
						INTERFACEDEFPTR interface_ptr = service_ptr->interfacesDefinition;
						interfacedef_set_uripath(interface_ptr, evt->value.value_string);
					}
					else
					{
						service_is_new_entry = 1;
						
						service_ptr = servicedef_new();
						//service_ptr->indexService = 0;
						INTERFACEDEFPTR interface_ptr = service_ptr->interfacesDefinition;
						servicedef_add_interface(service_ptr, interface_ptr);
						
						proxy_service_discover_table_add_entry(&services_table, evt->stream_id, service_ptr);
						interfacedef_set_uripath(interface_ptr, evt->value.value_string);
					}
				}
					
					
				case GENERATE_PROXY:
				{
					if (proxy_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
					
						
						
						SERVICEDEFPTR service_ptr = proxy_service_discover_table_get_service_by_stream_id(services_table, evt->stream_id);

						char* uri_iface_path = service_ptr->interfacesDefinition->uriPath;
						fprintf(stdout, "\nuri_iface_path=%s\n",uri_iface_path);
						
						if (service_ptr->host == NULL)
							servicedef_set_host(service_ptr, get_host_ip());
						if (service_ptr->port == -1)
							servicedef_set_port(service_ptr, get_soapproxy_port());
						
						char* device_filename = upnpproxy_create_devicedescription(service_ptr);
						char url[255];
						sprintf(url, "http://%s:%d%s%s", get_host_ip(), get_httpserver_port(), get_httpserver_urlpath(), device_filename);
						
						composer_publish_event(event_create_string(SDP_SERVICEPROXY_URL_CTRL, url /*"http://192.168.2.2:8080/tmp/device1.xml"*/, evt->stream_id, (void*)evt->src), composer);
						
						
					    INTERFACEDEFPTR interface_ptr_tmp;
					    for (interface_ptr_tmp = service_ptr->interfacesDefinition; interface_ptr_tmp != NULL; interface_ptr_tmp=interface_ptr_tmp->next)
					    {
					    	upnpproxy_create_servicedescription(interface_ptr_tmp, service_ptr);
					    }
					    
					    /* Adds the service definition to the table that associates servicedefinitions to 
						the related path used in SOAP message calls  in POST of the HTTP HEADER.
	
					       Each RMI service has got only one interface, so the path for the service is:
					       "/"+unique_interface_name+"/control" */
					    INTERFACEDEFPTR iface = service_ptr->interfacesDefinition;
					    if (iface != NULL) {
					    	char path1[255] ="/";
					    	strcat(path1, iface->name);
					    	strcat(path1, "/control");
					/*	
							// add RMI info to service description
							servicedef_set_RMI_ObjNum(service_ptr, num);
							servicedef_set_RMI_ObjUnique(service_ptr, unique);
							servicedef_set_RMI_ObjTime(service_ptr, time);
							servicedef_set_RMI_ObjCount(service_ptr, count);
							servicedef_set_RMI_ObjPort(service_ptr, port);
					*/	
					    	Table_add_PathServiceDef_Elem( &ServiceDefinitionTable, path1, service_ptr);
					    	
						    SERVICEDEFPTR srvTmp = Table_get_ServiceDef(ServiceDefinitionTable, path1);
						    
						    if (srvTmp != NULL)
						    	fprintf(stdout,"\nService %s Found: %s\n",path1, srvTmp->name);
						    else
						    	fprintf(stdout,"\nService %s NOT Found\n", path1);
					    }
	/*
					    char* path2 = "path2";
					    Table_add_PathServiceDef_Elem( &ServiceDefinitionTable, path2, service_ptr);
					    char* path3 = "path3";
					    Table_add_PathServiceDef_Elem( &ServiceDefinitionTable, path3, service_ptr);
					    
					    char* path4 = "/washer_rmislp.WasherImpl/control";
					    */

					}
				}
				    break;

			}
				
		}
		usleep(10);
	}
	
}


void upnpproxy_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout,"UPNPPROXY");
	composer_event_received(evt, node);
}

COMPOSER_PTR upnpproxy_composer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating UPNPPROXY composer\n");
		
	COMPOSER_PTR c = composer_create(u);

	c->run_composer = upnpproxy_run_composer;
	c->factory = upnpproxy_composer_create;

	c->func_event_received = upnpproxy_composer_event_received;
	return c;	
}
