#include "composer/composer.h"
#include "composer/wsdcomposer.h"
#include "unit/unit.h"
#include "protocol/wsdmsg.h"
#include "protocol/wsd.h"
#include "io/io.h"
#include <stdio.h>
#include "event/eventqueue_table.h"
#include <pthread.h>	

char *ELEMENT_PREPRECESSOR ="<?xml version=\"1.0\" encoding=\"utf-8\"?>";
char *ELEMENT_ENVELOPE_START ="<s:Envelope xmlns:a=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\" xmlns:d=\"http://schemas.xmlsoap.org/ws/2005/04/discovery\" xmlns:s=\"http://www.w3.org/2003/05/soap-envelope\">"; 
char *ELEMENT_ENVELOPE_END = "</s:Envelope>";
char *ELEMENT_HEADER_START = "<a:Header>";
char *ELEMENT_HEADER_END = "</a:Header>";
char *ELEMENT_BODY_START = "<s:Body>";
char *ELEMENT_BODY_END = "</s:Body>";

char *ELEMENT_ACTION_HELLO = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Hello</a:Action>";
char *ELEMENT_ACTION_BYE = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Bye</a:Action>";
char *ELEMENT_ACTION_PROBE = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Probe</a:Action>";
char *ELEMENT_ACTION_PROBE_MATCH = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/ProbeMatches</a:Action>";
char *ELEMENT_ACTION_RESOLVE = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Resolve</a:Action>";
char *ELEMENT_ACTION_RESOLVE_MATCH = "<a:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/ResolveMatches</a:Action>";

char *ELEMENT_MESSAGEID_START = "<a:MessageID>";
char *ELEMENT_MESSAGEID_DATA = "xs:anyURI";
char *ELEMENT_MESSAGEID_END = "</a:MessageID>";

char *ELEMENT_TO = "<a:To>urn:schemas-xmlsoap-org:ws:2005:04:discovery</a:To>";

char *ELEMENT_RELATESTO_START_EMPTY ="<a:RelatesTo>";

char *ELEMENT_RELATESTO_START ="<a:RelatesTo ";
char *ELEMENT_RELATESTO_CLOSE =">";
char *ELEMENT_RELATESTO_RELTYPE = "RelationshipType=";
char *ELEMENT_RELATESTO_RELTYPE_VAL = "\"d:Suppression\"";
char *ELEMENT_RELATESTO_VAL = "xs:anyURI";
char *ELEMENT_RELATESTO_END = "</a:RelatesTo>";

char *ELEMENT_APPSEQ_START = "<d:AppSequence ";
char *ELEMENT_APPSEQ_INSTANCEID = "InstanceId=";
char *ELEMENT_APPSEQ_INSTANCEID_VAL = "\"1077004800\"";
char *ELEMENT_APPSEQ_MSGNUM = " MessageNumber=";
char *ELEMENT_APPSEQ_MSQNUM_VAL= "\"1\"";
char *ELEMENT_APPSEQ_END = "/>";

char *ELEMENT_REPLYTO_START = "<a:ReplyTo>";
char *ELEMENT_REPLYTO_END = "</a:ReplyTo>";

char *ELEMENT_ENDPOINTREF_START = "<a:EndpointReference>";
char *ELEMENT_ENDPOINTREF_END = "</a:EndpointReference>";  

char *ELEMENT_ADDR_START = "<a:Address>";
char *ELEMENT_ADDR_END = "</a:Address>";
char *ELEMENT_ADDR = "uuid:98190dc2-0890-4ef8-ac9a-5940995e6119";

char *ELEMENT_METADATAVER_START = "<d:MetadataVersion>";
char *ELEMENT_METADATAVER_END = "</d:MetadataVersion>";

char *ELEMENT_METADATAVER_VAL = "75965";

char *ELEMENT_TYPES_START = "<d:Types>";
char *ELEMENT_TYPES_VAL = "xs:QName";
char *ELEMENT_TYPES_END = "</d:Types>";

char *ELEMENT_SCOPES_START = "<d:Scopes ";
char *ELEMENT_SCOPES_MATCHBY = "MatchBy=";
char *ELEMENT_SCOPES_MATCHBY_VAL = "\"http://schemas.xmlsoap.org/ws/2005/04/discovery/ldap\"";
char *ELEMENT_SCOPES_CLOSE = ">";
char *ELEMENT_SCOPES_VAL = "xs:AnyURI";
char *ELEMENT_SCOPES_END = "</d:Scopes>";

char *ELEMENT_XADRS_START = "<d:XAddrs>";
char *ELEMENT_XADRS_VAL = "xs:AnyURI";
char *ELEMENT_XADRS_END = "</d:XAddrs>";

char *ELEMENT_HELLO_START = "<d:Hello>";
char *ELEMENT_HELLO_END = "</d:Hello>";

char *ELEMENT_BYE_START = "<d:Bye>";
char *ELEMENT_BYE_END = "</d:Bye>";

char *ELEMENT_PROBE_START = "<d:Probe>";
char *ELEMENT_PROBE_END = "</d:Probe>";

char *ELEMENT_PROBEMATCHES_START = "<d:ProbeMatches>";
char *ELEMENT_PROBEMATCHES_END = "</d:ProbeMatches>";

char *ELEMENT_PROBEMATCH_START = "<d:ProbeMatch>";
char *ELEMENT_PROBEMATCH_END = "</d:ProbeMatch>";

char *ELEMENT_RESOLVE_START = "<d:Resolve>";
char *ELEMENT_RESOLVE_END = "</d:Resolve>";

char *ELEMENT_RESOLVEMATCHES_START = "<d:ResolveMatches>";
char *ELEMENT_RESOLVEMATCHES_END = "</d:ResolveMatches>";

char *ELEMENT_RESOLVEMATCH_START = "<d:ResolveMatch>";
char *ELEMENT_RESOLVEMATCH_END = "</d:ResolveMatch>";

int iAppSegInstanceID = 0;
int iAppSegMessageNum = 0;

char AppSegInstanceID_str[5];
char AppSegMessageNum_str[5];

MSG_PTR wsdcomposer_wsd_reply(WSDMSG_PTR wsdmsg /*,int iMsgType*/)
{
	char *data=NULL;
	int iSize =0;
	int bRelatesToPresent = FALSE, bReplyToPresent = FALSE;	
	int bTypesPresent = FALSE,bScopesPresent = FALSE,bXAddrsPresent = FALSE;

	int iMsgType = wsdmsg_get_message_type(wsdmsg);
		
	if(iMsgType == WSD_PROBE_MATCH_MSG)
	{
		bXAddrsPresent = TRUE;
		bTypesPresent = TRUE;
		bScopesPresent = TRUE;
	}
	else if(iMsgType == WSD_PROBE_MSG)
	{
		bXAddrsPresent = FALSE;
		bTypesPresent = TRUE;
		bScopesPresent = TRUE;
	}
	
	iSize += strlen(ELEMENT_PREPRECESSOR);
	iSize += strlen(ELEMENT_ENVELOPE_START);
	
	// Adjusting Header Length
	iSize += strlen(ELEMENT_HEADER_START);
	AdjustACTLength(iMsgType,&iSize);
	AdjustMSGIDLength(&iSize);
	AdjustTOLength(&iSize);
	AdjustAPPSEQLength(iMsgType,&iSize);
	AdjustRELATESTOLength(wsdmsg,iMsgType,bRelatesToPresent,&iSize);
	AdjustREPLYTOLength(iMsgType,bReplyToPresent,&iSize);	
	iSize += strlen(ELEMENT_HEADER_END);

	// Adjusting Body Length	
	iSize += strlen(ELEMENT_BODY_START);	
	AdjustBodyMainStartElement(iMsgType,&iSize);
	AdjustEndPointReferenceLength(iMsgType,&iSize);
	AdjustTypesLength(wsdmsg,iMsgType,bTypesPresent,&iSize);
	AdjustScopesLength(wsdmsg,iMsgType,bScopesPresent,&iSize);
	AdjustXAddrsLength(wsdmsg,iMsgType,bXAddrsPresent,&iSize);
	AdjustMetaDataVersionLength(iMsgType,&iSize);
	if(iMsgType == WSD_PROBE_MATCH_MSG)
	{
		AdjustProbeMatchLength(wsdmsg,bTypesPresent,bScopesPresent,bXAddrsPresent,&iSize);
	}
	if(iMsgType == WSD_RESOLVE_MATCH_MSG)
	{
		AdjustResolveMatchLength(bTypesPresent,bScopesPresent,bXAddrsPresent,&iSize);
	}
	AdjustBodyMainEndElement(iMsgType,&iSize);		
	iSize += strlen(ELEMENT_BODY_END);	

	iSize += strlen(ELEMENT_ENVELOPE_END);

	data = (char*)malloc(sizeof(char)*(iSize));
	
	strcpy(data,ELEMENT_PREPRECESSOR);
	strcat(data,ELEMENT_ENVELOPE_START);	
	ComposeHeaderElement(wsdmsg,iMsgType,bRelatesToPresent,bReplyToPresent,data);
	ComposeBodyElement(wsdmsg,iMsgType,bTypesPresent,bScopesPresent,bXAddrsPresent,data);
	strcat(data,ELEMENT_ENVELOPE_END);
	
	MSG_PTR reply_msg = msg_create(data,iSize, (char*)get_host_ip(),0, wsdmsg_get_dest_addr(wsdmsg), wsdmsg_get_dest_port(wsdmsg),/*"239.255.255.250", 3702,*/ NULL);	


	fprintf(stdout, "ProbeMatch Lenght is %d\n",iSize );
	
	fprintf(stdout, "REPLY_MSG->DATA = %p\n",&reply_msg->data );
	
	return reply_msg;
}
void *wsd_run_composer(void* ptr)
{
	fprintf(stdout,"wsd_run_composer: -----------------------------------------------\n");
	fprintf(stdout,"wsd_run_composer: -----------------------------------------------\n");
	fprintf(stdout,"wsd_run_composer: -----------------------------------------------\n");
	
	COMPOSETHREADDATA_PTR data = (COMPOSETHREADDATA_PTR)ptr;
	
	EVENTQUEUE_TABLE_ENTRY_PTR entry = (EVENTQUEUE_TABLE_ENTRY_PTR)data->entry_event_queue;
	COMPOSER_PTR composer = (COMPOSER_PTR)data->composer;	
	
	pthread_mutex_lock(&(entry->mutex));
	
	// Add the identifier of the thread composer to the eventqueue_table 
	pthread_t thread_composer = pthread_self();	
	/*
	char* reply_dest_addr;
	int reply_dest_port;
	int bSendProbe = FALSE, bSendProbeMatch = FALSE;
	WSDMSG_PTR wsdmsg = wsdmsg_create();		
	*/
	WSD_SERVICE_DISCOVER_TABLE services_table;
	wsd_service_discover_table_init(&services_table);
	
	//int bSendProbe = FALSE, bSendProbeMatch = FALSE;
	
	while (1)
	{
		EVENT_PTR evt = eventqueue_table_get_event(entry);
		if (evt != NULL)
		{
			fprintf(stdout,"wsd_run_composer: evt is type=%d, src=%p\n", evt->type,evt->stream_id);
			event_print(evt);  
			

				switch (evt->type) 
				{
				case SDP_SERVICE_REQUEST :
				{
					fprintf(stdout,"wsd_run_composer: SDP_SERVICE_REQUEST\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_message_type(wsdmsg, WSD_PROBE_MSG);
				}	
					//fprintf(stdout,"wsd_run_composer: SDP_SERVICE_REQUEST\n");
					//bSendProbe = TRUE;
					break;								
				case SDP_SERVICE_RESPONSE :
				{
					fprintf(stdout,"wsd_run_composer: SDP_SERVICE_RESPONSE\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_message_type(wsdmsg, WSD_PROBE_MATCH_MSG);
				}	
					//bSendProbeMatch = TRUE;
					break;
				case SDP_SOURCE_ADDR :
				{
					fprintf(stdout,"wsd_run_composer: SDP_SOURCE_ADDR : %s\n", evt->value.value_string);
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_dest_addr(wsdmsg, evt->value.value_string);
					//wsdmsg_set_dest_addr(wsdmsg, evt->value.value_string);
				}	
					fprintf(stdout,"wsd_run_composer: SDP_SOURCE_ADDR\n");
					//reply_dest_addr = strdup(evt->value.value_string);
					//wsdmsg_set_dest_addr(wsdmsg, reply_dest_addr);
					break;		
				case SDP_SOURCE_PORT :
				{
					fprintf(stdout,"wsd_run_composer: SDP_SOURCE_PORT : %d \n", evt->value.value_int);
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_dest_port(wsdmsg, evt->value.value_int);
					//wsdmsg_set_dest_addr(wsdmsg, evt->value.value_string);
				}	
					
					//reply_dest_port = evt->value.value_int;
					//wsdmsg_set_dest_port(wsdmsg, reply_dest_port);
					break;				
				/*case SDP_REQ_LANGAGE :
					fprintf(stdout,"wsd_run_composer: SDP_REQ_LANGAGE\n");
					break;*/
				case SDP_REQ_SCOPE :
				{
					fprintf(stdout,"wsd_run_composer: SDP_REQ_SCOPE\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_scopes(wsdmsg, evt->value.value_string);
				}
					
					
					break;	
				/*case SDP_REQ_PREDICAT :
					fprintf(stdout,"wsd_run_composer : SDP_REQ_PREDICATE\n");
					break;	
				case SDP_REQ_REQUESTER :
					fprintf(stdout,"wsd_run_composer: SDP_REQ_REQUESTER\n");
					break;*/	
				case SDP_REQ_SERVICE_TYPE :
				{
					fprintf(stdout,"wsd_run_composer: SDP_REQ_SERVICE_TYPE\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_types(wsdmsg, evt->value.value_string);
				}
					break;	
				case SDP_REQ_ID :
				{
					fprintf(stdout,"wsd_run_composer: SDP_REQ_ID: %s\n",evt->value.value_string);
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_message_id(wsdmsg, evt->value.value_string);
				}
					
					break;	
					/*
				case SDP_VERSION :
					fprintf(stdout,"wsd_run_composer: SDP_VERSION\n");
					break;*/			
				case SDP_SERVICE_DESCR_URL_NOPATH :
				case SDP_SERVICE_CTRL_URL_NOPATH:
				{
					fprintf(stdout,"wsd_run_composer: SDP_SERVICE_DESCR_URL_NOPATH\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_no_url(wsdmsg,evt->value.value_string);
				}
					break;									
				case SDP_SERVICE_DESCR_URL_PATH:
				case SDP_INTERFACE_CTRL_URL_PATH:
				{
					fprintf(stdout,"wsd_run_composer: SDP_SERVICE_DESCR_URL_PATH\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsdmsg_set_url(wsdmsg,evt->value.value_string);
				}
					break;	
				case SDP_SERVICEPROXY_URL_CTRL:
				{
					fprintf(stdout,"wsd_run_composer: SDP_SERVICEPROXY_URL_CTRL\n");
					WSDMSG_PTR wsdmsg;
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
						wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					else
					{
						wsdmsg = wsdmsg_create();
						wsd_service_discover_table_add_entry(&services_table, evt->stream_id, wsdmsg);
					}
					wsd_service_discover_table_set_proxy_url(&services_table, evt->stream_id, evt->value.value_string);
				}
					break;
					
				//case SDP_STOP :	
				case SDP_SEND_SERVICE_REPLY:
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						WSDMSG_PTR wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
						char* proxy_url = wsd_service_discover_table_get_proxy_url(services_table, evt->stream_id);
						
						fprintf(stdout,"wsd_run_composer: SDP_SEND_SERVICE_REPLY\n");
						wsd_PrintWSDMSG(wsdmsg);
						/*
						MSG_PTR msg = NULL;
						
						if(bSendProbe == TRUE)
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MSG);
						}
						else if (bSendProbeMatch == TRUE) 
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MATCH_MSG);
						}
						else if(bSendProbe == FALSE && bSendProbeMatch == FALSE)
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MSG);					
						}
						bSendProbe = FALSE; 
						bSendProbeMatch = FALSE;
						*/
						
						if (proxy_url != NULL)
						{
							char *proxy_url_nopath;
							char *proxy_url_path;
							parseURL_nopath_path(proxy_url, &proxy_url_nopath, &proxy_url_path);
							wsdmsg_set_url(wsdmsg, proxy_url_path);
							wsdmsg_set_no_url(wsdmsg, proxy_url_nopath);
						}
						MSG_PTR msg = wsdcomposer_wsd_reply(wsdmsg);
						
						composer_publish_msg(msg, composer);	
						fprintf(stdout, "wsd_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
		
		
						COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
						
						// set current_socket_reply_port for ssdp parser to find the right entry on the table
						entry->current_socket_reply_port = first_composer_thread->socket_reply_port;				
						
						pthread_cond_signal(&(first_composer_thread->cond));
						
		
						fprintf(stdout, "wsd_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
						
						
						pthread_mutex_unlock(&(entry->mutex));
						
						fprintf(stdout, "wsd_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
						return;								
					}
					break;
					
				case SDP_SEND_SERVICE_REQUEST:
					if (wsd_service_discover_table_contains_stream_id(services_table, evt->stream_id))
					{
						WSDMSG_PTR wsdmsg = wsd_service_discover_table_get_wsdmsg_by_stream_id(services_table, evt->stream_id);
					//	char* proxy_url = wsd_service_discover_table_get_proxy_url(services_table, evt->stream_id);
						
						wsdmsg_set_message_id(wsdmsg, "uuid:072e6e10-e687-425a-ab96-4114f7a94ecd");
						wsdmsg_set_scopes(wsdmsg, "urn:AmigoDemo");
						fprintf(stdout,"wsd_run_composer: SDP_SEND_SERVICE_REQUEST\n");
						
						
						wsdmsg_set_dest_addr(wsdmsg, "239.255.255.250");
						wsdmsg_set_dest_port(wsdmsg, 3702);
						wsd_PrintWSDMSG(wsdmsg);
						
						
					
						/*
						MSG_PTR msg = NULL;
						
						if(bSendProbe == TRUE)
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MSG);
						}
						else if (bSendProbeMatch == TRUE) 
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MATCH_MSG);
						}
						else if(bSendProbe == FALSE && bSendProbeMatch == FALSE)
						{
							msg = wsdcomposer_wsd_reply(wsdmsg,WSD_PROBE_MSG);					
						}
						bSendProbe = FALSE; 
						bSendProbeMatch = FALSE;
						*/
				/*		
						if (proxy_url != NULL)
						{
							char *proxy_url_nopath;
							char *proxy_url_path;
							parseURL_nopath_path(proxy_url, &proxy_url_nopath, &proxy_url_path);
							wsdmsg_set_url(wsdmsg, proxy_url_path);
							wsdmsg_set_no_url(wsdmsg, proxy_url_nopath);
						}
						*/
						MSG_PTR msg = wsdcomposer_wsd_reply(wsdmsg);
						
						composer_publish_msg(msg, composer);
						
						fprintf(stdout, "wsd_run_composer: (before remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
		
		
						COMPOSERTHREAD_LIST_ENTRY_PTR first_composer_thread  = eventqueue_table_remove_first_composer_thread(entry);
						
						// set current_socket_reply_port for ssdp parser to find the right entry on the table
						entry->current_socket_reply_port = first_composer_thread->socket_reply_port;				
						
						pthread_cond_signal(&(first_composer_thread->cond));
						
		
						fprintf(stdout, "wsd_run_composer: (after remove) eventqueue_table_get_composer_thread_list_length(entry=%p) = %d\n",entry, eventqueue_table_get_composer_thread_list_length(entry));
						
						
						pthread_mutex_unlock(&(entry->mutex));
						
						fprintf(stdout, "wsd_run_composer: signal cond var %p\n", &(first_composer_thread->cond));
						return;								
					}
					break;
				}
			}					
	}	
}

	

void wsd_service_discover_table_init(WSD_SERVICE_DISCOVER_TABLE* table)
{
	table->n_services = 0;
	int i =0;
	for (i=0 ; i<100; i++)
	{
		table->services[i].stream_id = -1;
		table->services[i].wsdmsg = NULL;
		table->services[i].proxy_url = NULL;
	}	
}

void wsd_service_discover_table_add_entry(WSD_SERVICE_DISCOVER_TABLE* table, int stream_id, WSDMSG_PTR wsdmsg)
{
	fprintf(stdout, "wsd_service_discover_table_add_entry begin (services_table=%p). (nservices=%d)\n",table, table->n_services);
	
	table->services[table->n_services].stream_id = stream_id;
	table->services[table->n_services].wsdmsg = wsdmsg;
	table->n_services += 1;
	
	fprintf(stdout, "wsd_service_discover_table_add_entry end. (nservices=%d) \n",table->n_services );
}

int wsd_service_discover_table_contains_stream_id(WSD_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		fprintf(stdout, "wsd_service_discover_table_contains_stream_id. str1=%p str2=%p\n",table.services[i].stream_id, stream_id);
		if (table.services[i].stream_id == stream_id)
			return 1;
	}
	return 0;
}

WSDMSG_PTR wsd_service_discover_table_get_wsdmsg_by_stream_id(WSD_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		fprintf(stdout, "wsd_service_discover_table_get_wsd_by_stream_id. str1=%p str2=%p\n",table.services[i].stream_id, stream_id);
		if (table.services[i].stream_id == stream_id)
			return table.services[i].wsdmsg;
	}
	return NULL;
}


char* wsd_service_discover_table_get_proxy_url(WSD_SERVICE_DISCOVER_TABLE table, int stream_id)
{
	int i =0;
	for (i=0 ; i<table.n_services; i++)
	{
		
		//fprintf(stdout, "wsd_service_discover_table_get_proxy_url. (nservices=%d) str1=%p str2=%p\n table.services[i].location_url_path=%s",table.n_services, table.services[i].stream_id, stream_id, table.services[i].location_url_path);
		if (table.services[i].stream_id == stream_id)
		{
			//fprintf(stdout, "upnp_service_discover_table_get_location_url_path.FOUND!!!\n");
			return  table.services[i].proxy_url;
		}
		
	}
	return NULL;
}

void wsd_service_discover_table_set_proxy_url(WSD_SERVICE_DISCOVER_TABLE* table, int stream_id, char* proxy_url)
{
	int i =0;
	for (i=0 ; i<table->n_services; i++)
	{
		//fprintf(stdout, "upnp_service_discover_table_set_location_url_path. (nservices=%d) str1=%p str2=%p location_url_path=%s\n",table->n_services, table->services[i].stream_id, stream_id, location_url_nopath);
		if (table->services[i].stream_id == stream_id)
		{
			table->services[i].proxy_url = strdup(proxy_url);
			//fprintf(stdout, "table.services[i].location_url_path set to : %s location_url_path=%s\n",table->services[i].location_url_nopath, location_url_nopath);
		}
	}
}
	
	
void wsd_composer_event_received(EVENT_PTR evt, LISTENER_PTR node)
{
	fprintf(stdout,"WSD");
	composer_event_received(evt, node);
}

COMPOSER_PTR wsdcomposer_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating WSD composer\n");
		
	
	COMPOSER_PTR c = composer_create(u);

	c->run_composer = wsd_run_composer;		
	c->factory = wsdcomposer_create;
	
	c->func_event_received = wsd_composer_event_received;	
	return c;	
}

void AdjustACTLength(int iMsgType,int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: *iSize += strlen(ELEMENT_ACTION_HELLO);
		break;
	case WSD_BYE_MSG: *iSize += strlen(ELEMENT_ACTION_BYE);
		break;
	case WSD_PROBE_MSG: *iSize += strlen(ELEMENT_ACTION_PROBE);
		break;
	case WSD_PROBE_MATCH_MSG: *iSize += strlen(ELEMENT_ACTION_PROBE_MATCH);
		break;
	case WSD_RESOLVE_MSG: *iSize += strlen(ELEMENT_ACTION_RESOLVE);
		break;
	case WSD_RESOLVE_MATCH_MSG: *iSize += strlen(ELEMENT_ACTION_RESOLVE_MATCH);
		break;
	default:
		break;
	}	
}
void AdjustMSGIDLength(int *iSize)
{
	*iSize += strlen(ELEMENT_MESSAGEID_START);
	*iSize += strlen(ELEMENT_MESSAGEID_DATA);
	*iSize += strlen(ELEMENT_MESSAGEID_END);
}
void AdjustTOLength(int *iSize)
{
	*iSize += strlen(ELEMENT_TO);
}
void AdjustAPPSEQLength(int iMsgType, int *iSize)
{
	/*
	iAppSegInstanceID++;
	iAppSegMessageNum++;
		
	if(iAppSegInstanceID == 1000)
		iAppSegInstanceID = 0;
	
	if(iAppSegMessageNum == 1000)
		iAppSegMessageNum = 0;
	
	sprintf(AppSegInstanceID_str,"%d",iAppSegInstanceID);
	sprintf(AppSegMessageNum_str,"%d",iAppSegMessageNum);
	*/
	
	switch(iMsgType)
	{
	case WSD_HELLO_MSG:
	case WSD_BYE_MSG:
	case WSD_PROBE_MATCH_MSG:
	case WSD_RESOLVE_MATCH_MSG:
		{
			*iSize += strlen(ELEMENT_APPSEQ_START);
			*iSize += strlen(ELEMENT_APPSEQ_INSTANCEID);
			*iSize += strlen(ELEMENT_APPSEQ_INSTANCEID_VAL);
			//*iSize += strlen(AppSegInstanceID_str);
			*iSize += strlen(ELEMENT_APPSEQ_MSGNUM);
			*iSize += strlen(ELEMENT_APPSEQ_MSQNUM_VAL);
			//*iSize += strlen(AppSegMessageNum_str);
			*iSize += strlen(ELEMENT_APPSEQ_END);		
		}
		break;
	default:
			break;
	}
}
void AdjustRELATESTOLength(WSDMSG_PTR wsdmsg,int iMsgType,int bRelatesToPresent, int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG:
		if(bRelatesToPresent == TRUE)
		{
			*iSize += strlen(ELEMENT_RELATESTO_START_EMPTY);
			*iSize += strlen(ELEMENT_RELATESTO_RELTYPE);
			*iSize += strlen(ELEMENT_RELATESTO_RELTYPE_VAL);
			*iSize += strlen(ELEMENT_RELATESTO_CLOSE);
			*iSize += strlen(ELEMENT_RELATESTO_VAL);
			*iSize += strlen(ELEMENT_RELATESTO_END);
		}
		break;
	case WSD_PROBE_MSG:
	case WSD_PROBE_MATCH_MSG:
	case WSD_RESOLVE_MATCH_MSG:
		{
			*iSize += strlen(ELEMENT_RELATESTO_START_EMPTY);
			*iSize += strlen(wsdmsg_get_message_id(wsdmsg));
			*iSize += strlen(ELEMENT_RELATESTO_END);
		}
		break;
	default:
		break;
	}
}
void AdjustREPLYTOLength(int iMsgType,int bReplyToPresent,int *iSize)
{
	switch(iMsgType)
	{
	case WSD_PROBE_MSG:
	case WSD_RESOLVE_MSG:
		if(bReplyToPresent == TRUE)
		{
			*iSize += strlen(ELEMENT_REPLYTO_START);
			*iSize += strlen(ELEMENT_ENDPOINTREF_START);
			*iSize += strlen(ELEMENT_ADDR_START);
			*iSize += strlen(ELEMENT_ADDR);
			*iSize += strlen(ELEMENT_ADDR_END);
			*iSize += strlen(ELEMENT_ENDPOINTREF_END);
			*iSize += strlen(ELEMENT_REPLYTO_END);
		}
		break;
	default:
		break;
	}
}
void AdjustBodyMainEndElement(int iMsgType, int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: *iSize += strlen(ELEMENT_HELLO_END);
		break;
	case WSD_BYE_MSG: *iSize += strlen(ELEMENT_BYE_END);
		break;
	case WSD_PROBE_MSG: *iSize += strlen(ELEMENT_PROBE_END);
		break;
	case WSD_PROBE_MATCH_MSG: *iSize += strlen(ELEMENT_PROBEMATCHES_END);
		break;
	case WSD_RESOLVE_MSG: *iSize += strlen(ELEMENT_RESOLVE_END);
		break;
	case WSD_RESOLVE_MATCH_MSG: *iSize += strlen(ELEMENT_RESOLVEMATCH_END);
		break;
	default:
		break;
	}		

}
void AdjustBodyMainStartElement(int iMsgType, int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: *iSize += strlen(ELEMENT_HELLO_START);
		break;
	case WSD_BYE_MSG: *iSize += strlen(ELEMENT_BYE_START);
		break;
	case WSD_PROBE_MSG: *iSize += strlen(ELEMENT_PROBE_START);
		break;
	case WSD_PROBE_MATCH_MSG: *iSize += strlen(ELEMENT_PROBEMATCHES_START);
		break;
	case WSD_RESOLVE_MSG: *iSize += strlen(ELEMENT_RESOLVE_START);
		break;
	case WSD_RESOLVE_MATCH_MSG: *iSize += strlen(ELEMENT_RESOLVEMATCH_START);
		break;
	default:
		break;
	}		
}
void AdjustEndPointReferenceLength(int iMsgType, int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: 
	case WSD_BYE_MSG: 
	case WSD_RESOLVE_MSG: 
		{
			*iSize += strlen(ELEMENT_ENDPOINTREF_START);
			*iSize += strlen(ELEMENT_ADDR_START);
			*iSize += strlen(ELEMENT_ADDR);
			*iSize += strlen(ELEMENT_ADDR_END);
			*iSize += strlen(ELEMENT_ENDPOINTREF_END);		
		}
		break;
	default:
		break;
	}		
}
void AdjustTypesLength(WSDMSG_PTR wsdmsg,int iMsgType,int bTypesPresent, int *iSize)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: 
	case WSD_PROBE_MSG: 
		if(bTypesPresent == TRUE)
		{
			*iSize += strlen(ELEMENT_TYPES_START);
			*iSize += strlen(wsdmsg_get_types(wsdmsg));
			*iSize += strlen(ELEMENT_TYPES_END);
		}
		break;
	default:
		break;
	}
}
void AdjustScopesLength(WSDMSG_PTR wsdmsg,int iMsgType, int bScopesPresent,int *iSize)
{
	if(bScopesPresent == TRUE)
	{
		switch(iMsgType)
		{
		case WSD_HELLO_MSG:
			{
				*iSize += strlen(ELEMENT_SCOPES_START);
				*iSize += strlen(ELEMENT_SCOPES_CLOSE);
				*iSize += strlen(wsdmsg_get_scopes(wsdmsg));
				*iSize += strlen(ELEMENT_SCOPES_END);
			}
			break;
		case WSD_PROBE_MSG: 			
			{
				*iSize += strlen(ELEMENT_SCOPES_START);
			//	*iSize += strlen(ELEMENT_SCOPES_MATCHBY);
			//	*iSize += strlen(ELEMENT_SCOPES_MATCHBY_VAL);
				*iSize += strlen(ELEMENT_SCOPES_CLOSE);
				*iSize += strlen(wsdmsg_get_scopes(wsdmsg));
				*iSize += strlen(ELEMENT_SCOPES_END);
			}
			break;
		default:
			break;
		}		
	}
}
void AdjustXAddrsLength(WSDMSG_PTR wsdmsg,int iMsgType,int bXAddrsPresent,int *iSize)
{
	if(bXAddrsPresent == TRUE && iMsgType == WSD_PROBE_MSG)
	{
		*iSize += strlen(ELEMENT_XADRS_START);
		*iSize += strlen(wsdmsg_get_no_url(wsdmsg));
		*iSize += strlen(wsdmsg_get_url(wsdmsg));
		*iSize += strlen(ELEMENT_XADRS_END);
	}
}
void AdjustMetaDataVersionLength(int iMsgType,int *iSize)
{
	if(iMsgType == WSD_HELLO_MSG)
	{
		*iSize += strlen(ELEMENT_METADATAVER_START);
		*iSize += strlen(ELEMENT_METADATAVER_VAL);
		*iSize += strlen(ELEMENT_METADATAVER_END);	
	}
}
void AdjustProbeMatchLength(WSDMSG_PTR wsdmsg,int bTypesPresent,int bScopesPresent,int bXAddrsPresent,int *iSize)
{
	
	*iSize += strlen(ELEMENT_PROBEMATCH_START);

	*iSize += strlen(ELEMENT_ENDPOINTREF_START);
	*iSize += strlen(ELEMENT_ADDR_START);
	*iSize += strlen(ELEMENT_ADDR);
	*iSize += strlen(ELEMENT_ADDR_END);
	*iSize += strlen(ELEMENT_ENDPOINTREF_END);

	*iSize += strlen(ELEMENT_METADATAVER_START);
	*iSize += strlen(ELEMENT_METADATAVER_VAL);
	*iSize += strlen(ELEMENT_METADATAVER_END);

	*iSize += strlen(ELEMENT_TYPES_START);
	*iSize += strlen(wsdmsg_get_types(wsdmsg));
	*iSize += strlen(ELEMENT_TYPES_END);
	
	//*iSize += strlen(ELEMENT_RELATESTO_START_EMPTY);
	//*iSize += strlen(wsdmsg_get_message_id(wsdmsg));
	//*iSize += strlen(ELEMENT_RELATESTO_END);
	
	if(bTypesPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_TYPES_START);
		*iSize += strlen(wsdmsg_get_types(wsdmsg));
		*iSize += strlen(ELEMENT_TYPES_END);
	}
	if(bScopesPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_SCOPES_START);
		*iSize += strlen(ELEMENT_SCOPES_CLOSE);
		*iSize += strlen(wsdmsg_get_scopes(wsdmsg));
		*iSize += strlen(ELEMENT_SCOPES_END);
	}
	if(bXAddrsPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_XADRS_START);
		*iSize += strlen(wsdmsg_get_no_url(wsdmsg));
		*iSize += strlen(wsdmsg_get_url(wsdmsg));
		//*iSize += strlen("http://192.168.2.2:2222");
		*iSize += strlen(ELEMENT_XADRS_END);
	}

	*iSize += strlen(ELEMENT_PROBEMATCH_END);
}
void AdjustResolveMatchLength(int bTypesPresent,int bScopesPresent,int bXAddrsPresent,int *iSize)
{
	*iSize += strlen(ELEMENT_RESOLVEMATCH_START);

	*iSize += strlen(ELEMENT_ENDPOINTREF_START);
	*iSize += strlen(ELEMENT_ADDR_START);
	*iSize += strlen(ELEMENT_ADDR);
	*iSize += strlen(ELEMENT_ADDR_END);
	*iSize += strlen(ELEMENT_ENDPOINTREF_END);

	*iSize += strlen(ELEMENT_METADATAVER_START);
	*iSize += strlen(ELEMENT_METADATAVER_VAL);
	*iSize += strlen(ELEMENT_METADATAVER_END);

	if(bTypesPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_TYPES_START);
		*iSize += strlen(ELEMENT_TYPES_VAL);
		*iSize += strlen(ELEMENT_TYPES_END);
	}
	if(bScopesPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_SCOPES_START);
		*iSize += strlen(ELEMENT_SCOPES_CLOSE);
	//	*iSize += strlen(ELEMENT_SCOPES_VAL);
		*iSize += strlen(ELEMENT_SCOPES_END);
	}
	if(bXAddrsPresent == TRUE)
	{
		*iSize += strlen(ELEMENT_XADRS_START);
		*iSize += strlen(ELEMENT_XADRS_VAL);
		*iSize += strlen(ELEMENT_XADRS_END);
	}

	*iSize += strlen(ELEMENT_RESOLVEMATCH_END);
}

void ComposeHeaderElement(WSDMSG_PTR wsdmsg,int iMsgType,int bRelatesToPresent,int bReplyToPresent,char *data)
{
	strcat(data,ELEMENT_HEADER_START);
	ComposeActionElement(iMsgType,data);
	ComposeMessageIDElement(data);
	ComposeTOElement(data);
	ComposeAPPSEQElement(iMsgType,data);
	ComposeRelatesToElement(wsdmsg,iMsgType,bRelatesToPresent,data);
	ComposeReplyToElement(iMsgType,bReplyToPresent,data);	
	strcat(data,ELEMENT_HEADER_END);
}
void ComposeActionElement(int iMsgType,char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: strcat(data,ELEMENT_ACTION_HELLO);
		break;
	case WSD_BYE_MSG: strcat(data,ELEMENT_ACTION_BYE);
		break;
	case WSD_PROBE_MSG: strcat(data,ELEMENT_ACTION_PROBE);
		break;
	case WSD_PROBE_MATCH_MSG: strcat(data,ELEMENT_ACTION_PROBE_MATCH);
		break;
	case WSD_RESOLVE_MSG: strcat(data,ELEMENT_ACTION_RESOLVE);
		break;
	case WSD_RESOLVE_MATCH_MSG: strcat(data,ELEMENT_ACTION_RESOLVE_MATCH);
		break;
	default:
		break;
	}	
}
void ComposeMessageIDElement(char *data)
{
	strcat(data,ELEMENT_MESSAGEID_START);
	strcat(data,ELEMENT_MESSAGEID_DATA);
	strcat(data,ELEMENT_MESSAGEID_END);
}
void ComposeTOElement(char *data)
{
	strcat(data,ELEMENT_TO);	
}
void ComposeAPPSEQElement(int iMsgType,char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG:
	case WSD_BYE_MSG:
	case WSD_PROBE_MATCH_MSG:
	case WSD_RESOLVE_MATCH_MSG:
		{
			strcat(data,ELEMENT_APPSEQ_START);
			strcat(data,ELEMENT_APPSEQ_INSTANCEID);
			strcat(data,ELEMENT_APPSEQ_INSTANCEID_VAL);
			//strcat(data,AppSegInstanceID_str);
			strcat(data,ELEMENT_APPSEQ_MSGNUM);
			strcat(data,ELEMENT_APPSEQ_MSQNUM_VAL);
			//strcat(data,AppSegMessageNum_str);
			strcat(data,ELEMENT_APPSEQ_END);
		}
		break;
	default:
			break;
	}	
}
void ComposeRelatesToElement(WSDMSG_PTR wsdmsg,int iMsgType,int bRelatesToPresent,char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG:
		if(bRelatesToPresent == TRUE)
		{
			strcat(data,ELEMENT_RELATESTO_START);
			strcat(data,ELEMENT_RELATESTO_RELTYPE);
			strcat(data,ELEMENT_RELATESTO_RELTYPE_VAL);
			strcat(data,ELEMENT_RELATESTO_CLOSE);
			strcat(data,ELEMENT_RELATESTO_VAL);
			strcat(data,ELEMENT_RELATESTO_END);
		}
		break;
	case WSD_PROBE_MSG:
	case WSD_PROBE_MATCH_MSG:
	case WSD_RESOLVE_MATCH_MSG:
		{
			strcat(data,ELEMENT_RELATESTO_START_EMPTY);
			strcat(data,wsdmsg_get_message_id(wsdmsg));
			strcat(data,ELEMENT_RELATESTO_END);
		}
		break;
	default:
		break;
	}

}
void ComposeReplyToElement(int iMsgType,int bReplyToPresent,char *data)
{
	switch(iMsgType)
	{
	case WSD_PROBE_MSG:
	case WSD_RESOLVE_MSG:
		if(bReplyToPresent == TRUE)
		{
			strcat(data,ELEMENT_REPLYTO_START);
			strcat(data,ELEMENT_ENDPOINTREF_START);
			strcat(data,ELEMENT_ADDR_START);
			strcat(data,ELEMENT_ADDR);
			strcat(data,ELEMENT_ADDR_END);
			strcat(data,ELEMENT_ENDPOINTREF_END);
			strcat(data,ELEMENT_REPLYTO_END);
		}
		break;
	default:
		break;
	}

}
void ComposeBodyElement(WSDMSG_PTR wsdmsg,int iMsgType,int bTypesPresent,int bScopesPresent,int bXAddrsPresent,char *data)
{
	strcat(data,ELEMENT_BODY_START);	
	ComposeBodyMainStartElement(iMsgType,data);
	if(iMsgType == WSD_PROBE_MATCH_MSG)
	{
		ComposeProbeMatchElement(wsdmsg,bTypesPresent,bScopesPresent,bXAddrsPresent,data);
	}
	else if(iMsgType == WSD_RESOLVE_MATCH_MSG)
	{
		ComposeResolveMatchElement(bTypesPresent,bScopesPresent,bXAddrsPresent,data);
	}
	else
	{
		ComposeEndPointReferenceElement(iMsgType,data);
		ComposeTypesElement(wsdmsg,iMsgType,bTypesPresent,data);
		ComposeScopesElement(wsdmsg,iMsgType,bScopesPresent,data);
		ComposeMetaDataVersionElement(iMsgType,data);	
	}
	ComposeBodyMainEndElement(iMsgType,data);	
	strcat(data,ELEMENT_BODY_END);
}
void ComposeBodyMainStartElement(int iMsgType, char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: strcat(data,ELEMENT_HELLO_START);
		break;
	case WSD_BYE_MSG: strcat(data,ELEMENT_BYE_START);
		break;
	case WSD_PROBE_MSG: strcat(data,ELEMENT_PROBE_START);
		break;
	case WSD_PROBE_MATCH_MSG: strcat(data,ELEMENT_PROBEMATCHES_START);
		break;
	case WSD_RESOLVE_MSG: strcat(data,ELEMENT_RESOLVE_START);
		break;
	case WSD_RESOLVE_MATCH_MSG: strcat(data,ELEMENT_RESOLVEMATCHES_START);
		break;
	default:
		break;
	}		
}
void ComposeBodyMainEndElement(int iMsgType, char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: strcat(data,ELEMENT_HELLO_END);
		break;
	case WSD_BYE_MSG: strcat(data,ELEMENT_BYE_END);
		break;
	case WSD_PROBE_MSG: strcat(data,ELEMENT_PROBE_END);
		break;
	case WSD_PROBE_MATCH_MSG: strcat(data,ELEMENT_PROBEMATCHES_END);
		break;
	case WSD_RESOLVE_MSG: strcat(data,ELEMENT_RESOLVE_END);
		break;
	case WSD_RESOLVE_MATCH_MSG: strcat(data,ELEMENT_RESOLVEMATCHES_END);
		break;
	default:
		break;
	}		

}

void ComposeEndPointReferenceElement(int iMsgType, char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: 
	case WSD_BYE_MSG: 
	case WSD_RESOLVE_MSG: 
		{
			strcat(data,ELEMENT_ENDPOINTREF_START);
			strcat(data,ELEMENT_ADDR_START);
			strcat(data,ELEMENT_ADDR);
			strcat(data,ELEMENT_ADDR_END);
			strcat(data,ELEMENT_ENDPOINTREF_END);
		}
		break;
	default:
		break;
	}		
}
void ComposeTypesElement(WSDMSG_PTR wsdmsg,int iMsgType,int bTypesPresent, char *data)
{
	switch(iMsgType)
	{
	case WSD_HELLO_MSG: 
	case WSD_PROBE_MSG: 
		if(bTypesPresent == TRUE)
		{
			strcat(data,ELEMENT_TYPES_START);
			strcat(data,wsdmsg_get_types(wsdmsg));
			strcat(data,ELEMENT_TYPES_END);
		}
		break;
	default:
		break;
	}

}
void ComposeScopesElement(WSDMSG_PTR wsdmsg,int iMsgType,int bScopesPresent,char *data)
{
	if(bScopesPresent == TRUE)
	{
		switch(iMsgType)
		{
		case WSD_HELLO_MSG:
			{
				strcat(data,ELEMENT_SCOPES_START);
				strcat(data,ELEMENT_SCOPES_CLOSE);
				strcat(data,wsdmsg_get_scopes(wsdmsg));
				strcat(data,ELEMENT_SCOPES_END);
			}
			break;
		case WSD_PROBE_MSG: 			
			{
				strcat(data,ELEMENT_SCOPES_START);
		//		strcat(data,ELEMENT_SCOPES_MATCHBY);
		//		strcat(data,ELEMENT_SCOPES_MATCHBY_VAL);
				strcat(data,ELEMENT_SCOPES_CLOSE);
				strcat(data,wsdmsg_get_scopes(wsdmsg));
				strcat(data,ELEMENT_SCOPES_END);

			}
			break;
		default:
			break;
		}		
	}
}
void ComposeXAddrsElement(WSDMSG_PTR wsdmsg,int iMsgType,int bXAddrsPresent,char *data)
{
	if(bXAddrsPresent == TRUE && iMsgType == WSD_PROBE_MSG)
	{
		strcat(data,ELEMENT_XADRS_START);
		strcat(data,wsdmsg_get_no_url(wsdmsg));
		strcat(data,wsdmsg_get_url(wsdmsg));
		strcat(data,ELEMENT_XADRS_END);
	}
}
void ComposeMetaDataVersionElement(int iMsgType,char *data)
{
	if(iMsgType == WSD_HELLO_MSG)
	{
		strcat(data,ELEMENT_METADATAVER_START);
		strcat(data,ELEMENT_METADATAVER_VAL);
		strcat(data,ELEMENT_METADATAVER_END);	
	}
}
void ComposeProbeMatchElement(WSDMSG_PTR wsdmsg,int bTypesPresent,int bScopesPresent,int bXAddrsPresent,char *data)
{
	strcat(data,ELEMENT_PROBEMATCH_START);

	strcat(data,ELEMENT_ENDPOINTREF_START);
	strcat(data,ELEMENT_ADDR_START);
	strcat(data,ELEMENT_ADDR);
	strcat(data,ELEMENT_ADDR_END);
	strcat(data,ELEMENT_ENDPOINTREF_END);

	strcat(data,ELEMENT_METADATAVER_START);
	strcat(data,ELEMENT_METADATAVER_VAL);
	strcat(data,ELEMENT_METADATAVER_END);

	//strcat(data,ELEMENT_RELATESTO_START_EMPTY);
	//strcat(data,wsdmsg_get_message_id(wsdmsg));
	//strcat(data,ELEMENT_RELATESTO_END);
	
	if(bTypesPresent == TRUE)
	{
		strcat(data,ELEMENT_TYPES_START);
		strcat(data,wsdmsg_get_types(wsdmsg));
		strcat(data,ELEMENT_TYPES_END);
	}
	if(bScopesPresent == TRUE)
	{
		strcat(data,ELEMENT_SCOPES_START);
		strcat(data,ELEMENT_SCOPES_CLOSE);
		strcat(data,wsdmsg_get_scopes(wsdmsg));
		strcat(data,ELEMENT_SCOPES_END);
	}
	if(bXAddrsPresent == TRUE)
	{
		strcat(data,ELEMENT_XADRS_START);
		strcat(data,wsdmsg_get_no_url(wsdmsg));
		strcat(data,wsdmsg_get_url(wsdmsg));
		//strcat(data,"http://192.168.2.2:2222");
		strcat(data,ELEMENT_XADRS_END);
	}

	strcat(data,ELEMENT_PROBEMATCH_END);	
}
void ComposeResolveMatchElement(int bTypesPresent,int bScopesPresent,int bXAddrsPresent,char *data)
{
	strcat(data,ELEMENT_RESOLVEMATCH_START);

	strcat(data,ELEMENT_ENDPOINTREF_START);
	strcat(data,ELEMENT_ADDR_START);
	strcat(data,ELEMENT_ADDR);
	strcat(data,ELEMENT_ADDR_END);
	strcat(data,ELEMENT_ENDPOINTREF_END);

	strcat(data,ELEMENT_METADATAVER_START);
	strcat(data,ELEMENT_METADATAVER_VAL);
	strcat(data,ELEMENT_METADATAVER_END);

	if(bTypesPresent == TRUE)
	{
		strcat(data,ELEMENT_TYPES_START);
		strcat(data,ELEMENT_TYPES_VAL);
		strcat(data,ELEMENT_TYPES_END);
	}
	if(bScopesPresent == TRUE)
	{
		strcat(data,ELEMENT_SCOPES_START);
		strcat(data,ELEMENT_SCOPES_CLOSE);
	//	strcat(data,ELEMENT_SCOPES_VAL);
		strcat(data,ELEMENT_SCOPES_END);
	}
	if(bXAddrsPresent == TRUE)
	{
		strcat(data,ELEMENT_XADRS_START);
		strcat(data,ELEMENT_XADRS_VAL);
		strcat(data,ELEMENT_XADRS_END);
	}

	strcat(data,ELEMENT_RESOLVEMATCH_END);	
}


 
