#include "event/event.h"
#include <stdio.h>
#include "composer/JRMPComposer.h"


EVENTSRC_PTR event_create_src(void* src_thread_parser, struct unit* src_unit)
{
	EVENTSRC_PTR evt_src = malloc(sizeof(EVENTSRC));
	evt_src->src_thread_parser = src_thread_parser;
	evt_src->src_unit = src_unit;
	return evt_src;
}

EVENT_PTR event_create(int type, int stream_id, void* src)
{
		EVENT_PTR evt = malloc(sizeof(EVENT));
		evt->type = type;
		evt->stream_id = stream_id;
		evt->src = src;
		return evt;
}

EVENT_PTR event_create_int(int type, int value, int stream_id,  void* src)
{
		EVENT_PTR evt = event_create(type, stream_id, src);
		evt->value.value_int = value;
		return evt;
}

EVENT_PTR event_create_string(int type, char* value, int stream_id, void* src)
{
		EVENT_PTR evt = event_create(type, stream_id, src);
		if (value != NULL)
			evt->value.value_string = strdup(value);
		else
			evt->value.value_string = NULL;
		return evt;
}

EVENT_PTR event_create_void_pointer(int type, void* value, int stream_id, void* src)
{
		EVENT_PTR evt = event_create(type, stream_id, src);
		evt->value.value_void_pointer = value;
		return evt;
}

EVENT_PTR event_create_longlong(int type, long long value, int stream_id, void* src)
{
	EVENT_PTR evt = event_create(type, stream_id, src);
	evt->value.value_longlong = value;
	return evt;
}

void event_print(EVENT_PTR evt)
{

	if (evt == NULL)
		return;
	if (evt->src == NULL)
		return;
	
	fprintf(stdout, "Event: stream_id= %p. EVTSRC=(pars= %p, unit= %p) ",evt->stream_id, 
			((EVENTSRC_PTR)evt->src)->src_thread_parser, ((EVENTSRC_PTR)evt->src)->src_unit);
	switch (evt->type)
	{
	    case SDP_REQ_LANGAGE:
	    	fprintf(stdout, "Event SDP_REQ_LANGAGE. value = %s\n",evt->value.value_string);
		break;
	    case SDP_REQ_SCOPE:
	    	fprintf(stdout, "Event SDP_REQ_SCOPE. value = %s\n",evt->value.value_string);
		break;	
	    case SDP_REQ_PREDICAT:
	    	fprintf(stdout, "Event SDP_REQ_PREDICAT. value = %s\n",evt->value.value_string);
		break;	
	    case SDP_REQ_REQUESTER:
	    	fprintf(stdout, "Event SDP_REQ_REQUESTER. value = %s\n",evt->value.value_string);
		break;	
	    case SDP_REQ_SERVICE_TYPE:
	    	fprintf(stdout, "Event SDP_REQ_SERVICE_TYPE. value = %s\n",evt->value.value_string);
		break;	
 	    case SDP_REQ_ID:
	    	fprintf(stdout, "Event SDP_REQ_ID. value = %s\n",evt->value.value_string);
		break;		
 	    case SDP_VERSION:
	    	fprintf(stdout, "Event SDP_VERSION. value = %d\n",evt->value.value_int);
		break;
 	    case SDP_WSD:
 	    	fprintf(stdout, "Event SDP_WSD\n");
 			break;
	    case SDP_UPNP:
	    	fprintf(stdout, "Event SDP_UPNP\n");
		break;
	    case SDP_SLP:
	    	fprintf(stdout, "Event SDP_SLP\n");
		break;
	    case SDP_START:
	    	fprintf(stdout, "Event SDP_START\n");
		break;	    		
	    case SDP_STOP:
	    	fprintf(stdout, "Event SDP_STOP\n");
		break;	
	    case SDP_SOURCE_ADDR:
	    	fprintf(stdout, "Event SDP_SOURCE_ADDR. value = %s\n",evt->value.value_string);
		break;		
	    case SDP_SOURCE_PORT:
	    	fprintf(stdout, "Event SDP_SOURCE_PORT. value = %d\n",evt->value.value_int);
		break;	
		case SDP_SERVICE_DESCR_URL_NOPATH:
			fprintf(stdout, "Event SDP_SERVICE_DESCR_URL_NOPATH. value = %s\n",evt->value.value_string);
			break;
		case SDP_SERVICE_DESCR_URL_PATH:
			fprintf(stdout, "Event SDP_SERVICE_DESCR_URL_PATH. value = %s\n",evt->value.value_string);
			break;
		case SDP_SERVICE_CTRL_URL_NOPATH:
			fprintf(stdout, "Event SDP_SERVICE_CTRL_URL_NOPATH. value = %s\n",evt->value.value_string);
			break;
	    case SDP_SERVICE_RESPONSE:
	    	fprintf(stdout, "Event SDP_SERVICE_RESPONSE\n");
	    	break;
	    case SDP_INTERFACE_DESCR_URL_PATH:
	    	fprintf(stdout, "Event SDP_INTERFACE_DESCR_URL_PATH value = %s\n",evt->value.value_string);
	    	break;
	    case SDP_INTERFACE_CTRL_URL_PATH:
	    	fprintf(stdout, "Event SDP_INTERFACE_CTRL_URL_PATH value = %s\n",evt->value.value_string);
	    	break;
	    case SDP_SERVICE_METHOD_DESCRIPTION:
	    {
	    	EVENT_METHOD_DEF_PTR event_method = (EVENT_METHOD_DEF_PTR)(evt->value.value_void_pointer);
	    	fprintf(stdout, "Event SDP_SERVICE_METHOD_DESCRIPTION: service_name=%s interface_name=%s  \n",event_method->service_name, event_method->interface_name);
	    }
	    	break;
	    case SDP_SERVICE_VAR_DESCRIPTION:
	    	fprintf(stdout, "SDP_SERVICE_VAR_DESCRIPTION\n");
	    	break;
		case SDP_SERVICE_REQUEST:
			fprintf(stdout, "SDP_SERVICE_REQUEST\n");
			break;
		case SDP_SEND_SERVICE_REQUEST:
			fprintf(stdout, "SDP_SEND_SERVICE_REQUEST\n");
			break;
		case SDP_SEND_SERVICE_REPLY:
			fprintf(stdout, "SDP_SEND_SERVICE_REPLY\n");
			break;
		case SDP_SERVICE_DESCRIPTION:
			fprintf(stdout, "SDP_SERVICE_DESCRIPTION\n");
			break;
			
		case SDP_SERVICE_DESCRIPTION_SERVICETYPE:
			fprintf(stdout, "SDP_SERVICE_DESCRIPTION_SERVICETYPE value= %s\n", evt->value.value_string);
			break;
		case SDP_SERVICE_DESCRIPTION_INTERFACETYPE:
			fprintf(stdout, "SDP_SERVICE_DESCRIPTION_INTERFACETYPE value= %s\n", evt->value.value_string);
			break;
			
		case SDP_SERVICE_DESCRIPTION_INTERFACE_NAMESPACE:
			fprintf(stdout, "SDP_SERVICE_DESCRIPTION_INTERFACE_NAMESPACE value= %s\n", evt->value.value_string);
			break;
		
		case GENERATE_PROXY:
			fprintf(stdout, "GENERATE_PROXY\n");
			break;
			
		case SCP_RMIOBJ_NUM:
			fprintf(stdout, "SCP_RMIOBJ_NUM value = %llu\n",evt->value.value_longlong);
			break;
		case SCP_RMIOBJ_UNIQUE:
			fprintf(stdout, "SCP_RMIOBJ_UNIQUE value = %d\n",evt->value.value_int);
			break;
		case SCP_RMIOBJ_TIME:
			fprintf(stdout, "SCP_RMIOBJ_TIME value = %llu\n",evt->value.value_longlong);
			break;
		case SCP_RMIOBJ_COUNT:
			fprintf(stdout, "SCP_RMIOBJ_COUNT value = %d\n",evt->value.value_int);
			break;
		case SCP_RMI_STUB_OBJ:
	    {
			RMIHOLDER_STUB_OBJ_PTR obj = (RMIHOLDER_STUB_OBJ_PTR)(evt->value.value_void_pointer);
	    	fprintf(stdout, "Event SCP_RMI_STUB_OBJ: index=%d\n",obj->arg_index);
	    }
	    break;
			
		case SCP_RMIOBJ_IP:
			fprintf(stdout, "SCP_RMIOBJ_IP value = %s\n",evt->value.value_string);
			break;
			
		case SCP_RMIOBJ_PORT:
			fprintf(stdout, "SCP_RMIOBJ_PORT value = %d\n",evt->value.value_int);
			break;
			
		case SDP_SERVICEPROXY_URL_CTRL:
			fprintf(stdout, "SDP_SERVICEPROXY_URL_CTRL value = %s\n",evt->value.value_string);
			break;
		case SCP_SOAP:
			fprintf(stdout, "SCP_SOAP\n");
			break;
		case SCP_REQUEST:
			fprintf(stdout, "SCP_REQUEST\n");
			break;
		case SCP_RESPONSE:
			fprintf(stdout, "SCP_RESPONSE\n");
			break;
		case SCP_METHOD_NAME:
			fprintf(stdout, "SCP_METHOD_NAME value = %s\n",evt->value.value_string);
			break;
		case SCP_METHOD_NAMESPACE:
			fprintf(stdout, "SCP_METHOD_NAMESPACE value = %s\n",evt->value.value_string);
			break;
		case SCP_ARGUMENT_IN:
			{
			ARGUMENTDEFPTR event_arg = (ARGUMENTDEFPTR)(evt->value.value_void_pointer);
			fprintf(stdout, "SCP_ARGUMENT_IN  name=%s value=%s datatype=%s\n",event_arg->name, event_arg->value, event_arg->datatype);
			}
			break;
		case SCP_ARGUMENT_OUT:
		{
			ARGUMENTDEFPTR event_arg = (ARGUMENTDEFPTR)(evt->value.value_void_pointer);
			fprintf(stdout, "SCP_ARGUMENT_OUT  name=%s value=%s datatype=%s\n",event_arg->name, event_arg->value, event_arg->datatype);
		}
			break;
		case SCP_SERVICE_NAME:
			fprintf(stdout, "SCP_SERVICE_NAME value = %s\n",evt->value.value_string);
			break;
		case SCP_SEND_SERVICE_REQUEST:
			fprintf(stdout, "SCP_SEND_SERVICE_REQUEST\n");
			break;
		case SCP_SEND_SERVICE_REPLY:
			fprintf(stdout, "SCP_SEND_SERVICE_REPLY\n");
			break;
		case SCP_SOCKET_ID:
			fprintf(stdout, "SCP_SOCKET_ID value = %d\n",evt->value.value_int);
			break;
			
	    default:
	    	fprintf(stdout, "Event not supported\n");
			break;	
	} 
}



