#include <stdio.h>
#include "event/servicedef.h"

int Table_add_PathServiceDef_Elem(PATH_SERVICEDEF_PTR* table, char* path, SERVICEDEFPTR service)
{
  if (path != NULL) { 
    PATH_SERVICEDEF_PTR new_elem = (PATH_SERVICEDEF_PTR)malloc(sizeof(PATH_SERVICEDEF));
    new_elem->path = (char*)malloc(strlen(path) + 1);
    strcpy(new_elem->path, path);
    new_elem->service = service;
    new_elem->next = NULL;

    if (*table == NULL) {
	*table = new_elem;
    } else {
	PATH_SERVICEDEF_PTR tableElem = *table;
	while (tableElem->next != NULL)
 	   tableElem = tableElem->next;
	tableElem->next = new_elem;
    }
  }
}


PATH_SERVICEDEF_PTR create_PathServiceDef_Table()
{
    return (PATH_SERVICEDEF_PTR)NULL;
}

SERVICEDEFPTR Table_get_ServiceDef(PATH_SERVICEDEF_PTR table, char* path)
{
    PATH_SERVICEDEF_PTR tableElem = table;
    while ( tableElem != NULL && strcmp(tableElem->path, path) != 0 )
    {
    //	servicedef_print(tableElem->service);
       tableElem = tableElem->next;
    }
    if (tableElem != NULL)
    	return tableElem->service;
    else 
    	return NULL;
}

SERVICEDEFPTR Table_get_ServiceDef_and_path_by_RMIObjID(PATH_SERVICEDEF_PTR table, j_long objNum, j_int unique, j_long time, j_short count, char** service_path)
{
    PATH_SERVICEDEF_PTR tableElem = table;
    while ( tableElem != NULL )
    {
    	if (tableElem != NULL && tableElem->service != NULL)
    	{
    		SERVICEDEFPTR srv = tableElem->service;
    		if (srv->RMI_ObjNum == objNum && srv->RMI_ObjUnique == unique && srv->RMI_ObjTime == time && srv->RMI_ObjCount == count)
    		{
    			*service_path = strdup(tableElem->path);
    			return srv;
    		}
    	}
    	tableElem = tableElem->next;
    }
	return NULL;
}



/*
 *	functions to handle servicedef
 */
SERVICEDEFPTR servicedef_new()
{
    SERVICEDEFPTR serv = malloc(sizeof(SERVICEDEF));
    serv->interfacesDefinition = NULL;
    serv->name = NULL; 
    serv->host = NULL;
    serv->type = NULL;
    serv->port = -1;
    serv->indexService = GlobalServiceIndex++; //-1;
    serv->XMLfile = NULL;

    serv->methods_hash_index = METHODS_HASH_INDEX;
    
    fprintf(stdout, "\nSERVICEDEFPTR servicedef_new(): serv->methods_hash_index=%d\n",serv->methods_hash_index);
    
    return serv;
}

int servicedef_set_name(SERVICEDEFPTR servicedef, char* name)
{
    if (name != NULL) {
	servicedef->name = malloc(strlen(name)+1);
    	strcpy(servicedef->name, name);
    } else
	servicedef->name = NULL;
}

int servicedef_set_host(SERVICEDEFPTR servicedef, char* host)
{
    if (host != NULL) {
    	servicedef->host = malloc(strlen(host)+1);
    	strcpy(servicedef->host, host);
    } else
    	servicedef->host = NULL;
}

int servicedef_set_type(SERVICEDEFPTR servicedef, char* type)
{
    if (type != NULL) {
    	servicedef->type = malloc(strlen(type)+1);
    	strcpy(servicedef->type, type);
    } else
    	servicedef->type = NULL;
}

int servicedef_set_port(SERVICEDEFPTR servicedef, int port)
{
    servicedef->port = port;
}

int servicedef_set_indexService(SERVICEDEFPTR servicedef, int indexService)
{
    servicedef->indexService = indexService;
}

int servicedef_add_interface(SERVICEDEFPTR servicedef, INTERFACEDEFPTR iface)
{
  fprintf(stdout,"\nCALLED servicedef_add_interface\n");
    if (servicedef->interfacesDefinition == NULL) {
  fprintf(stdout,"\nCALLED servicedef_add_interface: void list\n");
	iface->next = NULL;
	servicedef->interfacesDefinition = iface;
    } else {
  fprintf(stdout,"\nCALLED servicedef_add_interface: not void list\n");
	iface->next = servicedef->interfacesDefinition;
	servicedef->interfacesDefinition = iface;
    }
}


METHODDEFPTR servicedef_getmethod_byname(SERVICEDEFPTR servicedef, char* name)
{
  METHODDEFPTR meth = NULL;
  INTERFACEDEFPTR ifaces = servicedef->interfacesDefinition;
  while (ifaces != NULL && meth == NULL) {
	meth = interfacedef_getmethod_byname(ifaces, name);
	ifaces = ifaces->next;
  }
  return meth;
}

METHODDEFPTR servicedef_getmethod_byHash(SERVICEDEFPTR servicedef, j_long hash)
{
  METHODDEFPTR meth = NULL;
  INTERFACEDEFPTR ifaces = servicedef->interfacesDefinition;
  while (ifaces != NULL && meth == NULL) {
	meth = interfacedef_getmethod_byHash(ifaces, hash);
	ifaces = ifaces->next;
  }
  return meth;
}

int servicedef_contains_method(SERVICEDEFPTR servicedef, METHODDEFPTR method)
{
	fprintf(stdout,"\n-servicedef_contains_method: %p", method);
	  int meth_found = 0;
	  METHODDEFPTR meth = NULL;
	  INTERFACEDEFPTR ifaces = servicedef->interfacesDefinition;
	  while (ifaces != NULL && !meth_found) {
		meth = ifaces->methods;
		while (meth != NULL && !meth_found) {
			fprintf(stdout,"\n-servicedef_contains_method(1): meth=%p method=%p", meth, method);  
		  if (methoddef_equal(meth, method))
			  return 1;
		  meth = meth->next;
		}
		ifaces = ifaces->next;
	  }
	  return meth_found;
}

int servicedef_print(SERVICEDEFPTR servicedef)
{ 
  fprintf(stdout,"\n- Service: %s (%s:%d)", servicedef->name, servicedef->host, servicedef->port);
  fprintf(stdout, "\n\t Service RMIObj:  objNum=%llu, &objId=%i, objTime=%llu, tmp_count=%d port=%d host=%s", servicedef->RMI_ObjNum, servicedef->RMI_ObjUnique, servicedef->RMI_ObjTime, servicedef->RMI_ObjCount, servicedef->RMI_ObjPort, servicedef->host);
  INTERFACEDEFPTR ifaces = servicedef->interfacesDefinition;
  while (ifaces != NULL) {
	interfacedef_print(ifaces);
	ifaces = ifaces->next;
  }
}


int servicedef_set_RMI_ObjNum(SERVICEDEFPTR servicedef, long long num){
    servicedef->RMI_ObjNum = num;
   fprintf(stdout, "\n\t Service setting RMIObj.objNum=%llu, ", servicedef->RMI_ObjNum);

}

int servicedef_set_RMI_ObjUnique(SERVICEDEFPTR servicedef, long unique){
 servicedef->RMI_ObjUnique = unique;
fprintf(stdout, "\n\t Service setting RMIObj.unique=%d, ", servicedef->RMI_ObjUnique);
  
}

int servicedef_set_RMI_ObjTime(SERVICEDEFPTR servicedef, long long time){
   servicedef->RMI_ObjTime = time;
fprintf(stdout, "\n\t Service setting RMIObj.objtime=%llu, time was=%llu ", servicedef->RMI_ObjTime, time);
}

int servicedef_set_RMI_ObjCount(SERVICEDEFPTR servicedef, short count){
servicedef->RMI_ObjCount = count;
fprintf(stdout, "\n\t Service setting RMIObj.objcount=%d, ", servicedef->RMI_ObjCount);
   
}

int servicedef_set_RMI_ObjPort(SERVICEDEFPTR servicedef, long port){
   servicedef->RMI_ObjPort = port;
fprintf(stdout, "\n\t Service setting RMIObj.objPort=%d, ", servicedef->RMI_ObjPort);

}





/*
 *	functions to handle interfacedef
 */
INTERFACEDEFPTR interfacedef_new2()
{
    INTERFACEDEFPTR iface = malloc(sizeof(INTERFACEDEF));
    iface->methods = NULL;
    iface->name = NULL;
    iface->uriPath = NULL;
    iface->XMLfile = NULL;
    iface->next = NULL;

    return iface;
}


INTERFACEDEFPTR interfacedef_new(SERVICEDEFPTR srv)
{
    INTERFACEDEFPTR iface = malloc(sizeof(INTERFACEDEF));
    iface->methods = NULL;
    iface->name = NULL;
    iface->uriPath = NULL;
    iface->XMLfile = NULL;
    iface->next = NULL;
    iface->service = srv;

    return iface;
}

int interfacedef_set_name(INTERFACEDEFPTR interfacedef, char* name)
{
    if (name != NULL) {
    	interfacedef->name = malloc(strlen(name)+1);
    	strcpy(interfacedef->name, name);
    } else
	interfacedef->name = NULL;

}

int interfacedef_set_namespace(INTERFACEDEFPTR interfacedef, char* namespace)
{
    if (namespace != NULL) {
    	interfacedef->namespace = malloc(strlen(namespace)+1);
    	strcpy(interfacedef->namespace, namespace);
    } else
	interfacedef->namespace = NULL;

}

int interfacedef_set_uripath(INTERFACEDEFPTR interfacedef, char* uripath)
{
    if (uripath != NULL) {
    	interfacedef->uriPath = malloc(strlen(uripath)+1);
    	strcpy(interfacedef->uriPath, uripath);
    } else
	interfacedef->uriPath = NULL;

}

int interfacedef_add_method(INTERFACEDEFPTR interfacedef, METHODDEFPTR meth)
{
    if (interfacedef->methods == NULL) {
	meth->next = NULL;
	interfacedef->methods = meth;
    } else {
	meth->next = interfacedef->methods;
	interfacedef->methods = meth;
    }
}


METHODDEFPTR interfacedef_getmethod_byname(INTERFACEDEFPTR interfacedef, char* name)
{
  METHODDEFPTR meth = NULL;
  METHODDEFPTR meths = interfacedef->methods;

  if (name != NULL)
	  while (meths != NULL && meth == NULL) {
		if (strcmp(meths->name, name) == 0)
		{
		    meth = malloc(sizeof(METHODDEF));
		    memcpy(meth, meths, sizeof(METHODDEF));
		    meth->next = NULL;
		}
		meths = meths->next;
	  }
  return meth;
}

METHODDEFPTR interfacedef_getmethod_byHash(INTERFACEDEFPTR interfacedef, j_long hash)
{
  METHODDEFPTR meth = NULL;
  METHODDEFPTR meths = interfacedef->methods;

  while (meths != NULL && meth == NULL) {
	if (meths->hash_index == hash)
	{
	    meth = malloc(sizeof(METHODDEF));
	    memcpy(meth, meths, sizeof(METHODDEF));
	    meth->next = NULL;
	}
	meths = meths->next;
  }
  return meth;
}


int interfacedef_print(INTERFACEDEFPTR interfacedef) 
{
  fprintf(stdout,"\n--- Interface: %s (URI=%s)",interfacedef->name, interfacedef->uriPath);
  METHODDEFPTR meths = interfacedef->methods;
  while (meths != NULL) {
	methoddef_print(meths);
	meths = meths->next;
  }
}



char* get_method_signature(ARGUMENTDEFPTR args, char* returnType)
{
	char* signature;
	char begin_signature[1000]; 
	char end_signature[1000]; 
	char type[100];
	ARGUMENTDEFPTR args_tmp;


	strcpy(begin_signature, "(");
	strcpy(end_signature, ")");

	for (args_tmp = args; args_tmp != NULL; args_tmp = args_tmp->next )
	{


		fprintf(stdout,"\nget_method_signature: args_tmp->datatype=%s",args_tmp->datatype);
		if (args_tmp->datatype == NULL)
			return NULL;
			
	    if (strcmp(args_tmp->datatype, "byte") == 0 || strcmp(args_tmp->datatype, "B")==0)
		strcpy(type, "B");
	    else if (strcmp(args_tmp->datatype, "char") == 0 || strcmp(args_tmp->datatype, "C")==0)
		strcpy(type, "C");
	    else  if (strcmp(args_tmp->datatype, "int") == 0 || strcmp(args_tmp->datatype, "I")==0)
		strcpy(type , "I");
	    else if (strcmp(args_tmp->datatype, "long") == 0 || strcmp(args_tmp->datatype, "J")==0)
		strcpy(type, "J");
	    else  if (strcmp(args_tmp->datatype, "short") == 0 || strcmp(args_tmp->datatype, "S")==0)
		strcpy(type, "S");
	    else {
		strcpy(type, "L");
		strcat(type ,args_tmp->datatype);
		strcat (type, ";");
	    }

  	    if (args_tmp->dir == ARG_IN)
		/*begin_signature = */strcat(begin_signature, type);
	    else 
		/*end_signature = */strcat(begin_signature, type);

	}
	if (strcmp(end_signature, ")") == 0)
		//strcat(end_signature, "V");
		strcat(end_signature, returnType);

	fprintf(stdout,"\nget_method_signature: begin_signature=%s",begin_signature);	
	fprintf(stdout,"\nget_method_signature: end_signature=%s",end_signature);	

	return strcat(begin_signature, end_signature);
}


/*
 *	functions to handle methoddef
 */
METHODDEFPTR methoddef_new2()
{
    METHODDEFPTR meth = malloc(sizeof(METHODDEF));
    meth->args = NULL;
    meth->name = NULL;
    meth->namespace = NULL;
    meth->next = NULL;
    meth->nArgs = 0;
    meth->hash_index = -1;
    return meth;
}

METHODDEFPTR methoddef_new(INTERFACEDEFPTR iface)
{
    METHODDEFPTR meth = malloc(sizeof(METHODDEF));
    meth->args = NULL;
    meth->name = NULL;
    meth->namespace = NULL;
    meth->next = NULL;
    meth->nArgs = 0;
    meth->interface = iface; 
    
    if (iface != NULL && iface->service != NULL)
    {
    	meth->hash_index = iface->service->methods_hash_index;
        iface->service->methods_hash_index++;
    }
    return meth;
}

int methoddef_set_name(METHODDEFPTR methoddef, char* name)
{
    if (name != NULL) {
    	methoddef->name = malloc(strlen(name)+1);
    	strcpy(methoddef->name, name);
    } else
	methoddef->name = NULL;
}

int methoddef_set_namespace(METHODDEFPTR methoddef, char* namespace)
{
    if (namespace != NULL) {
    	methoddef->namespace = malloc(strlen(namespace)+1);
    	strcpy(methoddef->namespace, namespace);
    } else
	methoddef->namespace = NULL;
}

int methoddef_add_argument(METHODDEFPTR methoddef, ARGUMENTDEFPTR arg)
{
	methoddef->nArgs++;
    if (methoddef->args == NULL) {
    	arg->next = NULL;
    	methoddef->args = arg;
    	arg->index = 0;
    } else {
    	arg->next = methoddef->args;
    	methoddef->args = arg;
    	arg->index = methoddef->nArgs - 1;
    }
}

int methoddef_equal(METHODDEFPTR methoddef1, METHODDEFPTR methoddef2)
{
	
	fprintf(stdout,"\n-methoddef_equal: methoddef1=%p methoddef2=%p", methoddef1, methoddef2);  
	if (strcmp(methoddef1->name, methoddef2->name))
		return 0;
	else if (methoddef1->nArgs != methoddef2->nArgs)
		return 0;
	else
	{
		fprintf(stdout,"\n-methoddef_equal(1): methoddef1\n");
		methoddef_print(methoddef1);
		fprintf(stdout,"\n-methoddef_equal(1): methoddef2\n");
		methoddef_print(methoddef2);
		
		fprintf(stdout,"\n-methoddef_equal(1): methoddef1=%p methoddef2=%p", methoddef1, methoddef2);  
		ARGUMENTDEFPTR arg1 = methoddef1->args;
		while (arg1 !=  NULL)
		{
			int arg1_equal_arg2 = 0;
			ARGUMENTDEFPTR arg2 = methoddef2->args;
			
			 
			
			while ((arg2 !=  NULL) && (!arg1_equal_arg2))
			{
				fprintf(stdout,"\n-methoddef_equal(2): arg1=%p arg2=%p", arg1->dir, arg2->dir); 
				
				if (/*(!strcmp(arg1->name,arg2->name)) &&*/ (arg1->dir==arg2->dir)/* && 
						(!strcmp(arg1->datatype,arg2->datatype))*/)
					arg1_equal_arg2 = 1;
				
				fprintf(stdout,"\n-methoddef_equal(3): arg1=%p arg2=%p", arg1->dir, arg2->dir); 
				
				arg2 = arg2->next;
			}
			arg1 = arg1->next;
			
			if (!arg1_equal_arg2)
				return 0;
		}
		fprintf(stdout,"\n-methoddef_equal(4): methoddef1=%p methoddef2=%p", methoddef1, methoddef2);
		return 1;
	}
}

int methoddef_print(METHODDEFPTR methoddef) 
{
	if (methoddef != NULL)
	{
	  fprintf(stdout,"\n----- Method (nargs=%d, hash_index=%d): %s (ns= %s)",methoddef->nArgs, methoddef->hash_index,methoddef->name, methoddef->namespace );
	  ARGUMENTDEFPTR args = methoddef->args;
	  while (args != NULL) {
		argumentdef_print(args);
		args = args->next;
	  }
	}
}

/*
char* methoddef_get_Java_signature(METHODDEFPTR methoddef) 
{
	char signature[255];
	strcat(signature, methoddef->name);
	strcat(signature, "(");
	ARGUMENTDEFPTR args = methoddef->args;
	while (args != NULL) {
		if (argumentdef->dir == ARG_IN) {
			if (strcmp(argumentdef->datatype, "boolean"))
			{
				strcat(signature, "");
			} 
			else if (strcmp(argumentdef->datatype, "int"))
			{
				strcat(signature, "");
			} 
			else if (strcmp(argumentdef->datatype, "string"))
			{
				strcat(signature, "");
			} 
		}
		args = args->next;
	}

	strcat(signature,")");
	strcat(signature, "V" );
	return signature;
}
*/

/*
 *	functions to handle argumentdef
 */
ARGUMENTDEFPTR argumentdef_new()
{
    ARGUMENTDEFPTR arg= (ARGUMENTDEFPTR)malloc(sizeof(ARGUMENTDEF));
    arg->name = NULL;
    arg->datatype = NULL;
    arg->dir = 0;
    arg->value = NULL;
    arg->next = NULL;

    return arg;
}

int argumentdef_set_name(ARGUMENTDEFPTR argumentdef, char* name)
{
    if (name != NULL) {
    	argumentdef->name = malloc(strlen(name)+1);
    	strcpy(argumentdef->name, name);
    } else
	argumentdef->name = NULL;
}

int argumentdef_set_dir(ARGUMENTDEFPTR argumentdef, int dir)
{
    argumentdef->dir = dir;
}

int argumentdef_set_datatype(ARGUMENTDEFPTR argumentdef, char* datatype)
{
    if (datatype != NULL) {
    	argumentdef->datatype = malloc(strlen(datatype)+1);
    	strcpy(argumentdef->datatype, datatype);
    } else
	argumentdef->datatype = NULL;
}

int argumentdef_set_value(ARGUMENTDEFPTR argumentdef, char* value)
{
    if (value != NULL) {
    	argumentdef->value = malloc(strlen(value)+1);
    	strcpy(argumentdef->value, value);
    } else
	argumentdef->value = NULL;
}

int argumentdef_set_index(ARGUMENTDEFPTR argumentdef, int index)
{
	argumentdef->index = index;
}

int argumentdef_print(ARGUMENTDEFPTR argumentdef) 
{
  fprintf(stdout,"\n------- Argument: %s", argumentdef->name);
  fprintf(stdout,"\n-------- dir: %s", (argumentdef->dir == 0)?"IN":"OUT");
  fprintf(stdout,"\n-------- datatype: %s", argumentdef->datatype);
  fprintf(stdout,"\n-------- value: %s", argumentdef->value);
  fprintf(stdout,"\n-------- index: %d",argumentdef->index);
}
