#include "parser/parser.h"
#include "parser/UPnPParser.h"
#include "unit/unit.h"

#include <stdio.h>
#include <pthread.h>
#include <string.h>
#include <ctype.h>

#include <libxml/xmlstring.h>
#include <libxml/parser.h>

#include "protocol/httpmsg.h"

void parse_URLBase_node(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser)
{
	printf("service node type: Element, name: %s a_node anode->content = %s\n", a_node->name, a_node->content);
	parser_publish_event(event_create_string(SDP_SERVICE_DESCR_URL_NOPATH, (char*)a_node->content, stream_id, (void*)src), parser);	
}

void parse_service_node(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser)
{
    xmlNode *cur_node = NULL;

    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("service node type: Element, name: %s = %s\n", cur_node->name, cur_node->content);
           
            if (!strcmp(cur_node->name, "SCPDURL"))
            {
            	printf("service node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("service node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("service node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	parser_publish_event(event_create_string(SDP_INTERFACE_DESCR_URL_PATH, (char*)cur_node->children->content, stream_id, (void*)src), parser);	
            }
            else if (!strcmp(cur_node->name, "controlURL"))
            	parser_publish_event(event_create_string(SDP_INTERFACE_CTRL_URL_PATH, (char*)cur_node->children->content, stream_id, (void*)src), parser);	
            else if (!strcmp(cur_node->name, "serviceType"))
            {
            	char* service_type;
            	
            	
            	printf("parse_service_node. URN_SERVICE_TYPE = %s, cur_node->children->content=%s, test=%s\n ",URN_SERVICE_TYPE, cur_node->children->content,strstr(cur_node->children->content, URN_SERVICE_TYPE) );
            	if (strstr(cur_node->children->content, URN_SERVICE_TYPE) != NULL)
            	{
            		char* str_urn_begin = strdup(cur_node->children->content);
            		char* str_urn_end = rindex(str_urn_begin, ':');
            		*str_urn_end = '\0';
            		int service_type_len = strlen(str_urn_begin) - strlen(URN_SERVICE_TYPE);
            		service_type = malloc(sizeof(char)*(service_type_len+1));
            		int i;
            		for (i=0; i < service_type_len; i++)
            			service_type[i] = str_urn_begin[strlen(URN_SERVICE_TYPE)+i];
            		service_type[i] = '\0';
            	}
            	else
            		service_type = strdup("interface_name");
            	parser_publish_event(event_create_string(SDP_SERVICE_DESCRIPTION_INTERFACETYPE, (char*)service_type, stream_id, (void*)src), parser);
            	parser_publish_event(event_create_string(SDP_SERVICE_DESCRIPTION_INTERFACE_NAMESPACE, (char*)cur_node->children->content, stream_id, (void*)src), parser);
            }
        }

        
    }
    //free(cur_node);
}

void print_element_names(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser)
{
    xmlNode *cur_node = NULL;

    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("node type: Element, name: %s\n", cur_node->name);
            
            if (!strcmp(cur_node->name, "service"))
            	parse_service_node(cur_node->children, stream_id, src, parser);
            else if (!strcmp(cur_node->name, "URLBase"))
            	parse_URLBase_node(cur_node->children, stream_id, src, parser);
            else if (!strcmp(cur_node->name, "deviceType"))
            {
            	char* device_type;
            	
            	
            	//printf("parse_service_node. URN_DEVICE_TYPE = %s, cur_node->children->content=%s, test=%s\n ",URN_SERVICE_TYPE, cur_node->children->content,strstr(cur_node->children->content, URN_SERVICE_TYPE) );
            	if (strstr(cur_node->children->content, URN_DEVICE_TYPE) != NULL)
            	{
            		char* str_urn_begin = cur_node->children->content;
            		char* str_urn_end = rindex(cur_node->children->content, ':');
            		*str_urn_end = '\0';
            		int device_type_len = strlen(str_urn_begin) - strlen(URN_DEVICE_TYPE);
            		device_type = malloc(sizeof(char)*(device_type_len+1));
            		int i;
            		for (i=0; i < device_type_len; i++)
            			device_type[i] = str_urn_begin[strlen(URN_DEVICE_TYPE)+i];
            		device_type[i] = '\0';
            	}
            	else
            		device_type = strdup("service_name");
           
            	parser_publish_event(event_create_string(SDP_SERVICE_DESCRIPTION_SERVICETYPE, (char*)device_type, stream_id, (void*)src), parser);	
            }
            else		
            	print_element_names(cur_node->children, stream_id, src, parser);
        }

        
    }
    
    //free(cur_node);
}



void upnp_devicedescr_processEvents(PARSER_PTR parser, HTTPMSG_PTR httpmsg, EVENTSRC_PTR evt_src )
{
	int evt_stream_id = (int)(evt_src->src_thread_parser);
	
	char* xml_device_descr;
	int xml_device_descr_size;
	httpmsg_get_content(httpmsg, &xml_device_descr, &xml_device_descr_size);
	
	xmlParserCtxtPtr ctxt_ptr = xmlNewParserCtxt();;
	xmlDocPtr xml_doc = xmlCtxtReadMemory(ctxt_ptr, xml_device_descr, xml_device_descr_size, NULL, NULL, 0);

	
	parser_publish_event(event_create(SDP_SERVICE_DESCRIPTION, evt_stream_id, (void*)evt_src), parser);
	parser_publish_event(event_create(SDP_UPNP, evt_stream_id, (void*)evt_src), parser);
	parser_publish_event(event_create(SDP_START, evt_stream_id, (void*)evt_src), parser);	
	
    xmlNodePtr root_element = NULL;
	
	
    root_element = xmlDocGetRootElement(xml_doc);

    print_element_names(root_element, evt_stream_id, (void*)evt_src, parser);

    parser_publish_event(event_create(SDP_STOP, evt_stream_id, (void*)evt_src), parser);
    
    /*free the document */
    xmlFreeDoc(xml_doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();
    
    //free(xml_device_descr);
}


void upnp_devicedescr_parse_msg_firstline(char** data, int length, HTTPMSG_PTR httpmsg)
{
	// READ FIRST LINE: IF REPLY IS CORRECT: HTTP/1.1 200 OK
	char* firstline = strtok_r (*data, "\r", data);
	char* token_version = strtok_r (firstline, " ", &firstline);
	char* token_code = strtok_r (NULL, " ", &firstline);
	char* token_ok = strtok_r (NULL, " ", &firstline);
	
	// to remove "\r" from the char stream "data" 
	(*data)++;
	
	fprintf(stdout,"\n firstline=%s,  token_version=%s token_code=%s token_ok =%s data=%s \n",firstline ,token_version,token_code,token_ok , *data);
	
	//ssdpmsg_set_method(ssdpmsg, "M-SEARCH");
	httpmsg_set_version(httpmsg, token_version);
	//ssdpmsg_set_uri(ssdpmsg, "*");
	httpmsg_set_code(httpmsg, token_code);
	
	//free(firstline);
	//free(token_version);
	//free(token_code);
	//free(token_ok);			
}

void upnp_devicedescr_parse_msg_headers(char** data, int length, HTTPMSG_PTR httpmsg)
{
	
	char* line;
	int i = 0;
	
	line = strtok_r (*data, "\r", data);
	if (line != NULL)
	{
		char* header_name = strtok_r (line, " ", &line);
		char* header_value = strtok_r (NULL, "\r", &line);
		
		fprintf(stdout,"\n tline=%s i=%d, header_name=%s header_value=%s \n", line, i++ ,header_name,header_value );
		// to remove "\r" from the line of the char stream "data" 
		(*data)++;
		
		// while read lines from the http header, add to the httpmsg header
		// when found "<", headers have finished and starts the http content
		while ( (**data != '<') && (line = strtok_r (NULL, "\r", data)) && (line != NULL))
		{
			
			char* header_name = strtok_r (line, ":", &line);
			
			if (header_name != NULL)
			{
				char* header_value = strtok_r (NULL, "\r", &line);
				
				 
				// remove spaces from the beginning of the header value
				while ((header_value != NULL) && isspace(*header_value))
						header_value++ ; 
				
				fprintf(stdout,"\n tline=%s i=%d,  header_name=%s header_value=%s \n", line, i++,header_name,header_value);
				
				// to remove "\r" from the line of the char stream "data" 
				while (**data == '\r' || **data == '\n')
					(*data)++;
				
				fprintf(stdout,"\n tline first char=%c \\r=%02x \n", **data, '\r');
				httpmsg_add_header(httpmsg, header_name, header_value);
			}
		}
		
		//free(header_name);
		//free(header_value);
	}

	
}



void upnp_devicedescr_parse_msg(PARSER_PTR parser, MSG_PTR message, EVENTSRC_PTR evt_src )
{
	char* msg = message->data;
	int len = message->length;

	fprintf(stdout, "upnp_devicedescr parse msg: ");
	int i;
//	for (i=0; i<len; i++)
//		fprintf(stdout,"%c",msg[i]);
//	fprintf(stdout, "\n");
	
	HTTPMSG_PTR httpmsg = httpmsg_create();
	
	char* message_start = message->data;
	upnp_devicedescr_parse_msg_firstline(&message->data, message->length, httpmsg);
	upnp_devicedescr_parse_msg_headers(&message->data, message->length, httpmsg);
	
	int length_header = message->data - message_start;
	int length_content = message->length - length_header ;
	fprintf(stdout, "upnp_devicedescr parse msg. length_header=%d length_content=%d",length_header,length_content);
	httpmsg_set_content(httpmsg, (message->data), length_content);
	
	upnp_devicedescr_processEvents(parser, httpmsg, evt_src );
	
	//httpmsg_free(httpmsg);
}

void *upnp_devicedescr_threaded_msg_parsing(void* ptr)
{
	PARSINGTHREADDATA_PTR data = (PARSINGTHREADDATA_PTR)ptr;
	MSG_PTR msg = (MSG_PTR)data->msg;
	PARSER_PTR parser = (PARSER_PTR)data->parser;
	
	// in case of the parsing of a reply message:
	//  - The events that are generated by this parser must not carry the identifier of this parser? thread  as their event src, but
	//    the thread identifier of the parser that received the request message and generated the related events. This identifier is stored in the
	//    unit eventqueue_table and the corresponding entry can be found using the dest port of the msg to be parsed (that corresponds to the 
	//	  port set in the table by "unit.c / set_socket_and_parser and set_socket_and_composer"
	EVENTQUEUE_TABLE_ENTRY_PTR entry = eventqueue_table_get_entry_by_current_socket_reply_port(parser->unit->event_table, msg->dest_port);
	
	if (entry == NULL)
		fprintf(stdout, "upnp_devicedescr_threaded_msg_parsing. entry not found for socket_reply_port=%d\n", msg->dest_port);
	
	else 
	{
		fprintf(stdout, "upnp_devicedescr_threaded_msg_parsing. parser=%p  parser->unit=%p msg=%p\n", parser, parser->unit,msg);
				
		//fprintf(stdout, "ssdp_threaded_msg_parsing. SOCKET port is %d. thread id is %p\n",msg->dest_port, entry->stream_id);
		
		// the sub_stream_id used for the events generated by the thread that parses this upnp_devicedescr reply msg 
		// is the value set by the ssdp parser 
		pthread_t thread_event_src = pthread_self();
		fprintf(stdout, "\nUPNP DEVICEDESCR PTHREAD parser = %p\n", thread_event_src);
		//entry->sub_stream_id = (int)thread_event_src;
		fprintf(stdout, "\nUPNP DEVICEDESCR PTHREAD parser entry->sub_stream_id = %p\n", entry->sub_stream_id);
		
		// the stream_id for the event is sub_stream_id to distinguish between the different event flows for reply messages 
		EVENTSRC_PTR evt_src = event_create_src((void*)entry->sub_stream_id, parser->unit);
	
		
		upnp_devicedescr_parse_msg(parser, msg, evt_src /*thread_event_src*/);
		
		//free(evt_src);
	}
	
	//msg_free(msg);
	//free(parser); NO!!! SEGFAULT if freed
	//free(ptr);

}

PARSER_PTR upnp_devicedescr_parser_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating UPNP DEVICEDESCR parser\n");
		
	PARSER_PTR p = parser_create(u);
	p->func_msg_received = parser_msg_received;
	
	p->func_parse = upnp_devicedescr_threaded_msg_parsing;
	
	p->func_publish_event = parser_publish_event;
	p->evt_listeners = NULL;
	
	p->factory = upnp_devicedescr_parser_create;
	
	return p;	
}

