#include "parser/parser.h"
#include "unit/unit.h"

#include <stdio.h>
#include <pthread.h>
#include <string.h>
#include <ctype.h>

#include <libxml/xmlstring.h>
#include <libxml/parser.h>

#include "protocol/httpmsg.h"
#include "event/servicedef.h"

typedef struct upnp_servicedescr_statevar {
	char* name;
	char* type;
	struct upnp_servicedescr_statevar* next;
} UPNP_SERVICEDESCR_STATEVAR;
typedef UPNP_SERVICEDESCR_STATEVAR* UPNP_SERVICEDESCR_STATEVAR_PTR;

UPNP_SERVICEDESCR_STATEVAR_PTR upnp_servicedescr_statevar_new()
{
	UPNP_SERVICEDESCR_STATEVAR_PTR var = malloc(sizeof(UPNP_SERVICEDESCR_STATEVAR));
    var->name = NULL;
    var->type = NULL;
    var->next = NULL;

    return var;
}

int upnp_servicedescr_statevar_set_name(UPNP_SERVICEDESCR_STATEVAR_PTR var, char* name)
{
    if (name != NULL) {
    	var->name = malloc(strlen(name)+1);
    	strcpy(var->name, name);
    } else
		var->name = NULL;
}

int upnp_servicedescr_statevar_set_type(UPNP_SERVICEDESCR_STATEVAR_PTR var, char* type)
{
    if (type != NULL) {
    	var->type = malloc(strlen(type)+1);
    	strcpy(var->type, type);
    } else
		var->type = NULL;
}

char* upnp_servicedescr_statevars_get_type(UPNP_SERVICEDESCR_STATEVAR_PTR vars, char* name)
{
	for (vars; vars != NULL; vars = vars->next)
	{
		if (!strcmp(vars->name, name))
			return vars->type;
	}
	return NULL;
}

void upnp_servicedescr_match_methodargs_with_statevars(METHODDEFPTR methods, UPNP_SERVICEDESCR_STATEVAR_PTR vars)
{
	for (methods; methods != NULL; methods = methods->next)
	{
		ARGUMENTDEFPTR args;
		for (args=methods->args; args!=NULL; args=args->next)
		{
			char* type = upnp_servicedescr_statevars_get_type(vars, args->name);
			if (type != NULL)
				args->datatype = strdup(type);
		}
	}
}

void upnp_servicedescr_parse_actionargument_node(xmlNode * a_node, METHODDEFPTR method)
{
    xmlNode *cur_node = NULL;
    ARGUMENTDEFPTR arg = argumentdef_new();
    
    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("actionargument node type: Element, name: %s = %s\n", cur_node->name, cur_node->content);
           
            if (!strcmp(cur_node->name, "name"))
            {
            	printf("actionargument node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("actionargument node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("actionargument node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	argumentdef_set_name(arg, cur_node->children->content);
            }
            else if (!strcmp(cur_node->name, "relatedStateVariable"))
            {
            	printf("actionargument node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("actionargument node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("actionargument node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	argumentdef_set_datatype(arg, cur_node->children->content);
            }
            else if (!strcmp(cur_node->name, "direction"))
            {
            	printf("actionargument node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("actionargument node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("actionargument node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	if (!strcmp(cur_node->children->content, "in"))
            		argumentdef_set_dir(arg, ARG_IN);
            	else if (!strcmp(cur_node->children->content, "out"))
            		argumentdef_set_dir(arg, ARG_OUT);
            }
        }
    }
    //free(cur_node);
    methoddef_add_argument(method, arg);
}


METHODDEFPTR upnp_servicedescr_parse_action_node(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser)
{
    xmlNode *cur_node = NULL;

    METHODDEFPTR method = methoddef_new2();
    
    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("action node type: Element, name: %s = %s\n", cur_node->name, cur_node->content);
           
            if (!strcmp(cur_node->name, "name"))
            {
            	printf("action node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("action node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("action node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	methoddef_set_name(method, cur_node->children->content);	
            }
            else if (!strcmp(cur_node->name, "argumentList"))
            {
            	xmlNode *arg_node = NULL;
            	
            	for (arg_node = cur_node->children; arg_node; arg_node = arg_node->next) {
                    if (arg_node->type == XML_ELEMENT_NODE) {
                        printf("argumentList node type: Element, name: %s = %s\n", arg_node->name, arg_node->content);
                       
                        if (!strcmp(arg_node->name, "argument"))
                        	upnp_servicedescr_parse_actionargument_node(arg_node->children, method);
                    }
            	}
            }
         }
    }
    //methoddef_print(method);
    //parser_publish_event(event_create_void_pointer(SDP_SERVICE_METHOD_DESCRIPTION, (void*)method, stream_id, (void*)src), parser);
    //free(cur_node);
    return method;
}

UPNP_SERVICEDESCR_STATEVAR_PTR upnp_servicedescr_parse_stateVariable_node(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser)
{
    xmlNode *cur_node = NULL;
    UPNP_SERVICEDESCR_STATEVAR_PTR state_var = upnp_servicedescr_statevar_new();

    
    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("stateVariable node type: Element, name: %s = %s\n", cur_node->name, cur_node->content);
            
            if (!strcmp(cur_node->name, "name"))
            {
            	printf("stateVariable node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("stateVariable node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("stateVariable node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	upnp_servicedescr_statevar_set_name(state_var, cur_node->children->content);
            }
            else if (!strcmp(cur_node->name, "dataType"))
            {
            	printf("stateVariable node type: Element, name: %s children = %p\n", cur_node->name,cur_node->children);
            	printf("stateVariable node type: Element, name: %s children->name = %s\n", cur_node->name,cur_node->children->name);
            	printf("stateVariable node type: Element, name: %s children->content = %s\n", cur_node->name,cur_node->children->content);
            	
            	upnp_servicedescr_statevar_set_type(state_var, cur_node->children->content);	
            }          	
        }
    }
    
    //free(cur_node);
    return state_var;
}

void upnp_servicedescr_print_element_names(xmlNode * a_node, int stream_id, void* src, PARSER_PTR parser, METHODDEFPTR* methods, UPNP_SERVICEDESCR_STATEVAR_PTR* state_vars)
{
    xmlNode *cur_node = NULL;


    for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if (cur_node->type == XML_ELEMENT_NODE) {
            printf("node type: Element, name: %s\n", cur_node->name);
            
            if (!strcmp(cur_node->name, "action"))
            {
            	METHODDEFPTR method = upnp_servicedescr_parse_action_node(cur_node->children, stream_id, src, parser);
            	if (method != NULL)
            	{
            		if (*methods == NULL)
            		{
            			*methods = method;
            			method->next = NULL;
            		}
            		else
            		{
            			method->next = *methods;
            			*methods = method;
            		}
            	}
            }
            else if (!strcmp(cur_node->name, "stateVariable"))
            {
            	UPNP_SERVICEDESCR_STATEVAR_PTR state_var = upnp_servicedescr_parse_stateVariable_node(cur_node->children, stream_id, src, parser);
            	if (state_var != NULL)
            	{
            		if (*state_vars == NULL)
            			*state_vars = state_var;
            		else
            		{
            			state_var->next = *state_vars;
            			*state_vars = state_var;
            		}
            	}
            }
            else
            	upnp_servicedescr_print_element_names(cur_node->children, stream_id, src, parser, methods, state_vars);
        }

        
    }
    
    //free(cur_node);
}



void upnp_servicedescr_processEvents(PARSER_PTR parser, HTTPMSG_PTR httpmsg, EVENTSRC_PTR evt_src )
{
	int evt_stream_id = (int)(evt_src->src_thread_parser);
	
	char* xml_device_descr;
	int xml_device_descr_size;
	httpmsg_get_content(httpmsg, &xml_device_descr, &xml_device_descr_size);
	
	xmlParserCtxtPtr ctxt_ptr = xmlNewParserCtxt();
	xmlDocPtr xml_doc = xmlCtxtReadMemory(ctxt_ptr, xml_device_descr, xml_device_descr_size, NULL, NULL, 0);

	
	parser_publish_event(event_create(SDP_SERVICE_DESCRIPTION, evt_stream_id, (void*)evt_src), parser);
	parser_publish_event(event_create(SDP_UPNP, evt_stream_id, (void*)evt_src), parser);
	parser_publish_event(event_create(SDP_START, evt_stream_id, (void*)evt_src), parser);	
	
    xmlNodePtr root_element = NULL;
	
	
    root_element = xmlDocGetRootElement(xml_doc);

    
    METHODDEFPTR methods = NULL;
    UPNP_SERVICEDESCR_STATEVAR_PTR state_vars = NULL;
    
    upnp_servicedescr_print_element_names(root_element, evt_stream_id, (void*)evt_src, parser, &methods, &state_vars);

    printf("upnp_servicedescr_print_element_names. PARSE FINISHED: METHODS\n");
    METHODDEFPTR meths;
    int index_METHS = 0;
    for (meths = methods; meths != NULL; meths = meths->next)
	{
    	index_METHS++;
    	methoddef_print(meths);
    	printf("\nupnp_servicedescr_print_element_names. index_METHS=%d",index_METHS);
	}
    upnp_servicedescr_match_methodargs_with_statevars(methods, state_vars);
    
    printf("upnp_servicedescr_print_element_names. PARSE FINISHED : METHODS AFTER MATCHING\n");
    
    for (meths = methods; meths != NULL; meths = meths->next)
	{
    	
	    EVENT_METHOD_DEF_PTR event_methoddef = malloc(sizeof(EVENT_METHOD_DEF));
	    
	    METHODDEFPTR meths2 = malloc(sizeof(METHODDEF));
	    memcpy(meths2, meths, sizeof(METHODDEF));
	    meths2->next = NULL;
	    event_methoddef->method = meths2;
	    event_methoddef->interface_name = strdup("interface_name");
	    event_methoddef->service_name = "service_name";
    	
	    
	    
	    parser_publish_event(event_create_void_pointer(SDP_SERVICE_METHOD_DESCRIPTION, (void*)event_methoddef, evt_stream_id, (void*)evt_src), parser);
	    printf("upnp_servicedescr_print_element_names. generate EVENT SDP_SERVICE_METHOD_DESCRIPTION\n");
	    //parser_publish_event(event_create_void_pointer(SDP_SERVICE_METHOD_DESCRIPTION, (void*)meths, evt_stream_id, (void*)evt_src), parser);
	    
    	methoddef_print(meths);
	}
    printf("upnp_servicedescr_print_element_names. FINISHED generate EVENT SDP_SERVICE_METHOD_DESCRIPTION\n");
    
    
    parser_publish_event(event_create(SDP_STOP, evt_stream_id, (void*)evt_src), parser);
    
    /*free the document */
    xmlFreeDoc(xml_doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();
    
    //free(xml_device_descr);
    
}


void upnp_servicedescr_parse_msg_firstline(char** data, int length, HTTPMSG_PTR httpmsg)
{
	// READ FIRST LINE: IF REPLY IS CORRECT: HTTP/1.1 200 OK
	char* firstline = strtok_r (*data, "\r", data);
	char* token_version = strtok_r (firstline, " ", &firstline);
	char* token_code = strtok_r (NULL, " ", &firstline);
	char* token_ok = strtok_r (NULL, " ", &firstline);
	
	// to remove "\r" from the char stream "data" 
	(*data)++;
	
	fprintf(stdout,"\n firstline=%s,  token_version=%s token_code=%s token_ok =%s data=%s \n",firstline ,token_version,token_code,token_ok , *data);
	
	//ssdpmsg_set_method(ssdpmsg, "M-SEARCH");
	httpmsg_set_version(httpmsg, token_version);
	//ssdpmsg_set_uri(ssdpmsg, "*");
	httpmsg_set_code(httpmsg, token_code);
	
	
	//free(firstline);
	//free(token_version);
	//free(token_code);
	//free(token_ok);	
}

void upnp_servicedescr_parse_msg_headers(char** data, int length, HTTPMSG_PTR httpmsg)
{
	
	char* line;
	int i = 0;
	
	line = strtok_r (*data, "\r", data);
	if (line != NULL)
	{
		char* header_name = strtok_r (line, " ", &line);
		char* header_value = strtok_r (NULL, "\r", &line);
		
		fprintf(stdout,"\n tline=%s i=%d, header_name=%s header_value=%s \n", line, i++ ,header_name,header_value );
		// to remove "\r" from the line of the char stream "data" 
		(*data)++;
		
		// while read lines from the http header, add to the httpmsg header
		// when found "<", headers have finished and starts the http content
		while ( (**data != '<') && (line = strtok_r (NULL, "\r", data)) && (line != NULL))
		{
			
			char* header_name = strtok_r (line, ":", &line);
			
			if (header_name != NULL)
			{
				char* header_value = strtok_r (NULL, "\r", &line);
				
				 
				// remove spaces from the beginning of the header value
				while ((header_value != NULL) && isspace(*header_value))
						header_value++ ; 
				
				fprintf(stdout,"\n tline=%s i=%d,  header_name=%s header_value=%s \n", line, i++,header_name,header_value);
				
				// to remove "\r" from the line of the char stream "data" 
				while (**data == '\r' || **data == '\n')
					(*data)++;
				
				fprintf(stdout,"\n tline first char=%c \\r=%02x \n", **data, '\r');
				httpmsg_add_header(httpmsg, header_name, header_value);
			}
		}
		//free(header_name);
		//free(header_value);
	}



}



void upnp_servicedescr_parse_msg(PARSER_PTR parser, MSG_PTR message, EVENTSRC_PTR evt_src )
{
	char* msg = message->data;
	int len = message->length;

	fprintf(stdout, "upnp_servicedescr parse msg: ");
	int i;
//	for (i=0; i<len; i++)
//		fprintf(stdout,"%c",msg[i]);
//	fprintf(stdout, "\n");
	
	HTTPMSG_PTR httpmsg = httpmsg_create();
	
	char* message_start = message->data;
	upnp_servicedescr_parse_msg_firstline(&message->data, message->length, httpmsg);
	upnp_servicedescr_parse_msg_headers(&message->data, message->length, httpmsg);
	
	int length_header = message->data - message_start;
	int length_content = message->length - length_header ;
	fprintf(stdout, "upnp_servicedescr parse msg. length_header=%d length_content=%d",length_header,length_content);
	httpmsg_set_content(httpmsg, (message->data), length_content);
	
	upnp_servicedescr_processEvents(parser, httpmsg, evt_src );
	
	//httpmsg_free(httpmsg);
}

void *upnp_servicedescr_threaded_msg_parsing(void* ptr)
{
	PARSINGTHREADDATA_PTR data = (PARSINGTHREADDATA_PTR)ptr;
	MSG_PTR msg = (MSG_PTR)data->msg;
	PARSER_PTR parser = (PARSER_PTR)data->parser;
	
	// in case of the parsing of a reply message:
	//  - The events that are generated by this parser must not carry the identifier of this parser? thread  as their event src, but
	//    the thread identifier of the parser that received the request message and generated the related events. This identifier is stored in the
	//    unit eventqueue_table and the corresponding entry can be found using the dest port of the msg to be parsed (that corresponds to the 
	//	  port set in the table by "unit.c / set_socket_and_parser and set_socket_and_composer"
	EVENTQUEUE_TABLE_ENTRY_PTR entry = eventqueue_table_get_entry_by_current_socket_reply_port(parser->unit->event_table, msg->dest_port);
	
	if (entry == NULL)
		fprintf(stdout, "upnp_servicedescr_threaded_msg_parsing. entry not found for socket_reply_port=%d\n", msg->dest_port);
	else 
	{
		fprintf(stdout, "upnp_servicedescr_threaded_msg_parsing. parser=%p  parser->unit=%p msg=%p\n", parser, parser->unit,msg);
				
		//fprintf(stdout, "ssdp_threaded_msg_parsing. SOCKET port is %d. thread id is %p\n",msg->dest_port, entry->stream_id);
		
		// the sub_stream_id used for the events generated by the thread that parses this upnp_servicedescr reply msg 
		// is the value set by the ssdp parser 
		pthread_t thread_event_src = pthread_self();
		fprintf(stdout, "\nUPNP SERVICEDESCR PTHREAD parser = %p\n", thread_event_src);
		//entry->sub_stream_id = (int)thread_event_src;
		fprintf(stdout, "\nUPNP SERVICEDESCR PTHREAD parser entry->sub_stream_id = %p\n", entry->sub_stream_id);
		
		// the stream_id for the event is sub_stream_id to distinguish between the different event flows for reply messages 
		EVENTSRC_PTR evt_src = event_create_src((void*)entry->sub_stream_id, parser->unit);
	
		
		upnp_servicedescr_parse_msg(parser, msg, evt_src /*thread_event_src*/);
		
		//free(evt_src);
	}
	
	//msg_free(msg);
	//free(parser);  NO!!! SEGFAULT if freed
	//free(ptr);

}

PARSER_PTR upnp_servicedescr_parser_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating UPNP SERVICEDESCR parser\n");
		
	PARSER_PTR p = parser_create(u);
	p->func_msg_received = parser_msg_received;
	
	p->func_parse = upnp_servicedescr_threaded_msg_parsing;
	
	p->func_publish_event = parser_publish_event;
	p->evt_listeners = NULL;
	
	p->factory = upnp_servicedescr_parser_create;
	
	return p;	
}

