#include <stdio.h>
#include "protocol/httpmsg.h"


HTTPMSG_PTR httpmsg_create()
{
	HTTPMSG_PTR msg = malloc(sizeof(HTTPMSG));
	
	msg->method = NULL;
	msg->version = NULL;
	msg->uri = NULL;
	msg->code = NULL;
	msg->headers = NULL;
	
	msg->content = NULL;
	msg->content_size = 0;
	
	return msg;
}





int httpmsg_set_method(HTTPMSG_PTR msg, char* method)
{
	msg->method = malloc(strlen(method)+1);
	strcpy(msg->method, method);
}

char* httpmsg_get_method(HTTPMSG_PTR msg)
{
	return msg->method;
} 		 

int httpmsg_set_version(HTTPMSG_PTR msg, char* version)
{
	msg->version = malloc(strlen(version)+1);
	strcpy(msg->version, version);
}

char* httpmsg_get_version(HTTPMSG_PTR msg)
{
	return msg->version;
} 	

int httpmsg_set_uri(HTTPMSG_PTR msg, char* uri)
{
	msg->uri = strdup(uri); 
}

char* httpmsg_get_uri(HTTPMSG_PTR msg)
{
	return msg->uri;
} 	

int httpmsg_set_code(HTTPMSG_PTR msg, char* code)
{
	msg->code = malloc(strlen(code)+1);
	strcpy(msg->code, code);
}

char* httpmsg_get_code(HTTPMSG_PTR msg)
{
	return msg->code;
} 	

int httpmsg_add_header(HTTPMSG_PTR msg, char* name, char* value)
{
	if (name != NULL)
	{
		HTTP_HEADER_PTR header = malloc(sizeof(HTTP_HEADER));
	
		header->name = strdup(name);
		if (value!= NULL)
			header->value = strdup(value);
		else
			header->value = NULL;
		header->next = NULL;
		
		if (msg->headers == NULL)
			msg->headers = header;
		else
		{
			header->next = msg->headers;
			msg->headers = header;
		}
	}
}

char* httpmsg_get_header(HTTPMSG_PTR msg, char* name)
{
	HTTP_HEADER_PTR header = NULL;
	for (header = msg->headers; header != NULL; header = header->next)
		if (!strcmp(header->name, name))
			return header->value;
	return NULL;
} 

HTTP_HEADER_PTR httpmsg_get_headers(HTTPMSG_PTR msg)
{
	return msg->headers;
} 

	



int httpmsg_set_src_addr(HTTPMSG_PTR msg, char* src_addr)
{
	msg->src_addr = malloc(strlen(src_addr)+1);
	strcpy(msg->src_addr, src_addr);
}

char* httpmsg_get_src_addr(HTTPMSG_PTR msg)
{
	return msg->src_addr;
}

int httpmsg_set_src_port(HTTPMSG_PTR msg, int src_port)
{
	msg->src_port = src_port;
}

int httpmsg_get_src_port(HTTPMSG_PTR msg)
{
	return msg->src_port;
}

int httpmsg_set_dest_addr(HTTPMSG_PTR msg, char* dest_addr)
{
	msg->dest_addr = malloc(strlen(dest_addr)+1);
	strcpy(msg->dest_addr, dest_addr);
}

char* httpmsg_get_dest_addr(HTTPMSG_PTR msg)
{
	return msg->dest_addr;
}

int httpmsg_set_dest_port(HTTPMSG_PTR msg, int dest_port)
{
	msg->dest_port = dest_port;
}

int httpmsg_get_dest_port(HTTPMSG_PTR msg)
{
	return msg->dest_port;
}


void httpmsg_get_content(HTTPMSG_PTR msg, char** content, int* content_size)
{
	*content_size = msg->content_size;
	*content = malloc(msg->content_size);
	int i;
	for (i=0; i<*content_size; i++)
		(*content)[i] = msg->content[i];
}

void httpmsg_set_content(HTTPMSG_PTR msg, char* content, int content_size)
{
	msg->content_size = content_size;
	msg->content = malloc(content_size);
	int i;
	for (i=0; i<content_size; i++)
		msg->content[i] = content[i];
}

void httpmsg_free(HTTPMSG_PTR msg)
{
	free(msg->method);
	free(msg->version);
	free(msg->uri);
	free(msg->code);
	free(msg->content);
	free(msg->src_addr);
	free(msg->dest_addr);

	HTTP_HEADER_PTR header = msg->headers; 
	
	while (header != NULL)
	{
		HTTP_HEADER_PTR header_next = header->next;
		free(header->name);
		free(header->value);
		free(header);
		header = header_next;
	}
}
