#include <vm/jclass.h>
#include <vm/opcode.h>
#include <vm/attributes.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <io/io.h>

ATTR_PTR parse_attr_charPtr(char** stream, POOL_PTR pool)
{
	int k;
	int i;


	ATTR_PTR attr = malloc(sizeof(ATTR));
	attr->pool = pool;


	j_short attr_name_index;
	parsejShort_charPtr(stream, &attr_name_index);
	attr->name_index = malloc(sizeof(uint16));
	memcpy(attr->name_index, &attr_name_index, sizeof(uint16));

	j_int attr_length;
	parsejInt_charPtr(stream, &attr_length);
			
	POOL_ENTRY_PTR* tab = pool->tab;

	char* type_attr = utf8ToString(tab[attr_name_index]->info.utf8->utf8);

	fprintf(stdout,"\tAttr) attr_name_index= %x, attr_type= %s, attr_length=%x\n", attr_name_index, type_attr, attr_length);

	if(!strcmp(type_attr, CODE))
	{

		attr->tag = CODE_ID;
		attr->info.code = malloc(sizeof(ATTR_CODE));
		attr->info.code->listCode=list_create(LISTCOUNT_T_MAX);
		attr->info.code->hashLabel=hash_create(HASHCOUNT_T_MAX,0,0);
		attr->info.code->listException=list_create(LISTCOUNT_T_MAX);
		attr->info.code->current_pc=0;
		attr->info.code->listAttributes = list_create(LISTCOUNT_T_MAX);


		j_short max_stack;
		parsejShort_charPtr(stream, &max_stack);

		j_short max_locals;
		parsejShort_charPtr(stream, &max_locals);
	
		j_int code_length;
		parsejInt_charPtr(stream, &code_length);

		
		for(i=0; i < code_length; i++)
		{
			j_byte code_byte;
			parsejByte_charPtr(stream, &code_byte);

			//fprintf(stdout,"%02x ", code_byte&0xff);
		}

		j_short exception_table_length;
		parsejShort_charPtr(stream, &exception_table_length);

		for(i=0; i < exception_table_length; i++)
		{
			j_short start_pc;
			parsejShort_charPtr(stream, &start_pc);

			j_short end_pc;
			parsejShort_charPtr(stream, &end_pc);		

			j_short handler_pc;
			parsejShort_charPtr(stream, &handler_pc);

			j_short catch_type;
			parsejShort_charPtr(stream, &catch_type);
		}

		j_short attributes_count;
		parsejShort_charPtr(stream, &attributes_count);

		fprintf(stdout,"\tAttr CODE) code_length=%x, exception_length=%x, attributes_count=%x\n",code_length,exception_table_length,attributes_count);

		for(i=0; i < attributes_count; i++)
		{
			ATTR_PTR attr_attr = parse_attr_charPtr(stream, pool);
			list_append(attr->info.code->listAttributes, lnode_create(attr_attr));
			
		}
		
	} 
	else if(!strcmp(type_attr, LOCAL_VARIABLE_TABLE))
	{
		attr->tag = LOCALVAR_ID;
		attr->info.local_var_table =  malloc(sizeof(ATTR_LOCAL_VAR_TABLE));
		attr->info.local_var_table->local_var_table = list_create(LISTCOUNT_T_MAX);

		j_short local_var_table_length;
		parsejShort_charPtr(stream, &local_var_table_length);

		attr->info.local_var_table->table_length = local_var_table_length;

		fprintf(stdout,"\tAttr CODE) local_var_table_length=%x\n", local_var_table_length);

		for(i=0; i < local_var_table_length; i++)
		{
			j_short start_pc;
			parsejShort_charPtr(stream, &start_pc);

			j_short length;
			parsejShort_charPtr(stream, &length);		

			j_short name_index;
			parsejShort_charPtr(stream, &name_index);

			j_short descriptor_index;
			parsejShort_charPtr(stream, &descriptor_index);

			j_short index;
			parsejShort_charPtr(stream, &index);

			fprintf(stdout,"\tLocal Var %i) name index=%x, descriptor index=%x\n",i,name_index, descriptor_index);

			LOCAL_VAR_ENTRY_PTR local_var = malloc(sizeof(LOCAL_VAR_ENTRY));
			local_var->pool = pool;
			local_var->start_pc = start_pc;
			local_var->name_index = malloc(sizeof(uint16));
			memcpy(local_var->name_index, &name_index, sizeof(uint16));
			local_var->descriptor_index = malloc(sizeof(uint16));
			memcpy(local_var->descriptor_index, &descriptor_index, sizeof(uint16));
			local_var->index = index;
	
			list_append(attr->info.local_var_table->local_var_table, lnode_create(local_var));
		}
	} 
	else if(!strcmp(type_attr, LINE_NUMBER_TABLE))
	{
		attr->tag = LINENUM_ID;
		attr->info.line_num_table =  malloc(sizeof(ATTR_LINE_NUMBER_TABLE));
		attr->info.line_num_table->line_number_table = list_create(LISTCOUNT_T_MAX);

		j_short line_num_table_length;
		parsejShort_charPtr(stream, &line_num_table_length);

		attr->info.line_num_table->table_length = line_num_table_length;

		for(i=0; i < line_num_table_length; i++)
		{
			j_short start_pc;
			parsejShort_charPtr(stream, &start_pc);

			j_short line_num;
			parsejShort_charPtr(stream, &line_num);		
		}
		fprintf(stdout,"\tAttr LINE NUMBER TABLE) length=%x, \n",line_num_table_length);

	} 
	else if(!strcmp(type_attr, EXCEPTIONS))
	{
		attr->tag = EXCEP_ID;
		attr->info.exception =  malloc(sizeof(ATTR_EXCEPTION));
		attr->info.exception->tab = NULL;

		j_short except_num;
		parsejShort_charPtr(stream, &except_num);

		attr->info.exception->nb = except_num;

		for(i=0; i < except_num; i++)
		{
			j_short except_index;
			parsejShort_charPtr(stream, &except_index);	
		}
		fprintf(stdout,"\tAttr EXCEPTIONS) number=%x, \n",except_num);

	}
	else
	{
		for(k=0; k < attr_length; k++)
		{
			j_byte attr_info;
			parsejByte_charPtr(stream, &attr_info);


			fprintf(stdout,"%02x ", attr_info&0xff);
		}
	}
	return attr;
}
/*
ATTR_PTR attributecode_add_attribute(ATTR_PTR attr_code, ATTR_PTR attr)
{

}
*/
