#include <netinet/in.h> /* for address structs */
#include <arpa/inet.h>  /* for sockaddr_in */
#include <stdio.h>      /* for printf() and fprintf() */
#include <stdlib.h>     /* for atoi() */
#include <string.h>     /* for strlen() */
#include <unistd.h>     /* for close() */


#include <netdb.h>


#define TRUE 1
#define FALSE 0

#define MAX_LEN  65536   /* maximum receive string size */
#define MIN_PORT 10   /* minimum port allowed */
#define MAX_PORT 65535  /* maximum port allowed */
#define MAX_HTTP_HEADER_SIZE 10000

#define HEADER_DELIMITER "\r\n\r\n"
#define HEADER_DELIMITER_LENGTH 4

#include "parser/parser.h"
#include "socket/socket.h"
#include "monitor/tools.h"


void* http_socket_send(void* ptr)
{
	SOCKETTHREADDATA_PTR data = (SOCKETTHREADDATA_PTR)ptr;
	MSG_PTR msg = data->msg;
	SOCKET_PTR this_socket = data->socket;	
	char* addr_str = msg->src_address;
	unsigned short port = msg->src_port;
	
	struct sockaddr_in addr,from_addr;
	int fd,bytes_read;
	int flag_on = 1;/* socket option flag */
	int index;
	unsigned int from_len; 
	char buffer[MAX_LEN],tempBuffer[MAX_LEN],*after_header;
 	int loop_index = 0;
	int iContentLength = 0, iBytesRemaining = 0;
		
	int bReceiptComplete = FALSE , bContentLengthRead = FALSE , bHeaderRead = FALSE , bAllBytesRead = FALSE;
		
	
	// msg->src_socket_id != -1 -> socket is already connected and send is on the already connected src_socket_id socket
	// msg->src_socket_id == -1 -> socket is not connected and send is on msg->dest_Addr and msg_>dest_port
	if (msg->src_socket_id != -1)
	{
		fprintf(stdout, "socket is already connected and send is on the already connected src_socket_id socket (%d)\n", msg->src_socket_id);
		//char w[255] = "HTTP/1.2 0 AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAfdfadfasdfd";
		//send(msg->src_socket_id , w, sizeof(w), 0);
		send(msg->src_socket_id, msg->data, msg->length, 0);
	}
	else
	{
		/* create what looks like an ordinary TCP socket */
		if ((fd=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0) 
		{
			perror("socket");
			exit(1);
		}
			
		/* set reuse port to on to allow multiple binds per host */
		if ((setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &flag_on,sizeof(flag_on))) < 0) 
		{
			perror("setsockopt() failed");
			exit(1);
		}	
		
		// get socket local port
		int localport = socket_get_localport(this_socket); //5230;
		fprintf(stdout, "\nlocalport =  %d\n", localport);
	  // bind socket to local port 
		struct sockaddr_in local_addr;
		memset(&local_addr, 0, sizeof(local_addr));
		local_addr.sin_family      = AF_INET;
		local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
		local_addr.sin_port        = htons(localport);
		if ((bind(fd, (struct sockaddr *) &local_addr,sizeof(local_addr))) < 0) 
		{
			fprintf(stdout, "httpsocket_send: binding to local port %d\n", localport);
	    perror("httpsocket_send: bind to local addr failed");
	    exit(1);
	  } 
		else 
		{
	  	fprintf(stdout, "httpsocket_send: bound to local port %d\n", localport);
	  }	
		
		/* set up destination address */
		memset(&addr,0,sizeof(addr));
		//addr.sin_family = AF_INET;
		//addr.sin_addr.s_addr = inet_addr(msg->dest_address);
		int return_pton = inet_pton(AF_INET, msg->dest_address, &addr.sin_addr);
		addr.sin_port = htons(msg->dest_port); 		
		addr.sin_family = AF_INET;
		
		fprintf(stdout, "httpsocket_send (return_pton=%d) to %s : %d the message:\n",return_pton, msg->dest_address, msg->dest_port);
		fprintf(stdout, "MESSAGE_LENGTH = %d\n",msg->length);
	
		if (return_pton == 0) /* not dotted value */
		{
			struct hostent * h;
			h = (struct hostent *)gethostbyname(msg->dest_address);
		
			//printf("\nh_name=%s \nh_addrtype=%d \nh_length=%d \n", h->h_name, h->h_addrtype, h->h_length);
		
			struct in_addr* addr0 = (struct in_addr*)h->h_addr;
			char* addr_str = inet_ntoa(*addr0);	
			printf("\naddr_str=%s \n",addr_str);
			
			inet_pton(AF_INET, addr_str, &addr.sin_addr);
		}
		
		if(connect(fd, (struct sockaddr*)&addr, sizeof(addr)) < 0) 
		{ 
			perror("connect");
			//exit(1);
		}
		else
		{
			
			for (index=0; index < msg->length; index++)
					fprintf(stdout, "%c",msg->data[index]&0xff,msg->data[index]&0xff);
					
			fprintf(stdout, "\n\n",msg->length);
			     
			if ((index = send(fd, msg->data, msg->length, 0)) < 0) 
			{
				perror("sendto");
				//exit(1);
			}	
			else
			{
				fprintf(stdout, "MESSAGE SENT: length=%d \n",index);	
			
				fprintf(stdout,"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");			
				do
				{
				    from_len = sizeof(from_addr);
				    memset(&from_addr, 0, from_len);
				    
					bzero(buffer, sizeof(buffer));
					from_len = 0;
					bytes_read = recvfrom(fd, buffer, sizeof(buffer),0,(struct sockaddr*)&from_addr, &from_len);
					
					
					
					buffer[bytes_read] = '\0';
					fprintf(stdout,"=========================== NEXT CHUNK ============================\n%s\n",buffer);
				  
					if(loop_index == 0)
					{			
						strcpy(tempBuffer,buffer);
					}
					else
					{
						strcat(tempBuffer,buffer);		
					}
					loop_index++;	
					fprintf(stdout,"=========================== CONCATENATED CHUNK ============================\n%s\n",tempBuffer);
					
					if(bHeaderRead == FALSE)
					{
						if (tempBuffer != NULL)
						{
							//char *temp = strdup(tempBuffer);
							//if(temp != NULL)
							//{
								after_header = strstr(/*temp*/tempBuffer,HEADER_DELIMITER);					
							//}				
						}
						fprintf(stdout,"=========================== after header ============================\n %s\n",after_header);			
						
						if( after_header != NULL)
						{
							fprintf(stdout,"=========================== HEADER FOUND ============================\n");			
							fprintf(stdout,"=========================== after header ============================\n%s\n",after_header);																
							bHeaderRead = TRUE;	
							
							iContentLength = atoi(ExtractHeaderField(tempBuffer, STR_CONTENTLENGTH));
							if(iContentLength > 0)
							{				
								bContentLengthRead = TRUE;
								//fprintf(stdout,"TOTAL_BUFFER_LENGTH = %d AFTER_HEADER_BUFFER_LENGTH = %d and CONTENT_LENGTH =  %d\n",strlen(strdup(tempBuffer)),strlen(strdup(after_header)),iContentLength);
								fprintf(stdout,"TOTAL_BUFFER_LENGTH = %d AFTER_HEADER_BUFFER_LENGTH = %d and CONTENT_LENGTH =  %d\n",strlen(tempBuffer),strlen(after_header),iContentLength);
								iBytesRemaining = iContentLength;
								iBytesRemaining = (iBytesRemaining) - (strlen(after_header) - HEADER_DELIMITER_LENGTH);											
								fprintf(stdout,"BYTES REMAINING = %d\n",iBytesRemaining);															
							}
						}
					}
					else
					{
						iBytesRemaining = (iBytesRemaining) - (bytes_read);														
						fprintf(stdout,"BYTES REMAINING = %d\n",iBytesRemaining);													
					}				
					
					if(iBytesRemaining <= 0 && bHeaderRead == TRUE)
					{
						bAllBytesRead = TRUE;
						fprintf(stdout,"=========================== ALL BYTES READ ============================\n");						
					}
					
					if( (bContentLengthRead == TRUE) && (bHeaderRead == TRUE) && (bAllBytesRead == TRUE) )
					{
						bReceiptComplete = TRUE;
					}
				}
				while(bReceiptComplete == FALSE);
				fprintf(stdout,"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");			
				
			  close(fd);  
				 
			  if ((&from_addr != NULL) && from_len >= 0  )
			  {
				printf("\nHTTP: Received %d bytes from %s: unsigned=%u int=%d", 
				strlen(tempBuffer),
				inet_ntoa(from_addr.sin_addr), 
				ntohs(from_addr.sin_port), 
				ntohs(from_addr.sin_port)
				);
				
				msg = msg_create(tempBuffer,strlen(tempBuffer),inet_ntoa(from_addr.sin_addr),ntohs(from_addr.sin_port),
						addr_str, localport,this_socket);	
									
				socket_publish_msg(msg,this_socket);	
			  }
			  else
			  {
					msg = msg_create(tempBuffer,strlen(tempBuffer),"0.0.0.0",0 ,
							addr_str, localport,this_socket);	
										
					socket_publish_msg(msg,this_socket);	
			  }
			}
		}
	}
	
}

void* http_socket_listen(void* ptr /*SOCKET_PTR this_socket, char* mc_addr_str, unsigned short mc_port */) 
{

	SOCKETLISTENTHREADDATA_PTR data = (SOCKETLISTENTHREADDATA_PTR)ptr;
	SOCKET_PTR this_socket = data->socket;
  
	char* addr_str = data->addr;
	unsigned short port = data->port;
   
	/*
	SOCKETTHREADDATA_PTR data = (SOCKETTHREADDATA_PTR)ptr;
	MSG_PTR msg = data->msg;
	SOCKET_PTR this_socket = data->socket;	
	char* addr_str =  msg->src_address;
	unsigned short port = msg->src_port;
	*/
	
	struct sockaddr_in addr,from_addr;
	int fd,recvSock,bytes_read;
	int flag_on = 1;/* socket option flag */
	int index;
	unsigned int from_len; 
	char buffer[MAX_LEN],tempBuffer[MAX_LEN],*after_header;
 	int loop_index = 0;
	int iContentLength = 0, iBytesRemaining = 0;
		
	int bReceiptComplete = FALSE , bContentLengthRead = FALSE , bHeaderRead = FALSE , bAllBytesRead = FALSE;

	/* create what looks like an ordinary TCP socket */
	if ((fd=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0)
	{
		perror("socket");
		exit(1);
	}
	
	/* set reuse port to on to allow multiple binds per host */
	if ((setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &flag_on,sizeof(flag_on))) < 0)
	{
		perror("setsockopt() failed");
		exit(1);
	}

	// get socket local port
	//int localport = socket_get_localport(this_socket); //5230;

	// bind socket to local port
	struct sockaddr_in local_addr;
	memset(&local_addr, 0, sizeof(local_addr));
	local_addr.sin_family      = AF_INET;
	local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	local_addr.sin_port        = htons(port);
	if ((bind(fd, (struct sockaddr *) &local_addr,sizeof(local_addr))) < 0)
	{
		fprintf(stdout, "httpsocket_listen: binding to local port %d\n", port);
		perror("httpsocket_listen: bind to local addr failed");
		exit(1);
	}
	else
	{
		fprintf(stdout, "httpsocket_listen: bound to local port %d\n", port);
	}
				
	if(listen(fd,5) != 0)
	{
		perror("listen() failed");
		exit(1);	
	}
				

	for (;;) 
	{/* loop forever */
		
		fprintf(stdout, "HTTP is waiting on address= %s, port= %d.\n",addr_str, port);
		
	    from_len = sizeof(from_addr);
	    memset(&from_addr, 0, from_len);
		
		if((recvSock = accept(fd,(struct sockaddr *) &from_addr, &from_len)) < 0)
		{
			perror("accept() failed");
			exit(1);	
		}		
		
		
		printf("\nHTTP: Received  (from_len=%d) bytes from %s: unsigned=%u int=%d", 
				from_len, inet_ntoa(from_addr.sin_addr), 
				ntohs(from_addr.sin_port), 
				ntohs(from_addr.sin_port)
				);
		
		
		fprintf(stdout,"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");			
		do
		{
			bzero(buffer, sizeof(buffer));	
			bytes_read = recvfrom(recvSock, buffer, sizeof(buffer),0,(struct sockaddr*)&from_addr, &from_len);		
			fprintf(stdout,"=========================== NEXT CHUNK ====== (%s) ======================\n%s\n",inet_ntoa(from_addr.sin_addr), buffer);
			
			if(loop_index == 0)
			{			
				strcpy(tempBuffer,buffer);
			}
			else
			{
				strcat(tempBuffer,buffer);		
			}
			loop_index++;	
			fprintf(stdout,"=========================== CONCATENATED CHUNK ============================\n%s\n",tempBuffer);
			
			if(bHeaderRead == FALSE)
			{
				after_header = strstr(tempBuffer,HEADER_DELIMITER);
				if( after_header != NULL)
				{
					fprintf(stdout,"=========================== HEADER FOUND ============================\n");			
					fprintf(stdout,"=========================== after header ============================\n%s\n",after_header);																
					bHeaderRead = TRUE;	
					
					iContentLength = atoi(ExtractHeaderField(tempBuffer,STR_CONTENTLENGTH ));
					if(iContentLength > 0)
					{				
						bContentLengthRead = TRUE;
						fprintf(stdout,"TOTAL_BUFFER_LENGTH = %d AFTER_HEADER_BUFFER_LENGTH = %d and CONTENT_LENGTH =  %d\n",strlen(tempBuffer),strlen(after_header),iContentLength);
						iBytesRemaining = iContentLength;
						iBytesRemaining = (iBytesRemaining) - (strlen(after_header) - HEADER_DELIMITER_LENGTH);											
						fprintf(stdout,"BYTES REMAINING = %d\n",iBytesRemaining);															
					}
				}
			}
			else
			{
				iBytesRemaining = (iBytesRemaining) - (bytes_read);														
				fprintf(stdout,"BYTES REMAINING = %d\n",iBytesRemaining);													
			}				
			
			if(iBytesRemaining <= 0 )
			{
				bAllBytesRead = TRUE;
				fprintf(stdout,"=========================== ALL BYTES READ ============================\n");						
			}
			
			if( (bContentLengthRead == TRUE) && (bHeaderRead == TRUE) && (bAllBytesRead == TRUE) )
			{
				bReceiptComplete = TRUE;
			}
		}
		while(bReceiptComplete == FALSE);
		fprintf(stdout,"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");			
			
		//close(recvSock);

		

		printf("\nHTTP: Received %d bytes from socket=%d adrr=%s: unsigned=%u int=%d", 
				
		strlen(buffer),
		recvSock,
		inet_ntoa(from_addr.sin_addr), 
		ntohs(from_addr.sin_port), 
		ntohs(from_addr.sin_port)
		);
		
		//MSG_PTR msg = msg_create(tempBuffer,strlen(tempBuffer),inet_ntoa(from_addr.sin_addr),ntohs(from_addr.sin_port),addr_str,port,this_socket);	
		MSG_PTR msg = msg_create_sock_id(tempBuffer,strlen(tempBuffer),inet_ntoa(from_addr.sin_addr),ntohs(from_addr.sin_port),addr_str,port,
				recvSock, this_socket);	
		
		socket_publish_msg(msg,this_socket);	
		//break;
	}
}

SOCKET_PTR http_socket_create(UNIT_PTR u)
{
	fprintf(stdout, "Creating http socket\n");
	
	SOCKET_PTR s = (SOCKET_PTR)malloc(sizeof(SOCKET));
	
	s->func_msg_received = socket_msg_received;
	s->func_publish_msg = socket_publish_msg;
	s->msg_listeners = NULL;
	s->unit = u;		
	s->func_send = http_socket_send;
	s->func_listen = http_socket_listen;	
	s->factory = http_socket_create;
	
	return s;	
}



